!========================================================================
! Copyright (C) VMware, Inc. 1986-2011.  All Rights Reserved.
!
! $Id: system.gs,v 1.40 2008-04-08 22:57:46 bille Exp $
!
! Superclass Hierarchy:
!   System, Object.
!
!=========================================================================

removeallmethods System
removeallclassmethods System

category: 'For Documentation Installation only'
classmethod: System
installDocumentation

| doc txt |
doc := GsClassDocumentation newForClass: self.

txt := (GsDocText new) details:
'System is an abstract class that has no instances.  It implements class
 methods for object locking and for operations that are usually found in
 traditional operating systems.  The data curator may restrict user access to
 these messages.  For an explanation of the role of the data curator, refer to
 your GemStone System Administration Guide.'.
doc documentClassWith: txt.

txt := (GsDocText new) details:
'Methods in this category are obsolete and are provided only for compatibility
 with earlier releases of GemStone.  They will be removed in a future release.'.
doc documentClassCategory: #'Backward Compatibility' with: txt.

txt := (GsDocText new) details:
'These methods are used in managing cluster buckets, the streams of disk pages
 in which objects are congregated during clustering.  Clustering is explained
 in the GemStone Programming Guide.'.
doc documentClassCategory: #Clustering with: txt.

txt := (GsDocText new) details:
'In each of these file system access methods, it is best to specify the full
 path name of the server text file in the method''s argument.

 Under Unix, be sure that the case of the argument matches the case of the Unix
 file name; Unix is case-sensitive.

 Also note that under Unix, each of these methods inherits environment
 variables from the GemStone session process, rather than from your user
 session.  In addition, the method performOnServer: invokes the Bourne shell,
 even if you use a different login shell.  For these reasons, you might want to
 avoid using environment variables in the arguments to these methods.'.
doc documentClassCategory: #'Host System Access' with: txt.

txt := (GsDocText new) details:
'GemStone maintains two sets of objects that you can manipulate with methods in
 this category.  The commit release locks set contains locked objects whose
 locks will be released as part of the next successful commit operation.  The
 commit-or-abort release locks set contains locked objects whose locks will be
 released as part of the next successful commit operation or abort operation.
 To gain complete control over the automatic releasing of locks at the end of a
 transaction, use these methods during the transaction to govern the membership
 of objects in these sets.'.
doc documentClassCategory: #'Releasing Locks' with: txt.

txt := (GsDocText new) details:
'Transactions are discussed in detail in the GemStone Programming Guide.'.
doc documentClassCategory: #'Transaction Control' with: txt.

self description: doc.
%

!==========================================================================
! Below this point: only method definitions in fileout order
!==========================================================================


! ------------------- Class methods for System
category: 'Transaction Control'
classmethod: System
abortTransaction

"Rolls back all modifications made to committed GemStone objects and provides
 the session with a new view of the most recently committed GemStone state.

 These operations are performed whether or not the session was previously in a
 transaction.  If the transaction mode is set to #autoBegin, then a new
 transaction is started.  If the transaction mode is set to #manualBegin, then
 a new transaction is not started."

| coordinator |

^ (coordinator := self _commitCoordinator) == nil
  ifTrue: [ self _localAbort ]
  ifFalse: [ coordinator abort ].
%
category: 'System Control'
classmethod: System
activeRepositories

"Returns an Array containing references to the repositories that are
 attached at the time the message is sent."

^ self _zeroArgCateg2Prim: 0
%
category: 'Releasing Locks'
classmethod: System
addAllToCommitOrAbortReleaseLocksSet: aCollection

"Add each element of aCollection to the commit-or-abort release locks set. 
 If an element of aCollection is not locked by the current session, then
 it is not added to the set."

aCollection do: [:each | self addToCommitOrAbortReleaseLocksSet: each ].
^self
%
category: 'Releasing Locks'
classmethod: System
addAllToCommitReleaseLocksSet: aCollection

"Add each element of aCollection to the commit release locks set.  If an
 element of aCollection is not locked by the current session, then that
 element is not added to the set."

aCollection do: [:each | self addToCommitReleaseLocksSet: each ].
^self
%
category: 'Notification'
classmethod: System
addAllToNotifySet: aCollection

"Add all the elements of aCollection to the notify set.  Special objects and
 uncommitted objects are not permitted, since neither of these can be modified
 by other sessions."

self _updateNotifySet: aCollection add: true
%
category: 'System Control'
classmethod: System
addAllToStoneLog: aString

"Appends text to the Stone's informational log file.  First, this method writes
 a banner that identifies the session from which aString came.  It then appends
 aString itself.  The argument must be a kind of String or DoubleByteString."

| logicalSize chunkSize offset endOffset |

aString basicSize <= 8000 "virtual machine constant" ifTrue:[
  ^ self _addAllToStoneLog: aString 
  ].
logicalSize := aString size .
chunkSize := 8000 "virtual machine constant" .
(aString isKindOf: DoubleByteString) ifTrue:[ chunkSize := 4000 ].
 
offset := 1.
[offset <= logicalSize] whileTrue:[
  endOffset := offset + chunkSize - 1 .
  endOffset > logicalSize ifTrue:[ endOffset := logicalSize ].
  self _addAllToStoneLog: (aString copyFrom: offset to: endOffset).
  offset := offset + chunkSize
  ].
%
category: 'Releasing Locks'
classmethod: System
addToCommitOrAbortReleaseLocksSet: anObject

"Add anObject to to the commit-or-abort release locks set.  If anObject
 is not locked by the current session, then it is not added to the set."

((self myLockKind: anObject) == #none )
  ifFalse: [self _add: anObject to: 30.].
^self
%
category: 'Releasing Locks'
classmethod: System
addToCommitReleaseLocksSet: anObject

"Add anObject to the commit release locks set.  If anObject is not locked 
 by the current session, then it is not added to the set."

((self myLockKind: anObject) == #none )
  ifFalse: [self _add: anObject to: 29.].
^self
%
category: 'Notification'
classmethod: System
addToNotifySet: anObject

"Add anObject to the notify set.  The argument anObject cannot be a special
 object nor an uncommitted object, since neither can be modified by other
 sessions."

self _updateNotifySet: #[ anObject ]  add: true
%
category: 'Transaction Control'
classmethod: System
beginTransaction

"Starts a new transaction for the session.  An abort is done before the new
 transaction is started - giving the session a new snapshot of the repository.

 If any permanent objects had been written by the session, their
 state is aborted.  This method returns the receiver (System)."

| coordinator |
^ (coordinator := self _commitCoordinator) == nil
  ifTrue: [ self _localBeginTransaction ]
  ifFalse: [ coordinator beginTransaction ].
%
category: 'Performance Monitoring'
classmethod: System
cacheSlotForSessionId: aSessionId

"Return the cache process slot number (a SmallInteger) for the given session
 ID.  The session must be connected to the same shared page cache as the 
 session invoking this method.  

 A return of nil indicates the session could not be located."

<primitive: 539>
aSessionId _validateClass: SmallInteger.
self _error: #rtErrArgOutOfRange args: #[ aSessionId ] .
^ self _primitiveFailed: #cacheSlotForSessionId:
%
category: 'Performance Monitoring'
classmethod: System
cacheStatistics: aProcessSlot

"Returns an Array whose contents are described by the result of the
 cacheStatisticsDescription method.  The Array contains statistics for the
 specified slot in the GemStone shared memory cache to which this session is
 attached.

 The argument aProcessSlot should be a SmallInteger between 0 and the number of
 process slots in the shared cache minus 1, inclusive.  If aProcessSlot is 
 outside the range of valid process slots, or the session executing this
 method is not using a shared cache, generate an error.  If the slot specified
 by aProcessSlot is an inactive slot, returns nil.  The method
 cacheSlotForSessionId: may be used to determine the process slot of a session
 on the same shared cache.
 
 The process slots that are predefined are:

    slot 0: The shared page cache monitor.

    slot 1: The Stone if the cache is on the same machine as the Stone.
            Otherwise, a page server that is used to monitor the cache for
            the Stone.

 No other slots are guaranteed.  However, slot 2 is the often the page server
 and slot 3 is often the GcGem.  These depend to some extent on the relative
 speed of the processes during startup.  In addition, the GcGem can be 
 shut down, and when it is restarted, it is unlikely to end up at the same
 position."

 ^self _cacheStatsForSlotOrSession: aProcessSlot
%
category: 'Performance Monitoring'
classmethod: System
cacheStatisticsDescription

"Returns an Array of Strings describing the result of the method
 cacheStatistics."

^ #(
" 1" 'ProcessName'                "Process identification, aString "
" 2" 'ProcessId'                  "This and all subsequent items are Integers"
" 3" 'SessionId'                  
" 4" 'LockedPage'                 "obsolete"
" 5" 'AttachDelta'
" 6" 'AttachedCount'              "PageCache Statistics"
" 7" 'SharedAttached'             "Attached by more than one process, only in
                                   monitor process slot"
" 8" 'TotalAttached'              "only in monitor process slot"
" 9" 'FreeFrameCount'             "only in monitor process slot"
"10" 'LocalPageCacheHits'         "total hits on shared plus private caches"
"11" 'LocalPageCacheMisses'       "total misses on shared plus private caches"
"12" 'LocalPageCacheWrites'       "total writes from shared plus private caches"
"13" 'PageReads'                  "Virtual Disk I/O (real disk I/O if extents "
"14" 'PageWrites'                 " ... on local disk)."
"15" 'CommitCount'                "Transaction Statistics"
"16" 'FailedCommitCount'
"17" 'AbortCount'
"18" 'ObjsCommitted'
"19" 'NewObjsCommitted'
"20" 'TotalObjsCommitted'         "obsolete"
"21" 'TotalNewObjsCommitted'      "obsolete"
"22" 'ScavengeCount'              "GarbageCollection Statistics"
"23" 'TimeInScavenges'
"24" 'DeadObjCount'
"25" 'MakeRoomInOldSpaceCount'
"26" 'GcNotConnectedCount'
"27" 'GcNotConnectedDeadCount'
"28" 'IntSendCount'               "obsolete"
"29" 'ClassCacheCount'            "obsolete"
"30" 'MethodCacheCount'           "obsolete"
"31" 'ExportedSetSize'
"32" 'NoRollbackSetSize'
"33" 'NotConnectedObjsSetSize'
"34" 'TotalCommits'               "only in Stone process slot"
"35" 'CommitRecordCount'          "only in Stone process slot"
"36" 'AsyncWritesInProgress'      "only in Stone process slot"
"37" 'AsyncWritesCount'           "only in Stone process slot"
"38" 'LogRecordsWritten'          "only in Stone process slot"
"39" 'LogRecordsIoCount'          "only in Stone process slot"
"40" 'DeadObjsCount'              "only in Stone process slot"
"41" 'ReclaimCount'               "only in Stone process slot"
"42" 'ReclaimedPagesCount'        "only in Stone process slot"
"43" 'CheckpointCount'            "only in Stone process slot"
"44" 'unused'                     "unused"
"45" 'AioDirtyCount'              "only in page server process slot"
"46" 'AioCkptCount'               "only in page server process slot"
"47" 'LocalDirtyPageCount'        "only in shared page cache monitor slot"
"48" 'GlobalDirtyPageCount'       "only in shared page cache monitor slot"
"49" 'PagesNeedReclaimSize'       "only in Stone process slot"
"50" 'PossibleDeadSize'           "only in Stone process slot"
"51" 'DeadNotReclaimedSize'       "only in Stone process slot"
"52" 'InTransaction'              "only in Gem process slot"
"53" 'TimeWaitingForCommit'       "only in Gem process slot"
"54" 'TimeProcessingCommit'       "only in Gem process slot"
"55" 'TimeStoneCommit'            "only in Gem process slot"
"56" 'CommitQueueSize'            "only in Stone process slot"
"57" 'LockReqQueueSize'           "only in Stone process slot"
"58" 'NotifyQueueSize'            "only in Stone process slot"
"59" 'LoginWaitQueueSize'         "only in Stone process slot"
"60" 'RunQueueSize'               "only in Stone process slot"
"61" 'PageKindsWrittenByGems'     "only in shared page cache monitor slot"
"62" 'PageKindsWrittenByStone'    "only in shared page cache monitor slot"
   " 61/62 PageKind counts are returned in an Array:

     Array   Kind
     Index:  Code:    PageType:
         1.       0       REP_INVALID_PAGE_KIND
         2.       1       REP_ROOT_PAGE
         3.       2       REP_FRAGMENT_BM_PAGE
         4.       3       REP_OLD_COMMIT_RECORD_PAGE
         5.       4       REP_DATA_PAGE
         6.       5       REP_OT_INTERNAL_PAGE
         7.       6       REP_OT_LEAF_PAGE
         8.       7       REP_BM_INTERNAL_PAGE
         9.       8       REP_BM_LEAF_PAGE
        10.       9       REP_MULT_OBJ_PAGE
        11.      10       REP_BKUP_REST_PAGE_UNUSED
        12.      11       REP_COUNT_BAG_LEAF_PAGE
        13.      12       REP_BITLIST_PAGE
        14.      13       REP_DEAD_DATA_PAGE
        15.      14       REP_COUNT_MAP_LEAF_PAGE
        16.      15       REP_COUNT_BAG_INTERIOR_PAGE
        17.      16       REP_LOG_RECORD
        18.      17       REP_ROOT_PAGE_40
        19.      18       REP_BACKUP_RECORD
        20.      19       REP_LOST_OT_PAGE
        21.      20       REP_COMMIT_RECORD_PAGE "
"63" 'MilliSecPerIoSample'        
"64" 'AllSymbolsQueueSize'        "only in Stone process slot"
"65" 'PageWaitQueueSize'          "only in Stone process slot"
"66" 'LogWaitQueueSize'           "only in Stone process slot"
"67" 'TempPagesDisposed'          "only in Stone process slot"
"68" 'PersistentPagesDisposed'    "only in Stone process slot"
"69" 'PageDisposesDeferred'       "only in Stone process slot"
"70" 'SigAbortCount'              "only in Gem process slot"
"71" 'SigLostOtCount'             "only in Gem process slot"
"72" 'MessagesToStone'            "only in Gem process slot"
"73" 'ProgressCount'              "only in Gem process slot"
"74" 'NewSymbolsCount'            "only in Gem process slot"
"75" 'BytesCommittedCount'        "only in Gem process slot"
"76" 'FreePages'                  "only in Gem & Stone process slots"
"77" 'ClientPageReads'            "only in page server slots"
"78" 'ClientPageWrites'           "only in page server slots"
"79" 'VcCacheScavengesCount'      "only in Gem process slot"
"80" 'VcCacheSizeBytes'           "only in Gem process slot"
"81" 'CodeCacheSizeBytes'         "only in Gem process slot"
"82" 'CodeCacheEntries'           "only in Gem process slot"
"83" 'CodeCacheStaleEntries'      "only in Gem process slot"
"84" 'CodeCacheScavengesCount'    "only in Gem process slot"
"85" 'FramesFromFreeList'
"86" 'FramesFromFindFree'
"87" 'GcSweepCount'               "only in Stone process slot"
"88" 'GcPossibleDeadSize'         "only in Stone process slot"
"89" 'GcPossibleDeadWsUnionSize'  "only in Stone process slot"
"90" 'GcPagesNeedReclaiming'      "only in Stone process slot"
"91" 'GcDeferEpochThreshold'      "only in Stone process slot"
"92" 'GcReclaimMaxPages'          "only in Stone process slot"
"93" 'GcReclaimNewDataPagesCount' "only in Stone process slot"
"94" 'VoteNotDead'                "only in Gem process slot"
"95" 'TransactionLevel'           "only in Gem process slot"
"96" 'NonSharedAttached'          
"97" 'PrivateAttachLimit'         
"98" 'LastWakeupInterval'         "only in shared page cache monitor process slot"
"99" 'ActiveProcessCount'         "only in shared page cache monitor process slot"
"100" 'RecentActiveProcessCount'  "only in shared page cache monitor process slot"
"101" 'TimeInPgsvrNetReads'       "only in Gem and Stone process slots"
"102" 'TimeInPgsvrNetWrites'      "only in Gem and Stone process slots"
"103" 'GsMsgCount'                "only in Stone process slot"
"104" 'GsMsgSessionId'            "only in Stone process slot"
"105" 'GsMsgKind'                 "only in Stone process slot"
"106" 'StnLoopState'              "only in Stone process slot"
"107" 'DeferCkptCompleteCount'    "only in Stone process slot"
"108" 'ReclaimWaitQueueSize'      "only in Stone process slot"
"109" 'TotalAborts'               "only in Stone process slot"
"110" 'FailedAioCount'            "only in Stone process slot"
"111" 'FreeFrameLimit'            "in all slots"
"112" 'EpochGcCount'              "only in Gem process slot"
"113" 'EpochNewObjsSize'          "only in Gem process slot"
"114" 'EpochScannedObjs'          "only in Gem process slot"
"115" 'EpochPossibleDeadSize'     "only in Gem process slot"
"116" 'CacheMisses'               "In all process slots "
"117" 'CacheEvents'               "ditto.."
"118" 'CacheMissRatio'
"119" 'CacheAttachFactor'
"120" 'CacheDetachFactor'
"121" 'TimeInFramesFromFindFree'
"122" 'FramesAddedToFreeList'
"123" 'PageLocateCount'
"124" 'ObjectTablePageReads'
"125" 'DataPageReads'
"126" 'BitMapPageReads'
"127" 'OtherPageReads'
"128" 'GemsInCacheCount'               "only in SHRPC monitor slot"
"129" 'PagesRemovedFromCacheCount'     "only in SHRPC monitor slot"
"130" 'PagesNotRemovedFromCacheCount'  "only in SHRPC monitor slot"
"131" 'DirtyPageSweepCount'            "only in PGSVR slots"
"132" 'TimeInStnGetLocks'              "only in stone slot"
"133" 'StoneGetLocksCount'             "ditto.."
"134" 'TotalNewObjsCommitted'
"135" 'FreeOopCount'
"136" 'TotalSessionsCount'
"137" 'CommitTokenSession'
"138" 'StnLoopCount'
"139" 'GcInReclaimAll'
"140" 'GcVoteUnderway'
"141" 'GcNotConnectedDeadCommittedCount'      "only in gem slot"
"142" 'TimeInGcNotConnected'                  "ditto.."
"143" 'AttachDeltaPagesSatisfiedCount'
"144" 'ShadowedPagesCount'
"145" 'RcConflictCount'
"146" 'AllSymbolsConflictCount'
"147" 'CommitRetryFailureCount'
"148" 'PageReadsWaitingForCommit'
"149" 'PageReadsProcessingCommit'
"150" 'PageReadsStoneCommit'
"151" 'MessagesToStnWaitingForCommit'
"152" 'MessagesToStnProcessingCommit'
"153" 'MessagesToStnStoneCommit'
"154" 'MessageKindToStone'
"155" 'TimeWaitingForStone'
"156" 'UpdateUnionsCommitCount'
"157" 'TimeInUpdateUnionsCommit'
"158" 'RebuildScavPagesForCommitCount'
"159" 'PageReadsRebuildScavPagesCommit'
"160" 'MessagesToStnRebuildScavPagesCommit'
"161" 'TimeInRebuildScavPagesCommit'
"162" 'OldSpaceOverflowCount'
"163" 'SigAbortsSent'
"164" 'SigAbortsReceived'
"165" 'LostOtsSent'
"166" 'LostOtsReceived'
"167" 'OldestCrSession'              "only in stone slot"
"168" 'TargetFreeFrameCount'         "only in SHRPC monitor slot"
"169" 'RecoverTranlogFileId'         "only in stone slot"
"170" 'RecoverTranlogBlockId'        "only in stone slot"
"171" 'RecoverCrBacklog'             "only in stone slot"
"172" 'SessionNotVoted'              "only in stone slot"
"173" 'SpinLockPageFrameSleepCount'  "only in SHRPC monitor slot"
"174" 'SpinLockHashTableSleepCount'  "only in SHRPC monitor slot"
"175" 'SpinLockFreeFrameSleepCount'  "only in SHRPC monitor slot"
"176" 'SpinLockFreePceSleepCount'    "only in SHRPC monitor slot"
"177" 'SpinLockSmcQSleepCount'       "only in SHRPC monitor slot"
"178" 'SpinLockOtherSleepCount'      "only in SHRPC monitor slot"
"179" 'LocalCacheOverflowCount'      "All Slots"
"180" 'LocalCacheFreeFrameCount'     "All Slots"
"181" 'ExtentFlushCount'             "All Slots"
"182" 'AsyncFlushesInProgress'       "only in page server slot"
"183" 'DetachAllPagesCount'          "only in gem slot"
"184" 'CommitRecordsDisposedCount'   "only in stone slot"
"185" 'AioRateLimit'                 "only in page server slot"
"186" 'GcForceEpoch'                 "only in stone slot"
"187" 'GcReclaimState'               "only in stone slot"
"188" 'GcEpochState'                 "only in stone slot"
"189" 'ClientPid'                    "only in page server slot"
"190" 'StnLoopNetPollCount'          "only in stone slot"
"191" 'StnLoopTimeInNetPoll'         "only in stone slot" 
"192" 'StnLoopNetPollOobCount'       "only in stone slot"
"193" 'StnLoopTimeInNetPollOob'      "only in stone slot"
"194" 'RemoteSessionCount'           "only in stone slot"
"195" 'LogIOSlotCount'               "only in stone slot"
"196" 'TimeInLogIOWait'              "only in stone slot"
"197" 'PgsvrPid'                     "only in gem slot"
"198" 'NumInPgsvrWaitQueue'          "only in stone slot"
"199" 'NumInSetLostOtQueue'          "only in stone slot"
"200" 'NumInRemotePidQueryQueue'          "only in stone slot"
"201" 'NumInRemoteKillQueue'          "only in stone slot"
"202" 'SmcQueueSize'                  "obsolete"
"203" 'PageMgrPagesNotRemovedFromCachesCount' "only in stone slot"
"204" 'PageMgrRemoveFromCachesPageCount'      "only in stone slot"
"205" 'PageMgrPagesRemovedFromCachesCount'    "only in stone slot"
"206" 'PageMgrPagesPendingRemovalRetryCount'  "only in stone slot"
"207" 'RemoteSharedPageCacheCount'            "only in stone slot"
"208" 'PageMgrPagesReceivedFromStoneCount'    "only in stone slot"
"209" 'PageMgrRemoveFromCachesCount'          "only in stone slot"
"210" 'PageMgrRemovePagesFromCachesPollCount' "only in stone slot"
"211" 'PageMgrTimeWaitingForCachePgsvrs'      "only in stone slot"
"212" 'PagesWaitingForRemovalInStoneCount'    "only in stone slot"
"213" 'RemoteCachesNeedServiceCount'          "only in stone slot"
"214" 'TimeInStonePageDisposal'               "only in stone slot"
"215" 'PagesNeedRemovingThreshold'            "only in stone slot"
"216" 'OldestCrSessNotInTrans'                "only in stone slot"
"217" 'PagesRemovedDirtyFromCacheCount'       "only in SHRPC monitor slot" 
"218" 'PagesNotFoundInCacheCount'             "only in SHRPC monitor slot" 
"219" 'PinnedPagesCount'                      "All Slots"
"220" 'PrivatePinnedPagesCount'               "All Slots"
"221" 'RemoteSharedPageCacheMax'              "only in stone slot"
"222" 'WaitsForOtherReader'                   "All Slots"
"223" 'AioWriteFailures'                      "only in AIO page server slots"
"224" 'GcHighWaterPage'                       "only in stone slot"
"225" 'BackupHighWaterPage'                   "only in stone slot"
"226" 'CheckpointState'                       "only in stone slot"
"227" 'OopNumberHighWaterMark'                "only in stone slot"
"228" 'GciRpcKeepAlivePacketCount'            "only in gem slot"
"229" 'StopSessionCount'                      "only in gem slot"
"230" 'ClientSigAborts'                       "only in gem page server slots"
"231" 'ClientLostOtRoots'                     "only in gem page server slots"
"232" 'ClientCallsToStone'                    "only in gem page server slots"
"233" 'ClientAborts'                          "only in gem page server slots"
"234" 'ClientCommits'                         "only in gem page server slots"
"235" 'ClientFailedCommits'                   "only in gem page server slots"
"236" 'ClientStopSessionRequests'             "only in gem page server slots"
"237" 'SmcQueuesInUse'                        "only in stone slot"
"238" 'SmcQueueSize0'                         "only in stone slot"
"239" 'SmcQueueSize1'                         "only in stone slot"
"240" 'SmcQueueSize2'                         "only in stone slot"
"241" 'SmcQueueSize3'                         "only in stone slot"
"242" 'SmcQueueSize4'                         "only in stone slot"
"243" 'SmcQueueSize5'                         "only in stone slot"
"244" 'SmcQueueSize6'                         "only in stone slot"
"245" 'SmcQueueSize7'                         "only in stone slot"
"246" 'ProcessesWaitingForQueueLocks'         "only in SHRPC monitor slot" 
"247" 'NumSlotsPendingReuse'                  "only in SHRPC monitor slot" 
"248" 'PageMgrPrintTimeoutThreshold'          "only in stone slot"
"249" 'PageMgrCompressionEnabled'             "only in stone slot"
"250" 'NumProcsSleepingForLock'               "only in SHRPC monitor slot" 
"251" 'TotalNewSymbolsCommitted'              "only in stone slot"
"252" 'GcLockSession'                         "only in stone slot"
"253" 'StoredPomObjsMapSize'                  "only in gem slot"
"254" 'StoredLomObjsMapSize'                  "only in gem slot"
)
%

category: 'Performance Monitoring'
classmethod: System
cacheStatisticsForSessionId: aSessionId

"Same as the cacheStatistics: method except the argument is the session ID
 of a session currently connected to the shared memory cache.  In systems
 that use multiple shared memory caches, the session must exist on the same
 cache as the session invoking this method.

 nil is returned if the session cannot be located."

 ^self _cacheStatsForSlotOrSession: aSessionId negated.
%
category: 'Authorization'
classmethod: System
canRead: anObject

"This method tests whether the user has authorization to read anObject without
 adding it to the read set and returns a Boolean result."

^self _checkAuthorization: anObject forRead: true
%
category: 'Authorization'
classmethod: System
canWrite: anObject

"This method tests whether the user has authorization to write anObject without
 adding it to the read set or write set and returns a Boolean result."

^self _checkAuthorization: anObject forRead: false
%
category: 'Private'
classmethod: System
changeCacheSlotFreeFrameLimit: aSlot to: aValue

"Changes the freeFrameLimit for the process associated with a cache slot to 
 aValue, which must be a positive SmallInteger.  The argument aSlot
 should be a SmallInteger between 0 and the number of process slots in the
 shared cache minus 1, inclusive. 

 To execute this method, you must have the SystemControl privilege."

<primitive: 349>

aSlot _validateClasses:#[SmallInteger].
aValue _validateClasses:#[SmallInteger].
self _primitiveFailed: #changeCacheSlotFreeFrameLimit:to:
%
category: 'Private'
classmethod: System
changeCacheSlotIoLimit: aSlot to: aValue

"Changes the ioLimit for the process associated with a cache slot to aValue,
 which must be in the range of 1 to 5000 I/Os per second.  The argument aSlot
 should be a SmallInteger between 0 and the number of process slots in the
 shared cache minus 1, inclusive. 

 To execute this method, you must have the SystemControl privilege."

<primitive: 464>

aSlot _validateClasses:#[SmallInteger].
aValue _validateClasses:#[SmallInteger].
self _primitiveFailed: #changeCacheSlotIoLimit:
%
category: 'Transaction Control'
classmethod: System
checkpoint

"Attempt to commit the transaction for the current session, as a checkpoint.
 If full logging mode is in use, this method waits for any asynchronous
 checkpoints already in progress to complete, and then starts an asynchronous
 checkpoint.  If partial logging mode is in use, this method performs a 
 synchronous checkpoint.

 This method is the same as 'commitAndReleaseLocks' except for the checkpoint.
 If the current transaction is a read-only transaction, then this method is
 equivalent to 'commitTransaction', and a checkpoint is not written.

 Frequent use of this method will seriously degrade performance.  If partial
 logging mode is in use and extents are replicated, it can be used to ensure
 that a large transaction is recoverable after a disk failure.  Otherwise, it
 should only be used for quality assurance testing."

^ self _commit: 2 "checkpoint and release locks if successful"
%
category: 'Private'
classmethod: System
clearAttachedPages

"Detaches the data pages currently attached."

^ self _zeroArgPrim: 28
%
category: 'Releasing Locks'
classmethod: System
clearCommitOrAbortReleaseLocksSet

"Remove all objects from the commit-or-abort release locks set."

self _hiddenSetReinit: 30.
^self
%
category: 'Releasing Locks'
classmethod: System
clearCommitReleaseLocksSet

"Remove all objects from the commit release locks set."

self _hiddenSetReinit: 29.
^self
%
category: 'Notification'
classmethod: System
clearNotifySet

"Remove all of the objects that are currently in the notify set."

self removeAllFromNotifySet: (self notifySet)
%
category: 'Reduced Conflict Support'
classmethod: System
clearRcValueCache

"Clears the cache of calculated values for reduced conflict classes by setting
 the temporary session state slot to nil."

self _sessionStateAt: 5 put: nil
%
category: 'Reduced Conflict Support'
classmethod: System
clearRedoLog

"Clear the redo log by setting the temporary session state slot to nil.

                            Warning:
 Clearing the redo log will probably prevent Reduced Conflict
 classes from resolving conflicts.  Sending this message negates this
 capability for the current transaction.

 This is a protected method."

< protected >

self _sessionStateAt: 4 put: nil
%
category: 'Environment Access'
classmethod: System
clientEnvironmentVariable: varName

"Expands the environment variable named varName in the GemBuilder for C client
 process, returning a String.  The varName argument should be a kind of String.

 Returns nil if any of the following are true:

 * varName is not a byte format object.
 * There is no environment variable defined with name varName.
 * The value of the environment variable is more than approximately 8000 bytes.
 * The size of varName exceeds approximately 8000 bytes."

^ GsFile _expandEnvVariable: varName isClient: true
%
category: 'Session Control'
classmethod: System
clientIsRemote

"Returns true if the GemBuilder for C client for this session is in a different
 process than the Gem, otherwise returns false."

^ self _zeroArgPrim: 20
%
category: 'Version Management'
classmethod: System
clientVersionAt: aSymbol

"Returns version information about the client GemBuilder for C process.  If the
 client is a session using the linkable GemBuilder for C login, this method is
 equivalent to gemVersionAt:.

 See System(C) | gemVersionAt: for further details."

| verId |
verId := VersionParameterDict at: aSymbol ifAbsent:[ ^ nil ].
^ GsFile _clientVersionAt: verId 
%
category: 'Version Management'
classmethod: System
clientVersionReport

"Return a SymbolDictionary whose keys are the names of operating system,
 hardware, or GemStone version attributes, and whose values are the
 current values of those attributes in the client GemBuilder for C process."

| result |
result := SymbolDictionary new .
VersionParameterDict keysAndValuesDo:[ :aName :anId | | aVal |
  aVal := GsFile _clientVersionAt: anId . 
  aVal ~~ nil ifTrue:[ 
    result at: aName put: aVal
    ].
  ].
^ result 
%
category: 'Clustering'
classmethod: System
clusterAllSymbols

"This method clusters the AllSymbols hash dictionary and all of the symbols to
 which it refers."

^ self _zeroArgCateg2Prim: 6
%
category: 'Backward Compatibility'
classmethod: System
clusterBucket

"Obsolete in GemStone 3.2."

^ self currentClusterId
%
category: 'Clustering'
classmethod: System
clusterBucket: aClusterBucketOrId

"This method sets the current default ClusterBucket to the ClusterBucket with
 the specified clusterId.  The argument may be an instance of ClusterBucket, or
 a positive SmallInteger which specifies an instance of ClusterBucket."

<primitive: 348>
self _primitiveFailed: #clusterBucket: .
self _uncontinuableError
%
category: 'Transaction Control'
classmethod: System
commitAndReleaseLocks

"Attempt to commit the transaction for the current session.

 This method is the same as 'commitTransaction' except for the handling of
 locks.  If the commit succeeds, this method releases all locks for the session
 and returns true.  Otherwise, it returns false and does not release locks.

 This method also clears the commit release locks and commit-or-abort release
 locks sets.  See the 'Releasing Locks' method category for more
 information."

^ self _commit: 1 "release locks if successful"
%
category: 'Releasing Locks'
classmethod: System
commitOrAbortReleaseLocksSetIncludes: anObject

"Returns true if anObject is in the commit-or-abort release locks set.
 Returns false otherwise."

^self _testIf: anObject isIn: 30
%
category: 'Releasing Locks'
classmethod: System
commitReleaseLocksSetIncludes: anObject

"Returns true if anObject is in the commit release locks set. 
 Returns false otherwise."

^self _testIf: anObject isIn: 29
%
category: 'Transaction Control'
classmethod: System
commitTransaction

"Attempts to update the persistent state of the Repository to include changes
 made by this transaction.

 If the commit operation succeeds, then this method returns true, and the
 current transaction's changes, if any, become a part of the persistent
 Repository.  After the repository update, the session exits the current
 transaction.  If the transaction mode is #autoBegin, then the session enters
 a new transaction.  If the transaction mode is #manualBegin, then the session
 remains outside of a transaction.

 If conflicts prevent the repository update, then this method returns false.
 Call the transactionConflicts method to determine the nature of the
 conflicts.  If the session is outside of a transaction, then this method
 raises the error #rtErrPrimOutsideTrans. 

 This method also updates the session's view of GemStone.  If the commit
 operation succeeds, then all objects in the session's view are consistent with
 the current state of GemStone.  If the commit fails, then this method retains
 all the changes that were made to objects within the current transaction.
 However, commits made by other sessions are visible to the extent that changes
 in this transaction do not conflict with them."

^ self _commit: 0 "don't release locks"
%
category: 'System Control'
classmethod: System
concurrencyMode

"Returns the Concurrency Mode in the form of one of the following symbols:
 #FULL_CHECKS or #NO_RW_CHECKS."

| result |
result := self configurationAt: #ConcurrencyMode .
(result == 0) ifTrue: [^#FULL_CHECKS].
(result == 1) ifTrue: [^#NO_RW_CHECKS].
^'Unknown Concurrency Control Mode'
%
category: 'System Control'
classmethod: System
concurrencyMode: aString

"Sets the Concurrency Control Mode, where aString is one of the following
 symbols: #FULL_CHECKS or #NO_RW_CHECKS.

 The default is #FULL_CHECKS, which enables read-write and write-write conflict
 checking.  In #NO_RW_CHECKS, a read-write conflict won't cause commit failure,
 but a read set is still maintained to support object locking.

 If you are not the only user logged in to GemStone this method generates an
 error.  To execute this method, you must have the SessionAccess privilege.

 The Garbage Collector session is shut down for the duration of this method."

| aValue aSymbol |
aValue := -1.
aSymbol := aString asUppercase asSymbol.
(aSymbol = #FULL_CHECKS) ifTrue: [aValue := 0].
(aSymbol = #NO_RW_CHECKS) ifTrue: [aValue := 1].
(aValue == -1) ifTrue: [^aString _error: #concurErrInvalidMode].
self configurationAt: #ConcurrencyMode put: aValue.
%
category: 'Configuration File Access'
classmethod: System
configurationAt: aName 

"Returns the value of the specified configuration file parameter, giving
 preference to the Gem process if the parameter applies to the Gem."
 
| result |
result := self gemConfigurationAt: aName .
result == nil ifTrue:[
  result := self stoneConfigurationAt: aName
  ].
^ result
%
category: 'Runtime Configuration Access'
classmethod: System
configurationAt: aName put: aValue

"Change the value of the specified configuration parameter.  

 The changeable parameters all require aValue to be a SmallInteger.

 Configuration parameters should not be changed unless there is a clear
 reason for doing so, since incorrect settings of parameters can
 have serious adverse effects on GemStone performance.

 Configuration parameters for Stone that are transferred to Gem processes
 are only read by the Gem at login, so changes using this method to
 Stone parameters may have no effect on existing sessions.

 Parameters in the Gem with the following names may be changed by any user at
 any time:

   * #NotConnectedThreshold
   * #GemIOLimit
   * #GemTempObjCacheSize
   * #GemNativeCodeThreshold
   * #GemNativeCodeMax
   * #GemFreeFrameLimit
   * #NotConnectedDelta

 Parameters in the Gem with the following names may be changed only by users
 who have the correct privilege and who follow any other restrictions:

   * #ConcurrencyMode - Requires SessionAccess privilege.  The current session
                        must be the only session logged in other than GcGem.
   * #LoginsSuspended - Requires SystemControl privilege.
   * #StnLogLoginFailureLimit - Requires OtherPassword privilege.
   * #StnLogLoginFailureTimeLimit - Requires OtherPassword privilege.
   * #StnDisableLoginFailureLimit - Requires OtherPassword privilege.
   * #StnDisableLoginFailureTimeLimit - Requires OtherPassword privilege.
   * #GcSessionEnabled - Requires GarbageCollection privilege.
   * #GcSessionConfiguration - Requires GarbageCollection privilege.

 All other parameters in the Gem that are changeable at run time may be changed
 only by SystemUser, and should not be changed in the course of normal GemStone
 operation."

| cfgId |

cfgId := ConfigurationParameterDict at: aName .
self _atConfigId: cfgId put: aValue .
^ self configurationAt: aName
%
category: 'Backward Compatibility'
classmethod: System
contentsOfServerDirectory: aSpecString

"Obsolete in GemStone 5.0.  Use the GsFile>>contentsOfDirectory:onClient:
 method instead."

<primitive: 345>

aSpecString _validateClass: String.
^ aSpecString _error: #hostErrFileDirectory args:  #() 
%

category: 'Transaction Control'
classmethod: System
inContinueTransaction

"Returns a boolean indicating if the current transaction has been 
 continued.  Answers true if System>>continueTransaction has been
 executed and no abort or successful commit has been performed.
 Otherwise returns false."

^ self _zeroArgPrim: 62
%


category: 'Transaction Control'
classmethod: System
continueTransaction

"Updates the session's view to the most recently committed GemStone state.

 If the session is in a transaction, this method preserves modifications 
 made to committed GemStone objects.  The read and write sets of the session 
 are carried forward and continue to accumulate until the session either 
 commits or aborts.  

 If the session is not in a transaction, this method performs an abort,
 discarding any accumulated changes to committed objects.

 Returns true if accumulated modifications to the committed objects would not
 cause concurrency conflicts; otherwise returns false.  If the method 
 returns false, you can call the transactionConflicts method to determine
 the nature of the conflicts.

 This method can be used whether or not the session is outside of a transaction.
 Of course, the session cannot commit accumulated changes unless it is
 inside a transaction.

 If transaction mode is #manualBegin, then continueTransaction does not alter
 the inside/outside of transaction state of the session.

 Modifications made by other committed transactions are accumulated for
 retrieval by GciDirtyObjs() and GciDirtySavedObjs() just as they are
 accumulated for commitTransaction or abortTransaction.

 This method has no effect on object locks.  Locks in the release locks sets 
 are not released.

                                Warnings:
 This method must not be executed if the session is adding/removing 
 objects to/from an RcQueue in the current transaction.  Attempts to
 modify an RcQueue in a continued transaction will result in a runtime
 error (#rtErrRcUpdateDisallowed).

 Once continueTransaction has been used within a transaction, a subsequent
 commit of that transaction will ignore read-write and write-read conflicts.
 To check for read-write and write-read conflicts, a transaction could use the
 sequence continueTransaction, transactionConflicts, commitTransaction and check
 the result of transactionConflicts before doing the commitTransaction."

| continueResult |

continueResult := self _zeroArgPrim: 9.
self clearRcValueCache.

continueResult == 0
  ifTrue: [ ^ true ].
    
continueResult == 1
  ifTrue: [ self _resolveRcConflicts
               ifTrue: [ ^true ]
               ifFalse: [ self _disallowSubsequentCommits.
                          ^false "RC validation failed" ]].
        
continueResult == 2 "retry for AllSymbols"
  ifTrue: [ ^ true ].

"commitResult of 
   3 means validation failure,
   4 means maximum number of commit retries was reached,
   5 means commit disallowed (usually because of indexes)"

^ false
%
category: 'Clustering'
classmethod: System
currentClusterBucket

"Returns the instance of ClusterBucket that is the current default."

^ AllClusterBuckets at: System currentClusterId
%
category: 'Clustering'
classmethod: System
currentClusterId

"This method returns a SmallInteger that is the ID of the ClusterBucket that
 is the current default bucket."

<primitive: 96>
self _primitiveFailed: #currentClusterId .
self _uncontinuableError
%
category: 'Session Control'
classmethod: System
currentSegment

"Returns the Segment in which objects created in the current session are
 stored.  At login, the current segment is the default segment of the
 UserProfile for the session of the sender."

<primitive: 341>
self _primitiveFailed: #currentSegment .
self _uncontinuableError
%
category: 'Session Control'
classmethod: System
currentSegment: aSegment

"Redefines the Segment in which subsequent objects created in the
 current session will be stored.  Returns the receiver.

 Exercise caution when executing this method.  If, at the time you
 attempt to commit your transaction, you no longer have write
 authorization for aSegment, an error will be generated, and you
 will be placed back into your default Segment."

<primitive: 339>
aSegment _validateClass: Segment.
self _primitiveFailed: #currentSegment: .
self _uncontinuableError
%
category: 'Session Control'
classmethod: System
currentSessionNames

"Returns a formatted String containing, for each current GemStone session, the
 session number and userId.

 If any sessions (including the GcGem) other than the current session are
 logged in, this method requires SessionAccess privilege."

| lf tempString anArrayOfSessions aSessionNumber aUserProfile |

tempString := String new.  " string to return "
anArrayOfSessions := self currentSessions.  " all session running "
lf := Character lf.

1 to: anArrayOfSessions size do:[:j|
   aSessionNumber := anArrayOfSessions at: j .
   aUserProfile := self userProfileForSession: aSessionNumber.
   (aUserProfile ~~ nil) ifTrue: [
     tempString
        add: lf;
        addAll: 'session number: ';
        addAll: (aSessionNumber asString);
        addAll: '    UserId: ';
        addAll: (aUserProfile userId).
      ]
   ].
^tempString
%
category: 'Session Control'
classmethod: System
currentSessions

"Returns an Array of SmallIntegers corresponding to all of the sessions
 currently running on the GemStone system."

<primitive: 329>
^ self _primitiveFailed: #currentSessions
%
category: 'Backward Compatibility'
classmethod: System
deleteServerFile: aFileSpec

"Obsolete in GemStone 5.0.  Use the GsFile>>removeServerFile: method instead."

<primitive: 346>
aFileSpec _validateClass: String.
aFileSpec _error: #hostErrFileDelete args:  #() 
%
category: 'Session Control'
classmethod: System
descriptionOfSession: aSessionId

"Returns an eleven-element Array describing the session.

 1.  The UserProfile of the session.
 2.  The process ID of the Gem process of the session (an Integer).
 3.  The hostname of the machine running the Gem process
     (a String, limited to 127 bytes).
 4.  Primitive number in which the Gem is executing, or 0 if it is not executing
     in a long primitive.
 5.  Time of the session's most recent beginTransaction, commitTransaction, or
     abortTransaction (from System timeGmt).
 6.  The session state (a SmallInteger).
 7.  A SmallInteger whose value is -1 if the session is in transactionless mode,
     0 if it is not in a transaction and 1 if it is in a transaction.
 8.  A Boolean whose value is true if the session is currently referencing the
     oldest commit record, and false if it is not.
 9.  The session's serial number (a SmallInteger).
 10. The session's sessionId (a SmallInteger).
 11. A String containing the ip address of the session's client's host.
     If a session has no client then the String contains the ip address
     of machine running the Gem process.

 Because a session can update its commit record without committing a
 transaction, it is possible that no session actually references the oldest
 commit record.  Therefore, the eighth element may be false for all current
 sessions.

 To execute this method for any session other than your current session, you
 must have the SessionAccess privilege."

^ self _descriptionOfSessionSerialNum: 0 sessionId: aSessionId
%
category: 'Session Control'
classmethod: System
descriptionOfSessionSerialNum: aSerialNumber

"Returns an eleven-element Array describing the session identified by aSerialNumber.

 See System (C) | descriptionOfSession: for documentation on the contents of the
 result Array.

 Requires SessionAccess privilege if aSerialNumber is not the current session."

^ self _descriptionOfSessionSerialNum: aSerialNumber sessionId: -1
%
category: 'Transaction Control'
classmethod: System
disableSignaledAbortError

"Disables the generation of an error when Stone signals the Gem session that it
 should abort when running outside of a transaction."

self _updateSignalErrorStatus: 2 toState: false
%
category: 'Notification'
classmethod: System
disableSignaledGemStoneSessionError

"Set the current GemStone session so that it cannot receive signals from
 other GemStone sessions."

self _updateSignalErrorStatus: 3 toState: false
%
category: 'Notification'
classmethod: System
disableSignaledObjectsError

"Disable the generation of an error when a member of the notify set
 is added to the signaled objects set."

self _updateSignalErrorStatus: 1 toState: false
%
category: 'Transaction Control'
classmethod: System
enableSignaledAbortError

"Enables the generation of an error when the Stone has signaled that the Gem
 process should abort to connect to a more current GemStone root. 

 This method must be invoked after each delivery of the signal-abort error, to 
 reenable generation of the error."

self _updateSignalErrorStatus: 2 toState: true
%
category: 'Notification'
classmethod: System
enableSignaledGemStoneSessionError

"Enable the current GemStone session to receive signals from other GemStone
 sessions.  One GemStone session receives a signal from another session when a
 RT_ERR_SIGNAL_GEMSTONE_SESSION exception is raised.

 The receiving session processes the signal with an exception handler.  When
 GemStone raises one signal exception, it also disables further signal
 exceptions, to allow the exception handler to run without receiving another
 interrupt.  The exception handler should therefore re-enable signal exceptions
 when it is done with its other processing.

 A signal is not exactly an interrupt, and it does not automatically awaken an
 idle session.  Both the GemStone Smalltalk virtual machine and GemBuilder for C
 can raise the signal exception.  But the process of the session must activate
 the virtual machine or interface before the signal can be received."

self _updateSignalErrorStatus: 3 toState: true
%
category: 'Notification'
classmethod: System
enableSignaledObjectsError

"Enable the generation of an error when a member of the notify set is added to
 the signaled objects set.  When this error (RT_ERR_SIGNAL_COMMIT) is signaled,
 it is also disabled to allow the exception handler to run without receiving 
 another interrupt.  Therefore, the exception handler should re-enable 
 the condition."

self _updateSignalErrorStatus: 1 toState: true
%
category: 'Setting Locks'
classmethod: System
exclusiveLock: anObject

"Requests an exclusive lock on anObject.  This method denies an exclusive lock
 on an object under the following circumstances:

 * Another session already holds any kind of lock to the object.

 * You do not have write authorization for the object.

 Returns the receiver if the requested lock was granted and was not dirty.

 This method grants an exclusive lock on an object whenever it finds no reason
 to deny it.  However, the lock that it grants may be dirty.  One session's
 lock is dirty if another session has committed a change to the locked object
 since the beginning of the first session's current transaction.  A session
 that holds a dirty lock cannot commit its transaction.  To clean its locks,
 it must abort the transaction and obtain updated values for each object whose
 lock is dirty.

 This method generates an error if the requested lock is denied.  It also
 generates an error if the lock that it grants is dirty, but in this case the
 lock remains, even after the transaction is aborted."

| result |

result := System _lock: anObject kind: #exclusive.
(result == 1)
  ifTrue: [^self].
self _lockError: result obj: anObject args: #[#exclusive]
%
category: 'Setting Locks'
classmethod: System
exclusiveLock: anObject ifDenied: denyBlock ifChanged: changeBlock

"Requests an exclusive lock on anObject.  This method denies an exclusive lock
 on anObject under any one of the following circumstances:

 * Another session already holds any kind of lock to the object.

 * You do not have write authorization for the object.

 * The object is special.

 Returns the receiver if the requested lock was granted and was not dirty.

 This method grants an exclusive lock on anObject whenever it finds no reason
 to deny it.  However, the lock that it grants may be dirty.  One session's
 lock is dirty if another session has committed a change to the locked object
 since the beginning of the first session's current transaction.  A session
 that holds a dirty lock cannot commit its transaction.  To clean its locks,
 it must abort the transaction and obtain updated values for each object whose
 lock is dirty.

 This method generates an error if you do not have write authorization for
 anObject.  If the requested lock is otherwise denied, it returns the value of
 the zero-argument block denyBlock.  If it grants a dirty lock, then it
 returns the value of the zero-argument block changeBlock.  In that case the
 lock remains, even after the transaction is aborted."

| result |

result := System _lock: anObject kind: #exclusive.
(result == 1) ifTrue:
   [^self].
^ self _lockEvaluateErr: result denied: denyBlock changed: changeBlock
%
category: 'Backward Compatibility'
classmethod: System
exclusiveLock: anObject ifDenied: denyBlock ifChanged: changeBlock
  ifNotCommitted: notCommittedBlock

"Obsolete in GemStone 4.0.  Locks are now allowed on objects that have not been
 committed."

^self exclusiveLock: anObject ifDenied: denyBlock ifChanged: changeBlock
%
category: 'Setting Locks'
classmethod: System
exclusiveLockAll: aCollection

"Requests an exclusive lock on each object in aCollection.  This method denies
 an exclusive lock on an object under the following circumstances:

 * Another session already holds any kind of lock to the object.

 * You do not have write authorization for the object.

 If you lack write authorization for any object in aCollection, this method
 generates an error and no locks are granted.  Otherwise, this method acquires
 locks on as many objects in aCollection as possible.

 This method grants an exclusive lock on an object whenever it finds no reason
 to deny it.  However, a lock that it grants may be dirty.  One session's
 lock is dirty if another session has committed a change to the locked object
 since the beginning of the first session's current transaction.  A session
 that holds a dirty lock cannot commit its transaction.  To clean its locks,
 it must abort the transaction and obtain updated values for each object whose
 lock is dirty.

 If a lock was acquired for every element of aCollection, and no locks are
 dirty, returns the receiver.

 This method generates an error if it is unable to acquire a lock for every
 element of aCollection, or if any lock that it acquires is dirty.  However,
 all the locks that it acquires remain in place, even after the current
 transaction is aborted."

| result |
result := System _lockAll: aCollection kind: #exclusive.
(result == System)
  ifTrue: [^self]
  ifFalse: " else we generate the error lock incomplete "
           [^ (result at: 1) _error: #lockErrIncomplete
                             args: #[(result at:2), (result at:3)]].
%
category: 'Setting Locks'
classmethod: System
exclusiveLockAll: aCollection ifIncomplete: incompleteBlock

"Requests an exclusive lock on each object in aCollection.  This method denies
 an exclusive lock on an object under any one of the following circumstances:

 * Another session already holds any kind of lock to anObject.

 * You do not have write authorization for anObject.

 * The object is special.

 If you lack write authorization for any object in aCollection, this method
 generates an error and no locks are granted.  Otherwise, this method acquires
 locks on as many objects in aCollection as possible. If all requested locks
 were granted and none of the locks are 'dirty', returns the receiver.

 This method grants an exclusive lock on an object whenever it finds no reason
 to deny it.  However, a lock that it grants may be dirty.  One session's
 lock is dirty if another session has committed a change to the locked object
 since the beginning of the first session's current transaction.  A session
 that holds a dirty lock cannot commit its transaction.  To clean its locks,
 it must abort the transaction and obtain updated values for each object whose
 lock is dirty.

 If this method is unable to acquire a lock for every element of aCollection,
 or if any lock that it acquires is dirty, then it returns the value of the
 three-argument block incompleteBlock.  The arguments to the block are:

 1.  An Array of objects that could not be locked.

 2.  An Array of objects that were locked but whose locks are dirty.

 3.  An empty Array, retained for backward compatibility with GemStone version
     3.2.  It was used formerly to hold uncommitted objects, which could not
     then be locked.

 All the locks that it acquires remain in place, even after the current
 transaction is aborted."

| result |
result := System _lockAll: aCollection kind: #exclusive.
(result == System)
  ifTrue: [^self]
  ifFalse: " else we execute the incomplete block with 3 arguments "
          [^incompleteBlock value: (result at: 1)
                            value: (result at:2)
                            value: (result at:3)].
%
category: 'Backward Compatibility'
classmethod: System
exclusiveLockObjAndIndexes: anObject

"Obsolete in GemStone 4.0.  Indexes now have reduced conflict behavior and
 locks are now allowed on objects that have not been committed."

^ self exclusiveLock: anObject
%
category: 'Backward Compatibility'
classmethod: System
exclusiveLockObjAndIndexes: anObject ifDenied: denyBlock ifChanged: changeBlock
  ifNotCommitted: notCommittedBlock

"Obsolete in GemStone 4.0.  Indexes now have reduced conflict behavior and
 locks are now allowed on objects that have not been committed."

^ self exclusiveLock: anObject ifDenied: denyBlock ifChanged: changeBlock
%
category: 'Disk Space Management'
classmethod: System
findObjectsLargerThan: aSize limit: aLimit

"Searches GemStone for objects larger than aSize, and returns an Array of
 any such objects.  The search continues until all such objects have been
 found, or until the size of the result reaches the specified maximum aLimit.
 Both aSize and aLimit must be positive SmallIntegers.

 The result contains only those objects which reside within segments that the
 user is authorized to read.  If this method encounters an object larger than
 aSize which it is not authorized to read, the final element of the result will
 be the String 'Read Authorization Error Encountered'.

 This method aborts the current transaction; if an abort would cause unsaved
 changes to be lost, it signals an error, #rtErrAbortWouldLoseData.    

 The result contains both permanent and temporary objects.  The temporary
 objects found may vary from run to run.

 Note that this method may take a considerable length of time to execute."

self needsCommit
  ifTrue: [ self _error: #rtErrAbortWouldLoseData ] .

^ self _findObjectsLargerThan: aSize limit: aLimit 
%

category: 'Disk Space Management'
classmethod: System
_findObjectsLargerThan: aSize limit: aLimit

"Searches GemStone for objects larger than aSize, and returns an Array of
 any such objects.  The search continues until all such objects have been
 found, or until the size of the result reaches the specified maximum aLimit.
 Both aSize and aLimit must be positive SmallIntegers.

 The result contains only those objects which reside within segments that the
 user is authorized to read.  If this method encounters an object larger than
 aSize which it is not authorized to read, the final element of the result will
 be the String 'Read Authorization Error Encountered'.

 The result contains both permanent and temporary objects.  The temporary
 objects found may vary from run to run.

 Note that this method may take a considerable length of time to execute."

<primitive: 398>
aSize _validateClass: SmallInteger.
(aSize < 0) ifTrue: [aSize _error: #rtErrArgNotPositive .  ^ Array new ].
aLimit _validateClass: SmallInteger.
(aLimit < 0) ifTrue: [aLimit _error: #rtErrArgNotPositive.  ^ Array new ].
^ self _primitiveFailed: #findObjectsLargerThan:Limit:
%

category: 'Garbage Collection'
classmethod: System
forceEpochGc

"Force an Epoch GC to run as soon as possible, regardless of the setting 
of the GcUser configuration parameters epochGc[Time|Trans|Byte]Limit.  
Note that the GcUser parameter epochGcEnabled must be TRUE for this method 
to have any effect.  If a normal or Epoch GcGem is not currently running,
this method returns with an rtErrGcSessionInvalid error.

Normally, there may be a short delay (up to several minutes) before the 
epoch GC is actually started, depending on the load on the normal/Epoch 
GcGem.  But under the following conditions the epoch may be delayed
longer, until the specified condition is resolved:

* On a large repository, the normal/Epoch GcGem is performing either 
  a write set union sweep or rebuilding the AllSymbols table.
* The system is processing an earlier MFC/epoch GC.
* The system is performing a reclaimAll.
* The GcUser parameter epochGcEnabled is FALSE.

This method sets the stone cache statistic GcForceEpoch to 1.  Once
the normal/Epoch GcGem has started the epoch GC, GcForceEpoch will
return back to 0.

You must have GarbageCollection privileges to be able to run this method."

| priv |
priv := 
( System myUserProfile privileges includes: #GarbageCollection ) 
  ifFalse: [ self _error: #rtErrNoPriv ].
^ self _startGcSession: 7 extentId: 0
%
category: 'Garbage Collection'
classmethod: System
GcDocumentation

^ 'The following describes the use of the new GC mechanisms provided in 
GS 6.x.  

There are several new types of GcGems provided in 6.x.  Refer to the System 
Administration Guide and the release notes for further details on how they
work.  The methods that control them use a symbolic code to reference
the appropriate GcGem.  Lower-level primitives convert this symbolic code into 
a numeric gcType:

Sym:   Num:   Description:
----   ----   ------------
#GC     1     The original GcGem
#RCL    2     Reclaim Gem (one per Repository)  (available in 6.1)
#EPC    3     Epoch Gem (one per Repository)    (available in 6.1)
#PSR    4     Parallel Shadow Reclaim Gem (one per extent)
#PDR    5     Parallel Dead Reclaim Gem (one per extent)

For clearer code, you may also use the longer symbol names listed below:

Sym:  Long Symbol:
----  ------------
#GC   #GcGem
#RCL  #ReclaimGem
#EPC  #EpochGem
#PSR  #ParallelShadowReclaim
#PDR  #ParallelDeadReclaim


Summary of GcGem Types:

#GC   The original (pre 6.0) GcGem.
          Reclaims shadowed pages, reclaims dead objects, 
          epoch GC, and vote-on-dead wrap-up (write-set union sweep).
#RCL  Reclaims shadowed pages, reclaims dead objects.
#EPC  Epoch GC and vote-on-dead wrap-up.
#PSR  Reclaims shadowed pages on a single extent file.
#PDR  Reclaims dead objects from a single extent file.

A RCL/EPC GcGem pair divides up the work originally accomplished by a single
original (pre 6.0) GcGem.  Running with these two together can be more
effective on a multiple processor machine.

Multiple PSR/PDR GcGems can be run, one on each extent file in the repository.
This can be useful if you have multiple extents and multiple processors.

Certain combinations of these new GcGems cannot be run with each other.
Attempts to run incompatible gems concurrently will result in an error
when attempting to start the second GcGem.  Compatible gems are as follows:

Sym:   Num:   Compatible Gems: 
----   ----   ---------------- 
#GC     1     NONE             
#RCL    2     #EPC
#EPC    3     #RCL
#PSR    4     #PSR (on other extents)
#PDR    5     #PDR (on other extents)

To clarify: *no* other GcGems can run when a #GC GcGem is present;  
#RCL/#EPC GcGems can run together (#GC/#PSR/#PDR not allowed); #PSR 
GcGems can only run with other #PSR GcGems on other extents 
(#GC/#RCL/#EPC/#PDR not allowed); and #PDR GcGems can only run with 
other #PDR GcGems on other extents (#GC/#RCL/#EPC/#PSR not allowed).

Use the appropriate GcCode symbol in the methods below to start, stop, or 
retrieve the session of that GcGem type.

-------------------------
Starting a special GcGem:
-------------------------

(Note:  GcGem will be started on the machine running the stone process)

  System startGC: <gcCode>
    * if gcCode is #GC, #RCL or #EPC, starts up that GcGem.
    * if gcCode is #PSR or #PDR, starts up GcGems of that type on all extents.

  System startGC: <gcCode> onExtent: <extentId>
    * if gcCode is #GC, #RCL or #EPC, starts up that GcGem (ignores extentId).
    * if gcCode is #PSR or #PDR, starts up that GcGem on selected extentId.

  System startGC: <gcCode> onExtents: <exentCollection>
    * if gcCode is #GC, #RCL or #EPC, starts up that GcGem 
      (ignores extentCollection).
    * if gcCode is #PSR or #PDR, starts up that GcGem on selected extents in 
      extentCollection.
    * if extentCollection is #ALL, starts up that GcGem on all extents.

Note: The startGC: methods return immediately while the system starts the
selected GcGem(s) in the background.  if you follow a startGC: call with 
a command that is sensitive to the presence/absence of GcGems (such as 
stopGC:, reclaimAll, objectAudit), you should wait a few seconds first 
to allow time for the system to start the selected GcGem(s).

The startGC: methods require that the user have the following privileges:

    #GarbageCollection
    #SessionAccess
    #SystemControl
  
-------------------------
Stopping a special GcGem:
-------------------------

  System stopGC: <gcCode>
    * if gcCode is #GC, #RCL or #EPC, stops that GcGem.
    * if gcCode is #PSR or #PDR, stops all GcGems of that type on all extents.
  
  System stopGC: <gcCode> onExtent: <extentId>
    * if gcCode is #GC, #RCL or #EPC, stops that GcGem (ignores extentId).
    * if gcCode is #PSR or #PDR, stops that GcGem on selected extentId.
  
  System stopGC: <gcCode> onExtents: <exentCollection>
    * if gcCode is #GC, #RCL or #EPC, stops that GcGem 
      (ignores extentCollection).
    * if gcCode is #PSR or #PDR, stops that GcGem on selected extents in 
      extentCollection.
    * if extentCollection is #ALL, stops all GcGems of that type on all 
      extents.

Note: The stopGC: methods return immediately while the system stops the
selected GcGem(s) in the background.  if you follow a stopGC: call with 
a command that is sensitive to the presence/absence of GcGems (such as 
startGC:, reclaimAll, objectAudit), you should wait a few seconds 
first to allow time for the system to stop the selected GcGem(s).

The stopGC: methods require that the user have the following privileges:

    #GarbageCollection
    #SessionAccess
    #SystemControl
  
-------------------
Session Information
-------------------

The following methods return the Session ID(s) of the selected GcGem(s),
or 0 if there is no GcGem of the selected type.

  System gcSession: <gcCode>
    * if gcCode is #GC, #RCL or #EPC, returns the session ID of that GcGem 
      ( 0 if none ).
    * if gcCode is #PSR or #PDR, returns an array of session ID''s 
      corresponding to each extent.
    * if gcCode is #ALL, returns an array of session ID''s in the 
      following order:
        1.  GcGem
        2.  RCL Gem
        3.  EPC Gem
        4.  An array of session ID''s, one for each PSR Gem on that extent.
        5.  An array of session ID''s, one for each PDR Gem on that extent.

  System gcSession: <gcCode> onExtent: <extentId>
    * if gcCode is #GC, #RCL or #EPC, returns the session ID of that GcGem 
      ( 0 if none ).  The extentId is ignored.
    * if gcCode is #PSR or #PDR, returns session ID for the GcGem on 
      selected extentId.

  System gcSession: <gcCode> onExtents: <extentArray>
    * if gcCode is #GC, #RCL or #EPC, returns the session ID of that GcGem 
      ( 0 if none ).  The extentId''s are ignored.
    * if gcCode is #PSR or #PDR, returns an array of session ID''s for 
      the GcGems running on the extents in extentArray.
    * if exentArray is #ALL, returns an array of session ID''s for all extents.

The gcSession: methods require that the user have the following privileges:

    #GarbageCollection
    #SessionAccess

-------------------------------
Automatic GC Session Management
-------------------------------

When the system configuration option STN_GC_SESSION_ENABLED = TRUE, or
the dynamic configuration option #GcSessionEnabled is set to 1, the stone
will automatically attempt to keep GcGems running, according to the value
set for the system configuration option STN_GC_SESSION_CONFIGURATION:

   1 : Keep a single GcGem running.
   2 : Keep a RCL/EPC Gem pair running. (in 6.1)

If a GcGem should die or be manually stopped, the stone will attempt to
restart that GcGem within 2 minutes, under the following conditions:

    For STN_GC_SESSION_CONFIGURATION = 1:

        * The GcGem will be restarted if no other GcGems are running.
 
    For STN_GC_SESSION_CONFIGURATION = 2:

        * The RCL GcGem will restarted if:
            * No other GcGems (other than EPC) are running

        * The EPC GcGem will be restarted if:
            * No other GcGems (other than RCL) are running

Note that for special GcGem configurations, you may need to set 
#GcSessionEnabled to 0 to prevent the stone from trying to start a GcGem
that is not appropriate for your special configuration.  Remember to 
return #GcSessionEnabled to 1 when you''re ready to resume automatic 
GC session management.

GcGem configuration can also be controlled dynamically by using the
stone configuration parameter #GcSessionConfiguration.  For example,
you can change to configuration 2 by executing:

    System stoneConfigurationAt: #GcSessionConfiguration put: 2.

Note that if any GcGems are currently running, this change will not 
immediately take effect.  If you want to immediately change the 
configuration, you should stop all currently running GcGems first.  
For example:

    System stopGC: #ALL.
    System stoneConfigurationAt: #GcSessionConfiguration put: 2.

Dynamic changes to #GcSessionConfiguration are not recorded in the
system configuration file.  You''ll need to edit the configuration file
and update the STN_GC_SESSION_CONFIGURATION parameter as appropriate
if you wish for this configuration change to be made permanent across
stone restarts.

---------------------------------
Example GC Configurations
---------------------------------

Light Load:

    Run with a single normal GcGem (STN_GC_SESSION_CONFIGURATION = 1).

Moderate Load:

    Run with a RCL/EPC GcGem pair (STN_GC_SESSION_CONFIGURATION = 2).

    You should have multiple processors to benefit from this configuration.

Heavy Load - high number of pages needing to be reclaimed:

    Run with a RCL/EPC GcGem pair (STN_GC_SESSION_CONFIGURATION = 2).
    During off-hours, shutdown the RCL/EPC GcGems and run PSR GcGems
    on the extents.  For example:

        System stopGC: #ALL.
        System sleep: 5.  "To give RCL/EPC gems time to logout"
        System startGC: #PSR onExtent: 1.
        System startGC: #PSR onExtent: 2.
        ....
        System startGC: #PSR onExtent: <max_extent_id>.

    You should have multiple processors and multiple extents to benefit
    from this configuration.

    Note that running with PSR GcGems on all extents may put too high a
    load on the system, and that it might be best to stagger the running
    of the PSR GcGems on just 2,3, or some smaller number of extents at
    any one time.  Some experimentation will be necessary to determine
    what works best on your architecture.

    When the number of pages needing to be reclaimed has returned back
    to normal, return to the RCL/EPC GcGem configuration by simply shutting
    down the PSR GcGems (the stone will restart the RCL/EPC GcGems within 2
    minutes).

Heavy Load - high number of dead objects needing to be reclaimed

    Run with a RCL/EPC GcGem pair (STN_GC_SESSION_CONFIGURATION = 2).
    During off-hours, shutdown the GcGems and start up PDR GcGems
    on the extents.  For example:

        System stopGC: #ALL.
        System sleep: 5.  "To give RCL/RCL gems time to logout"
        System startGC: #PDR onExtent: 1.
        System startGC: #PDR onExtent: 2.
        ....
        System startGC: #PDR onExtent: <max_extent_id>.

    You should have multiple processors and multiple extents to benefit
    from this configuration.

    Also remember that commits are not allowed while any PDR GcGems are
    running.  Other sessions may operate on the DB, but any commits
    attempted will fail.

    Note that running with PDR GcGems on all extents may put too high a
    load on the system, and that it might be best to stagger the running
    of the PDR GcGems on just 2,3, or some smaller number of extents at
    any one time.  Some experimentation will be necessary to determine
    what works best on your architecture.

    PDR GcGems will automatically shutdown when they have completed
    reclaiming all dead objects on their designated extent.  The stone
    will then automatically restart the RCL/EPC GcGems within 2 minutes 
    of the last PDR GcGem shutting down.

'
%
category: 'Garbage Collection'
classmethod: System
gcSession: gcCode

  "Return a SmallInteger, or an Array of SmallIntegers, which are the 
   session ID's for the selected GcGem code.
   * if gcCode is #GC #RCL or #EPC, returns the session ID (a SmallInteger) of 
     that GcGem (0 if none).
   * if gcCode is #PSR or #PDR, returns an Array SmallIntegers which are the 
     session ID's for all extents.
   * if gcCode is #ALL, returns an Array of SmallIntegers which are the 
     session ID's specified as follows:
        1.  GC Gem
        2.  RCL Gem
        3.  EPC Gem
        4.  An Array of SmallIntegers which are the session ID's for each 
            PSR Gem on that extent.
        5.  An Array of SmallIntegers which are the session ID's for each 
            PDR Gem on that extent.  "

| sessions index result item4 item5 numExtents |

"Get the raw session information"
sessions := self _gcSessions.

"Check for #ALL.. construct and return result"
( gcCode = #ALL ) ifTrue: [
  numExtents := SystemRepository numberOfExtents.
  result := Array new: 5.
  result at: 1 put: ( sessions at: 1 ).
  result at: 2 put: ( sessions at: 2 ).
  result at: 3 put: ( sessions at: 3 ).
  item4 := Array new: numExtents.
  item5 := Array new: numExtents.
  index := 4.
  1 to: numExtents do: [ :i | 
  item4 at: i put: ( sessions at: index ).
  index := index + 1 ].
  1 to: numExtents do: [ :i | 
    item5 at: i put: ( sessions at: index ).
    index := index + 1 ].
  result at: 4 put: item4.
  result at: 5 put: item5.
  ^result ].

" Check for the Parallel Gems and pass on to onExtent version.. "
( #( PSR PDR ParallelShadowReclaim ParallelDeadReclaim ) includes: gcCode ) 
  ifTrue: [
    ^self gcSession: gcCode onExtents: #ALL ].

" Handle the basic case "
( #( GC RCL EPC GcGem ReclaimGem EpochGem ) includes: gcCode ) 
  ifTrue: [
    ^ sessions at: ( self _gcSessionIndexFor: ( self _gcType: gcCode ) 
      extentId: -1 ) ].

" We shouldn't have gotten here.. "
System signal: (ErrorSymbols at: #rtErrBadGcType)
  args: #[ gcCode ] 
  signalDictionary: GemStoneError
%
category: 'Garbage Collection'
classmethod: System
gcSession: gcCode onExtent: extentId

"Return a SmallInteger that is the session ID for the selected GcGem code
and extentId.

  * if gcCode is #GC #RCL or #EPC, returns the session ID of that GcGem 
    ( 0 if none ).  The extentId is ignored.
  * if gcCode is #PSR or #PDR, returns session ID for the GcGem on 
    selected extentId. "

^ self _gcSessions at: ( self _gcSessionIndexFor: ( self _gcType: gcCode ) 
  extentId: extentId )
%
category: 'Garbage Collection'
classmethod: System
gcSession: gcCode onExtents: extentArray

"Return a SmallInteger that is the session ID for the selected GcGem code 
and extentId's held by the extentArray.

  * if gcCode is #GC #RCL or #EPC, returns the session ID of that GcGem 
    ( 0 if none ).  The extentId's are ignored.
  * if gcCode is #PSR or #PDR, returns an Array of session ID's 
    for the GcGems corresponding to the extents in extentArray.
  * if exentArray is #ALL, returns an array of session ID's for all extents. "

| sessions gcType extents numExtents result |
( #( GC RCL EPC GcGem ReclaimGem EpochGem ) includes: gcCode ) 
    ifTrue: [ ^ self gcSession: gcCode ].
extents := extentArray.
( extentArray == #ALL ) ifTrue: [  
    numExtents := SystemRepository numberOfExtents.
    extents := Array new: numExtents.
    1 to: numExtents do: [ :i | extents at: i put: i ] ].
sessions := self _gcSessions.
gcType := self _gcType: gcCode.
result := Array new: extents size.
1 to: extents size do: [ :i |
  result at: i put: ( sessions at: ( self _gcSessionIndexFor: gcType 
  extentId: ( extents at: i ) ) ) ].
^ result
%
category: 'Configuration File Access'
classmethod: System
gemConfigurationAt: aName

"Returns the value of the specified configuration file parameter from the
 current session.  Returns nil if that parameter is not applicable to a Gem."

| cfgId |
cfgId := ConfigurationParameterDict at: aName otherwise: nil .
cfgId == nil ifTrue:[ ^ nil ].
^ self _configurationAt: cfgId isStone: false kind: $C
%
category: 'Runtime Configuration Access'
classmethod: System
gemConfigurationAt: aName put: aValue

"Changes the value of the specified Gem configuration parameter.  

 See comments in the method configurationAt:put: for complete documentation."

^ self configurationAt: aName put: aValue
%
category: 'Configuration File Access'
classmethod: System
gemConfigurationReport

"Returns a SymbolDictionary whose keys are the names of configuration file
 parameters, and whose values are the current settings of those parameters in
 the current session's Gem process.  Parameters that are not applicable to Gem
 and those that are undefined are not included in the result."

^ self _configurationReport: false 
%
category: 'Environment Access'
classmethod: System
gemEnvironmentVariable: varName

"Expands the environment variable named varName in the Gem
 process, returning a String. varName should be a kind of String.

 Returns nil if any of the following are true:

 * varName is not a byte format object.
 * There is no environment variable defined with name varName.
 * The value of the environment variable is more than approximately 8000 bytes.
 * The size of varName exceeds approximately 8000 bytes."

^ GsFile _expandEnvVariable: varName isClient: false
%
category: 'Backward Compatibility'
classmethod: System
gemStatistics

"Obsolete in GemStone 4.1.  Use the pageReads and pageWrites methods instead
 for I/O statistics."

"Returns a String giving statistics for the Gem (GemStone session) process,
 including such information as elapsed real time, elapsed CPU time, and I/O
 statistics."

^ self _zeroArgCateg2Prim: 1
%
category: 'Version Management'
classmethod: System
gemVersionAt: aSymbol

"Returns information about the Gem process of the current session. 
 aSymbol must be equal to a key in VersionParameterDict, otherwise nil
 is returned.  The semantics of these keys are:

 aSymbol              meaning
 -------              -------
 #cpuKind             detailed CPU type obtained at runtime: 'sun4m', '486'.
 #cpuArchitecture     target CPU for which GemStone was compiled:
                      'SPARC', 'X86'.
 #gsBuildArchitecture operating system name and CPU for which GemStone 
                      was compiled: 'SunOS SPARC', 'NT Intel'.
 #gsBuildDate         time at which the Gem executable was compiled (a String).
 #gsRelease           major and minor version of GemStone, such as '5.0.0'.
 #gsVersion           major version of GemStone, such as '5.0'.
 #imageKind           a Symbol: #server.
 #nodeName            network node name: 'speedy'.
 #osName              operating system name: 'SunOS', 'NT'.
 #osRelease           release number of the operating system: '4.1.3', '3.5'.
 #osVersion           vendor defined major version of the OS: '3'.
 #processId           operating system process identifier (an Integer): 13529.
 #processorCount      number of processors on the machine running the process."

| verId |
aSymbol = #imageKind ifTrue:[ ^ self imageVersionAt: aSymbol ] .
verId := VersionParameterDict at: aSymbol ifAbsent:[ ^ nil ].

^ self _configurationAt: verId isStone: false kind: $V 
%
category: 'Version Management'
classmethod: System
gemVersionReport

"Return a SymbolDictionary whose keys are the names of operating system,
 hardware, or GemStone version attributes, and whose values are the
 current values of those attributes in the Gem process."

^ self _serverVersionReport: false
%
category: 'Error Handling'
classmethod: System
genericSignal: errIdentifier text: aString

"Raise a user-defined signal with no arguments.

 The argument errIdentifier is a user-defined object, to distinguish user
 errors, and may be nil.  The argument aString appears in GemStone's error
 message for this error, and may be nil."

self genericSignal: errIdentifier text: aString args: #[]
%
category: 'Error Handling'
classmethod: System
genericSignal: errIdentifier text: aString arg: anArg

"Raise a user-defined signal with one argument.

 The argument errIdentifier is a user-defined object, to distinguish user
 errors, and may be nil.  The argument aString appears in GemStone's error
 message for this error, and may be nil.  The argument anArg appears as the
 third argument to the error."

| errorNumber allArgs |
allArgs := #[ errIdentifier, aString, anArg ] .
errorNumber := self _errorSymbolToNumber: #genericError args: allArgs .

self signal: errorNumber args: allArgs signalDictionary: GemStoneError
%
category: 'Error Handling'
classmethod: System
genericSignal: errIdentifier text: aString args: anArray

"Raise a user-defined signal.

 The argument errIdentifier is a user-defined object, to distinguish user
 errors, and may be nil.  The argument aString appears in GemStone's error
 message for this error, and may be nil.  The argument anArray appears as the
 third argument to the error."

| errorNumber allArgs |
allArgs := #[ errIdentifier, aString ] .
allArgs addAll: anArray .
errorNumber := self _errorSymbolToNumber: #genericError args: allArgs .

self signal: errorNumber args: allArgs signalDictionary: GemStoneError
%
category: 'User-Defined Actions'
classmethod: System
hasUserAction: aSymbol

"Returns true if the user action named aSymbol is installed in this GemStone
 session.  Returns false otherwise."

<primitive: 105>
aSymbol _validateClass: String.
^ self _primitiveFailed: #hasUserAction:
%
category: 'Hidden Set Support'
classmethod: System
HiddenSetSpecifiers

"Returns a list of the hiddenSet specifiers."

^#(
" 1"   'ExportedObjs'
" 2"   'NoRollbackObjs'
" 3"   'NotConnectedObjs'
" 4"   'NewPomObjs'
" 5"   'DeadObjs'
" 6"   'ClusteredOops'
" 7"   'RcReadSet'
" 8"   'ReadSet'
" 9"   'WriteSet'
"10"   'SaveReadSetUnion'
"11"   'SaveWriteSetUnion'
"12"   'SaveWrittenObjs'
"13"   'ReadWriteConflicts'
"14"   'WriteReadConflicts'
"15"   'WriteWriteconflicts'
"16"   'ReadExclLockConflicts'
"17"   'WriteReadLockConflicts'
"18"   'WriteWriteLockConflicts'
"19"   'RefPomObjs'
"20"   'AllocatedGciOops'
"21"   'PlaceHolderObjs'
"22"   'GciDirtyObjs'
"23"   'NscsWithNewIdxs'
"24"   'NewObjsWithDepTags'
"25"   'NotifySet'
"26"   'Reserved1'
"27"   'Reserved2'
"28"   'Reserved3'
"29"   'CommitReleaseLocksSet'
"30"   'CommitOrAbortReleaseLocksSet'
"31"   'ObjsCreated'
"32"   'ObjsDisposed'
"33"   'NewSymbolSet'
"34"   'GcCandidates'
"35"   'ObjsWithWeakRefs'
"36"   'WriteLockWriteSubset'
"37"   'NewDataPages'
"38"   'StrongReadSet'
"39"   'PureExportSet'
"40"   'TrackedObjects'
"41"   'Reserved for AllSymbols rebuilds.'
)
%
category: 'Version Management'
classmethod: System
imageVersionAt: aSymbol

"Returns information about the GemStone kernel class image where aSymbol is one
 of the following:

 #gsBuildDate  DateTime of last kernel class filein or upgrade.
 #gsRelease    Version String of last kernel class filein or upgrade: '5.0.0'.
 #gsVersion    Major version of image, such as '5.0' .
 #imageKind    A Symbol: #server.

 If aSymbol is not one of the above, returns nil."

^ ImageVersion at: aSymbol otherwise: nil
%
category: 'Transaction Control'
classmethod: System
inTransaction

"Returns true to indicate that the session is in a transaction, false
 otherwise."

^ self _zeroArgPrim: 5
%
category: 'User-Defined Actions'
classmethod: System
loadUserActionLibrary: aString

"Loads the session user action library specified by aString.
 This method always returns the receiver (System)."

<primitive: 426>
^ self _primitiveFailed: #loadUserActionLibrary:
%
category: 'Lock Status'
classmethod: System
lockKind: anObject

"Returns a Symbol (#none, #read, #write, or #exclusive) representing the kind
 of lock held on anObject by any session in the system."

| anArray |

anArray := System lockStatus: anObject.
^anArray at: 1.
%
category: 'Lock Status'
classmethod: System
lockOwners: anObject

"Returns an Array of session numbers (SmallIntegers) representing the sessions
 that hold a lock on anObject.  If the object is not locked by any session, the
 result Array is empty.  Note that a write or exclusive lock can have only one
 owner."

| anArray |

anArray := System lockStatus: anObject.
^anArray at: 2.
%
category: 'Lock Status'
classmethod: System
lockStatus: anObject

"Returns a two-element Array, where the first element is a Symbol representing
 the kind of lock held on anObject (#none, #read, #write, or #exclusive) and
 the second element is an Array of session numbers (SmallIntegers) representing
 the sessions that hold the lock.

 If there are no locks on anObject, the first element is the Symbol #none and
 the second element is an empty Array.

 Only locks on permanent objects are reported."

<primitive: 351>
self _primitiveFailed: #lockStatus: .
self _uncontinuableError
%
category: 'Clustering'
classmethod: System
maxClusterBucket

"Returns the maximum legal clusterId as a SmallInteger."

^ AllClusterBuckets size
%
category: 'Session Control'
classmethod: System
maxSessionId

"Returns a SmallInteger representing the maximum number of sessions
 allowed on the system based upon the Stone configuration parameter."

^ self _zeroArgPrim: 6
%
category: 'Performance Monitoring'
classmethod: System
millisecondsToRun: aBlock

"Returns the number of CPU milliseconds used while evaluating aBlock.
 The argument aBlock must be a zero-argument block."

| startTime endTime |

startTime := self _readClock.
aBlock value.
endTime := self _readClock.
(endTime >= startTime) ifTrue: [
  ^ endTime - startTime
] ifFalse: [
  ^ endTime + (16rffffffff - startTime)
]
%
category: 'Performance Monitoring'
classmethod: System
myCacheProcessSlot

"Returns the process slot in the SharedPageCache that corresponds
 to my process.  If the SharedPageCache is not in use, returns -1."

^ self _zeroArgPrim: 7
%
category: 'Lock Status'
classmethod: System
myLockKind: anObject

"Returns a Symbol that indicates what kind of lock the current session has on
 anObject: one of #none, #read, #write, or #exclusive."

| status |
status := self lockStatus: anObject.
( (status at: 1) ~~ #none _and:
[ (status at: 2) includesIdentical: System session] )
  ifTrue: [ ^status at: 1 ].

^#none
%
category: 'Backward Compatibility'
classmethod: System
myUserGlobals

"Obsolete in GemStone 5.0."

"Returns the current user's UserGlobals dictionary.  If the user does not have
 one, an error is flagged."

| assn |

(assn := self myUserProfile resolveSymbol: #UserGlobals) == nil ifTrue: [
  ^self _error: #rtErrKeyNotFound args: #(#UserGlobals)
].
^assn value.
%
category: 'Session Control'
classmethod: System
myUserProfile

"Returns the UserProfile of the current session."

"Implemented as a primitive to handle image bootstrap problems."

<primitive: 318>
^ self _primitiveFailed: #myUserProfile
%
category: 'Transaction Control'
classmethod: System
needsCommit

"Returns true if the session has made changes to the repository.  Returns false
 otherwise."

^ self _needsCommit
%
category: 'Instance Creation'
classmethod: System
new

"Disallowed.  You may not create new instances of System."

self shouldNotImplement: #new
%
category: 'Notification'
classmethod: System
notifySet

"Returns an Array of the objects that the user has registered for notification
 when a new state is committed."

^self _hiddenSetAsArray: 25
%
category: 'Performance Monitoring'
classmethod: System
pageReads

"Returns the number of Repository page read operations performed since the start
 of the Gem process.  If the Gem is remote, this corresponds to reads performed
 in the current session.  If the Gem is linked, it corresponds to reads
 performed since the application was invoked."

^ self _zeroArgPrim: 2
%
category: 'Performance Monitoring'
classmethod: System
pageWrites

"Returns the number of Repository page write operations performed since the
 start of the Gem process.  If the Gem is remote, this corresponds to writes
 performed in the current session.  If the Gem is linked, it corresponds to
 writes performed since the application was invoked."

^ self _zeroArgPrim: 3
%
category: 'Host System Access'
classmethod: System
performOnServer: aString

"This method causes the operating system commands in aString to be executed as
 a spawned subprocess.  Generates an error if aString cannot be executed by the
 operating system.

 Under Unix, commands in aString can have exactly the same form as a shell
 script.  For example, newlines or semicolons can separate commands, and a
 backslash can be used as an escape Character."

<primitive: 347>
aString _validateClass: String.
^ aString _error: #hostErrPerform args:  #() 
%
category: 'Reduced Conflict Support'
classmethod: System
rcValueCache

"Returns the cache dictionary that is stored in temporary session state
 used to hold calculated values for reduced conflict classes.  If it does
 not exist, create it."

| dict |
dict := self _rcValueCache.
dict == nil
    ifTrue: [
        dict := IdentityKeyValueDictionary new: 47.
        self _sessionStateAt: 5 put: dict
    ].
^ dict
%
category: 'Reduced Conflict Support'
classmethod: System
rcValueCacheAt: aKey for: anObject ifAbsent: aBlock

"Returns the associated value at the given key for anObject.
 If the key is not present, execute the zero-argument block."

| valueArray cache |
cache := self _rcValueCache.
" if the cache does not exist, then the key is not present "
cache == nil
  ifTrue: [ ^ aBlock value ].

valueArray := cache at: anObject ifAbsent: [ ^ aBlock value ].

1 to: valueArray size by: 2 do: [ :i |
    aKey == (valueArray at: i)
        ifTrue: [ ^ valueArray at: (i + 1) ]
].

^ aBlock value
%
category: 'Reduced Conflict Support'
classmethod: System
rcValueCacheAt: aKey for: anObject otherwise: aValue

"Returns the associated value at the given key for anObject.
 If the key is not present, returns aValue."

| valueArray cache |
cache := self _rcValueCache.
" if the cache does not exist, then the key is not present "
cache == nil
  ifTrue: [ ^ aValue ].

valueArray := cache at: anObject otherwise: nil.

valueArray == nil
    ifTrue: [ ^ aValue ].

1 to: valueArray size by: 2 do: [ :i |
    aKey == (valueArray at: i)
        ifTrue: [ ^ valueArray at: (i + 1) ]
].

^ aValue
%
category: 'Reduced Conflict Support'
classmethod: System
rcValueCacheAt: aKey otherwise: aValue

"Returns the associated value at the given key for anObject.
 If the key is not present, returns aValue."

| cache |
cache := self _rcValueCache.
" if the cache does not exist, then the key is not present "
cache == nil
  ifTrue: [ ^ aValue ].

^ cache at: aKey otherwise: aValue.
%
category: 'Reduced Conflict Support'
classmethod: System
rcValueCacheAt: aKey put: aValue for: anObject

"Adds the given key/value pair for anObject.  Returns the receiver."

| valueArray |
valueArray := self rcValueCache at: anObject otherwise: nil.
valueArray == nil
  ifTrue: [
    valueArray := Array new.
    self rcValueCache at: anObject put: valueArray.
  ].

1 to: valueArray size by: 2 do: [ :i |
    aKey == (valueArray at: i)
        ifTrue: [
            valueArray at: (i + 1) put: aValue.
            ^ self
        ]
].

"add a new entry"
valueArray addLast: aKey; addLast: aValue
%
category: 'Setting Locks'
classmethod: System
readLock: anObject

"Analogous to System | exclusiveLock:ifDenied:ifChanged:, but these methods
 differ in these respects:

 * This method requests and grants read locks.

 * This method grants a read lock on anObject if another session already holds
   a read lock, but grants no lock if another session already holds an
   exclusive or a write lock to anObject.

 * This method requires only read authorization for the object, not write."

| result |
result := System _lock: anObject kind: #read.
(result == 1)
  ifTrue: [^self].
self _lockError: result obj: anObject args: #[#read]
%
category: 'Setting Locks'
classmethod: System
readLock: anObject ifDenied: denyBlock ifChanged: changeBlock

"Analogous to System | exclusiveLock:ifDenied:ifChanged:, but these methods
 differ in these respects:

 * This method requests and grants read locks.

 * This method grants a read lock on anObject if another session already holds
   a read lock, but grants no lock if another session already holds an
   exclusive or a write lock to anObject.

 * This method requires only read authorization for the object, not write."

| result |

result := System _lock: anObject kind: #read.
(result == 1) ifTrue:
   [^self].
^ self _lockEvaluateErr: result denied: denyBlock changed: changeBlock
%
category: 'Backward Compatibility'
classmethod: System
readLock: anObject ifDenied: denyBlock ifChanged: changeBlock
  ifNotCommitted: notCommittedBlock

"Obsolete in GemStone 4.0.  Locks are now allowed on objects that have not been
 committed."

^self readLock: anObject ifDenied: denyBlock ifChanged: changeBlock
%
category: 'Setting Locks'
classmethod: System
readLockAll: aCollection

"Analogous to System | exclusiveLockAll:, but these methods differ in these
 respects:

 * This method requests and grants read locks.

 * This method denies a read lock to the object if another session already
   holds an exclusive or a write lock to it.  However, it grants a read lock
   if another session already holds a read lock.

 * This method requires only read authorization for the object, not write."

| result |
result := System _lockAll: aCollection kind: #read.
(result == System)
  ifTrue: [^self]
  ifFalse: " else we generate the error lock incomplete "
           [^ (result at: 1) _error: #lockErrIncomplete
                             args: #[(result at:2), (result at:3)]].
%
category: 'Setting Locks'
classmethod: System
readLockAll: aCollection ifIncomplete: incompleteBlock

"Analogous to System | exclusiveLockAll:ifIncomplete:, but these methods
 differ in these respects:

 * This method requests and grants read locks.

 * This method denies a read lock to the object if another session already
   holds an exclusive or a write lock to it.  However, it grants a read lock
   if another session already holds a read lock.

 * This method requires only read authorization for the object, not write."

| result |
result := System _lockAll: aCollection kind: #read.
(result == System)
  ifTrue: [^self]
  ifFalse: " else we execute the incomplete block with 3 arguments "
          [^incompleteBlock value: (result at: 1)
                            value: (result at:2)
                            value: (result at:3)].
%
category: 'Backward Compatibility'
classmethod: System
readLockObjAndIndexes: anObject

"Obsolete in GemStone 4.0.  Indexes now have reduced conflict behavior and
 locks are now allowed on objects that have not been committed."

^ self readLock: anObject
%
category: 'Backward Compatibility'
classmethod: System
readLockObjAndIndexes: anObject ifDenied: denyBlock ifChanged: changeBlock
  ifNotCommitted: notCommittedBlock

"Obsolete in GemStone 4.0.  Indexes now have reduced conflict behavior and
 locks are now allowed on objects that have not been committed."

^ self readLock: anObject ifDenied: denyBlock ifChanged: changeBlock
%
category: 'Reduced Conflict Support'
classmethod: System
redoLog

"Returns the redo log that is stored in the temporary session state.
 Create it if it does not exist."

| redoLog |
redoLog := self _sessionStateAt: 4.
redoLog == nil
    ifTrue: [
        redoLog := RedoLog new.
        System _sessionStateAt: 4 put: redoLog
    ].
^ redoLog
%
category: 'Releasing Locks'
classmethod: System
removeAllFromCommitOrAbortReleaseLocksSet: aCollection

"Remove all the elements of aCollection from the commit-or-abort release
 locks set.  If an element of aCollection is not a member of the set,
 it is ignored."

aCollection do: [:each | self removeFromCommitOrAbortReleaseLocksSet: each ].
^self
%
category: 'Releasing Locks'
classmethod: System
removeAllFromCommitReleaseLocksSet: aCollection

"Remove all elements of aCollection from the commit release locks set.
 If an element of aCollection is not a member of the set, it is ignored."

aCollection do: [:each | self removeFromCommitReleaseLocksSet: each ].
^self
%
category: 'Notification'
classmethod: System
removeAllFromNotifySet: aCollection

"Removes all elements of aCollection from the notify set.  Does not generate an
 error if any of the elements are not in the notify set."

self _updateNotifySet: aCollection add: false
%
category: 'Releasing Locks'
classmethod: System
removeFromCommitOrAbortReleaseLocksSet: anObject

"Remove anObject from the commit-or-abort release locks set.  If anObject
 is not a member of the set, do nothing."

self _remove: anObject from: 30.
%
category: 'Releasing Locks'
classmethod: System
removeFromCommitReleaseLocksSet: anObject

"Remove anObject from the commit release locks set.  If anObject is 
 not a member of the set, do nothing."

self _remove: anObject from: 29.
%
category: 'Notification'
classmethod: System
removeFromNotifySet: anObject

"Removes anObject from the notify set.  Does not generate an error if anObject
 is not in the notify set."

self _updateNotifySet: #[ anObject ] add: false
%
category: 'Removing Locks'
classmethod: System
removeLock: anObject

"Removes the lock held by the current session on anObject.
 This method succeeds even if you do not have read authorization
 for anObject.  Returns the receiver."

self removeLockAll: #[ anObject ].
^self
%
category: 'Removing Locks'
classmethod: System
removeLockAll: aCollection

"Removes all locks held by the current session on the objects in aCollection.
 If an object in aCollection is not locked by the current session, that
 object is ignored. Returns the receiver."

<primitive: 99>
| anArray |
anArray := aCollection asArray.
anArray == aCollection ifFalse:[
  ^ self removeLockAll: anArray .
  ] .
aCollection _validateClass: Collection.
self _primitiveFailed: #removeLockAll: .
self _uncontinuableError
%
category: 'Backward Compatibility'
classmethod: System
removeLockAllNoErr: aCollection

"Obsolete in GemStone 4.0.  Use the removeLockAll: method instead."

^ self removeLockAll: aCollection
%
category: 'Backward Compatibility'
classmethod: System
removeLockNoErr: anObject

"Obsolete in GemStone 4.0.  Use the removeLockAll: method instead."

^ self removeLockAll: #[ anObject ]
%
category: 'Removing Locks'
classmethod: System
removeLocksForSession

"Removes all locks held by this session.  Returns the receiver.
 This method succeeds even if the session no longer has
 read authorization for one or more of its locked objects."

^ self _zeroArgPrim: 0
%
category: 'System Control'
classmethod: System
resumeLogins

"Allows new sessions to be initiated.  (Enables users to login.)  Logins are
 enabled when the GemStone system is started.  This message reverses the effect
 of System | suspendLogins.  Requires the SystemControl privilege."

self configurationAt: #StnLoginsSuspended put: 0
%
category: 'System Control'
classmethod: System
sendSigAbortToSession: sessionIdOrSerial

"Sends a sigAbort signal to the specified session.
 Has no effect if the session is in a transaction.
 To execute this method, you must have the SystemControl privilege.

 The sessionIdOrSerial argument is a SmallInteger specifying a session.  A
 positive value is interpreted as a serialNumber.  A negative value is
 interpreted as a negated sessionId."

<primitive: 327>
sessionIdOrSerial _validateClasses:#[SmallInteger].
self _primitiveFailed: #sendSigAbortToSession:
%

category: 'Private'
classmethod: System
_sendSigLostOtToSession: sessionId

"Sends a SigLostOtRoot signal to the specified session.
 Has no effect if the session is in a transaction.
 To execute this method, you must have the SystemControl privilege."

^self _stopSession: sessionId negated
%

category: 'Signals'
classmethod: System
sendSignal: aSignal to: aSessionId withMessage: aString

"Sends a signal (a SmallInteger) to the specified session (a SmallInteger) with
 aString as a message.  The aString argument is currently limited to 1023
 bytes."

aSessionId < 1 ifTrue:[ aSessionId _error: #rtErrArgNotPositive  ].
self _sendSignal: aSignal toSess: aSessionId negated withMessage: aString
%
category: 'Session Control'
classmethod: System
session

"Returns a SmallInteger representing the session of the sender."

^self _zeroArgPrim: 33
%
category: 'Lock Status'
classmethod: System
sessionLocks

"Returns a three-element Array describing the locks held by the current
 session.  The first element is an Array of all read-locked objects, the second
 is an Array of all write-locked objects, and the third is an Array of all
 exclusive-locked objects.  If the current session holds no locks of a
 particular kind (read, write, exclusive), then the corresponding Array is
 empty.  If the current session holds no locks at all, then all three of these
 Arrays are empty."

<primitive: 352>
self _primitiveFailed: #sessionLocks .
self _uncontinuableError
%
category: 'Environment Access'
classmethod: System
sessionPerformingBackup

"Returns the session ID of the session that is performing a backup.
 If there is no such session, returns -1."

^ self configurationAt: #SessionInBackup
%
category: 'Session Control'
classmethod: System
sessionsReferencingOldestCr

"Returns an Array containing the sessionIds of the sessions that are
 currently referencing the oldest commit record.  Because a session can update
 its commit record without committing a transaction, it is possible that no
 session actually references the oldest commit record.  Therefore, this method
 may return an empty Array."
       
^self _zeroArgPrim: 21
%
category: 'System Control'
classmethod: System
shutDown

"Aborts all current sessions, then terminates them.  Finally, the GemStone
 system is shut down.  The session issuing this message terminates with a
 broken connection.  Requires the SystemControl privilege."

^ self _zeroArgPrim: 8
%
category: 'Error Handling'
classmethod: System
signal: anInteger args: anArray signalDictionary: anErrorDict

"This method generates the specified signal (or error), along with its
 associated arguments.  If an Exception is available to field the error, the
 Exception is invoked, otherwise returns control to the
 controlling GemBuilder for C (GCI) interface."

"GciContinueWith() depends upon this implementation and selector."

| ex |
ex := Exception _exceptionFor: anInteger signalDictionary: anErrorDict .
ex ~~ nil ifTrue:[
  ^ ex _signal: anErrorDict number: anInteger args: anArray .
  ] .
^ self _signalGciError: anInteger args: anArray signalDictionary: anErrorDict
%
category: 'Transaction Control'
classmethod: System
signaledAbortErrorStatus

"Returns true to indicate that the system generates an error when it receives
 the abort signal from Stone.  (In other words, verify that
 enableSignaledAbortError has been called to activate detection of the
 RT_ERR_SIGNAL_ABORT signal.)  Returns false otherwise."

^self _signalErrorStatus: 2
%
category: 'Notification'
classmethod: System
signaledGemStoneSessionErrorStatus

"Returns true to indicate that the current GemStone session can receive signals
 from other GemStone sessions.  Returns false otherwise."

^self _signalErrorStatus: 3
%
category: 'Notification'
classmethod: System
signaledObjects

"Returns an Array containing the objects that have been signaled since the last
 time this method was executed.  The elements in the Array are a subset of the
 notify set.  Clear the set of signaled objects."

<primitive: 365>
self _primitiveFailed: #signaledObjects .
self _uncontinuableError
%
category: 'Notification'
classmethod: System
signaledObjectsErrorStatus

"Returns true to indicate that the system generates errors when objects are
 added to the signaled objects set.  Returns false otherwise."

^self _signalErrorStatus: 1
%
category: 'Backward Compatibility'
classmethod: System
signalFromGemStoneSession

"Obsolete in GemStone 5.0."

"Returns a three-element Array containing information about a signal from
 another GemStone session:

 1.  An instance of GsSession representing the session that sent the signal.
 2.  The signal value (a SmallInteger).
 3.  A signal message (a String).

 If there is no signal in the queue, returns an empty Array."

| result |
result := self _signalFromGemStoneSession .
result == nil ifTrue:[ ^ #[] ].
result at: 1 put: (GsSession sessionWithSerialNumber: (result at: 1)).
^ result
%
category: 'Session Control'
classmethod: System
sleep: aTime

"Sleep for aTime seconds.  aTime must be a positive SmallInteger.  
 If aTime is zero, this method has no effect.
 
 This method is not currently interruptible."

<primitive: 370>
aTime _validateClass: SmallInteger .
aTime < 0 ifTrue: [ aTime _error: #rtErrArgOutOfRange ] .
^ self _primitiveFailed: #sleep:
%
category: 'Debugging Support'
classmethod: System
stackDepth

"Returns current depth of the GemStone Smalltalk stack."

^ self _zeroArgPrim: 26
%
category: 'Debugging Support'
classmethod: System
stackDepthHighwater

"Returns largest depth of the GemStone Smalltalk stack since session login."

^ self _zeroArgPrim: 27
%
category: 'Debugging Support'
classmethod: System
stackLimit

"Returns the approximate limit on the depth of the GemStone Smalltalk stack.
 The stack size is determined by the configuration file parameter
 GEM_MAX_SMALLTALK_STACK_DEPTH."

^ self _zeroArgPrim: 1
%
category: 'Debugging Support'
classmethod: System
stackLimit: anInteger

"Obsolete. Has no effect in GemStone 5.0.  Provided for compatibility."

anInteger _validateClass: SmallInteger .
(anInteger < 0) ifTrue: [ anInteger _error: #rtErrArgNotPositive ].
^ self
%
category: 'System Control'
classmethod: System
startFreeListPageServer

"Start an additional free list page server on this gem's shared cache.
 Returns true if successful, or a string describing an error if
 unsuccessful.  Requires the System Control privilege."

^ self _zeroArgPrim: 53
%
category: 'Garbage Collection'
classmethod: System
startGC: gcCode

"Start up the selected type of GcGem(s) (a Symbol).  If the gcCode is 
#PSR or #PDR, start up GcGems of that type on all extents"

( #( PSR PDR ParallelShadowReclaim ParallelDeadReclaim ) includes: gcCode )
   ifTrue: [ self startGC: gcCode onExtents: #ALL ]
   ifFalse: [ 
     self _checkGcPrivileges.
     self _startGcSession: ( self _gcType: gcCode ) extentId: -1 ].
^ self
%
category: 'Garbage Collection'
classmethod: System
startGC: gcCode onExtent: extentId

"Start up the selected type of GcGem(s) (a Symbol) on the selected extentId.  
If the gcCode is #GC #RCL or #EPC, just start it up. "

( #( GC RCL EPC GcGem ReclaimGem EpochGem ) includes: gcCode ) 
  ifTrue: [ self startGC: gcCode ]
  ifFalse: [ 
    self _checkGcPrivileges.
    ( ( extentId < 1 ) _or: 
      [ extentId > SystemRepository numberOfExtents ] ) ifTrue: [
        System signal: ( ErrorSymbols at: #rtErrBadExtentId )
        args: #[ extentId ] 
        signalDictionary: GemStoneError ].
    self _startGcSession: ( self _gcType: gcCode ) extentId: extentId ].
^ self
%
category: 'Garbage Collection'
classmethod: System
startGC: gcCode onExtents: extentCollection

"Start up the selected type of GcGem(s) (a Symbol) on each element of 
extentCollection, a Collection of SmallIntegers that are extentIds.
If the gcCode is #GC #RCL or #EPC, just start it up (extentCollection 
is ignored). "

| extents numExtents |
( #( GC RCL EPC GcGem ReclaimGem EpochGem ) includes: gcCode ) 
  ifTrue: [ self startGC: gcCode ]
  ifFalse: [ 
    (extentCollection == #ALL) 
      ifTrue: [ 
        numExtents := SystemRepository numberOfExtents.
        extents := Array new: numExtents.
        1 to: numExtents do: [ :i | extents at: i put: i ] ]
      ifFalse: [ extents := extentCollection ].
    extents do: [ :extent | self startGC: gcCode onExtent: extent ] ].
^ self
%
category: 'Configuration File Access'
classmethod: System
stoneConfigurationAt: aName

"Returns the value of the specified configuration file parameter from the
 repository monitor process (Stone).  Returns nil if that parameter is not
 applicable to the Stone."

| cfgId |
cfgId := ConfigurationParameterDict at: aName otherwise: nil .
cfgId == nil ifTrue:[ ^ nil ].
^ self _configurationAt: cfgId isStone: true kind: $C
%
category: 'Configuration File Access'
classmethod: System
stoneConfigurationAt: aName put: aValue

"Changes the value of the specified Stone configuration parameter.

 See comments in the method configurationAt:put: for complete documentation."

^ self configurationAt: aName put: aValue
%
category: 'Configuration File Access'
classmethod: System
stoneConfigurationReport

"Returns a SymbolDictionary whose keys are the names of configuration file
 parameters, and whose values are the current settings of those parameters in
 the repository monitor process (Stone).  Parameters that are not applicable to
 Stone and those that are undefined are not include in the result." 

^ self _configurationReport: true 
%
category: 'Environment Access'
classmethod: System
stoneName

"Returns a Symbol whose value is the full network name of the
 Stone to which this session is logged in."

^ self _zeroArgPrim: 11
%
category: 'Backward Compatibility'
classmethod: System
stoneStatistics

"Obsolete in GemStone 4.1.  Use the pageReads and pageWrites methods instead
 for I/O statistics."

"Returns a String giving statistics for the Stone (repository monitor) process,
 including such information as elapsed real time, elapsed CPU time, and I/O
 statistics.

 To execute this method, you must have explicit privilege from your
 system data curator."

^ self _zeroArgCateg2Prim: 2
%
category: 'Version Management'
classmethod: System
stoneVersionAt: aSymbol

"Returns version information about the Stone (repository monitor) process.

 See System (C) | gemVersionAt: for further details.
 aSymbol = #imageKind returns nil for the Stone."

| verId |

aSymbol = #imageKind ifTrue:[ ^ nil "image not applicable to Stone"].

verId := VersionParameterDict at: aSymbol ifAbsent:[ ^ nil ].
  
^ self _configurationAt: verId isStone: true kind: $V 
%
category: 'Version Management'
classmethod: System
stoneVersionReport

"Return a SymbolDictionary whose keys are the names of operating system,
 hardware, or GemStone version attributes, and whose values are the
 current values of those attributes in the Gem process."

^ self _serverVersionReport: true
%
category: 'Garbage Collection'
classmethod: System
stopGC: gcCode

"Stop the selected type of GcGem(s).  If the gcCode is #PSR or #PDR,
stop all GcGems of that type on all extents.  If gcCode is #ALL, stop
all GcGems of any type. "

| status sessionId |
( gcCode == #ALL) 
  ifTrue: [
    self stopGC: #GC.
    self stopGC: #RCL.
    self stopGC: #EPC.
    self stopGC: #PSR.
    self stopGC: #PDR.
    ^self ].
( #( PSR PDR ParallelShadowReclaim ParallelDeadReclaim ) includes: gcCode ) 
  ifTrue: [ self stopGC: gcCode onExtents: #ALL ]
  ifFalse: [ 
    self _checkGcPrivileges.
    status := self _gcSessions.
    sessionId := status at: ( self _gcType: gcCode ).
    ( sessionId > 0 ) ifTrue: [ System stopSession: sessionId ] ].
^ self
%
category: 'Garbage Collection'
classmethod: System
stopGC: gcCode onExtent: extentId

"Stop the selected type of GcGem(s) on the selected extentId (a SmallInteger). 
If the gcCode is #GC #RCL or #EPC, just stop it (ignore extentId). "

| status index sessionId |
( #( GC RCL EPC GcGem ReclaimGem EpochGem ) includes: gcCode ) 
  ifTrue: [ self stopGC: gcCode ]
  ifFalse: [ 
    self _checkGcPrivileges.
    status := self _gcSessions.
    index := ( ( self _gcType: gcCode ) - 4 ) * 
      ( SystemRepository numberOfExtents ) + extentId + 3.
    sessionId := status at: index.
    ( sessionId > 0 ) ifTrue: [ self stopSession: sessionId ] ].
^ self
%
category: 'Garbage Collection'
classmethod: System
stopGC: gcCode onExtents: extentCollection

"Stop the selected type of GcGem(s) on the selected extentId (a SmallInteger). 
If the gcCode is #GC #RCL or #EPC, just stop it (ignore extentCollection). "

| extents numExtents |
( #( GC RCL EPC GcGem ReclaimGem EpochGem ) includes: gcCode ) 
  ifTrue: [ self stopGC: gcCode ]
  ifFalse: [ 
    (extentCollection == #ALL) 
      ifTrue: [ 
        numExtents := SystemRepository numberOfExtents.
        extents := Array new: numExtents.
        1 to: numExtents do: [ :i | extents at: i put: i ] ]
      ifFalse: [ extents := extentCollection ].
    extents do: [ :extent | self stopGC: gcCode onExtent: extent ] ].
^ self
%
category: 'Garbage Collection'
classmethod: System
stopGcGem

"Stop the generic GcGem"

| gcSession |
self _checkGcPrivileges.
( self configurationAt: #GcSessionEnabled put: 0 ).
gcSession := self _gcSessions at: 1.
( gcSession == 0 ) ifFalse: [ self stopSession: gcSession ].
^ self
%
category: 'Session Control'
classmethod: System
stopOtherSessions

"Prevents any new sessions from being initiated; then, for each
 active session other than the session of the user executing this
 method, aborts the transaction and terminates that session.
 The also stops the Garbage Collector session.

 To reenable logins, send the message System resumeLogins.  Otherwise,
 logins are automatically reenabled when this session logs out.

 To execute this method, you must have explicit privilege for
 SessionAccess and SystemControl in your UserProfile."


"Performs a System | suspendLogins, then
 performs a System | stopSession."

| anArrayOfSessions mySession myPrivs |

myPrivs := System myUserProfile privileges.
((myPrivs includesValue: #SystemControl) _and:
   [myPrivs includesValue: #SessionAccess])
   ifFalse: [ self _error: #rtErrNoPriv .  ^ self "do nothing" ].
self suspendLogins.  " Prevent anyone from logging in "

anArrayOfSessions := self currentSessions.  " All Session running "
mySession := self session.  " My Session number "

anArrayOfSessions do: " Loop all session and log then off if not me "
   [:aSessionNumber | (aSessionNumber = mySession )
                         ifFalse:[self stopSession: aSessionNumber].
   ].
%
category: 'Session Control'
classmethod: System
stopSession: aSessionId

"Aborts the transaction of the specified session (a SmallInteger), then
 terminates that session.  Returns the receiver.  If the indicated
 sessionId is not valid (no session exists with that sessionId), 
 no operation is performed.

 To execute this method, you must have SessionAccess and SystemControl
 privileges. "

| serialNum |

serialNum := GsSession serialOfSession: aSessionId .
self _stopSession: serialNum.
%
category: 'System Control'
classmethod: System
suspendLogins

"Prevents any new sessions from being initiated.  That is, no new user is
 allowed to login.  However, users already active will be allowed to continue
 processing.

 To reenable logins, send the message System | resumeLogins.  (If you fail to do
 so, GemStone automatically reenables logins when the last user logs out.)

 Requires the SystemControl privilege."

self configurationAt: #StnLoginsSuspended put: 1 
%
category: 'Lock Status'
classmethod: System
systemLocks

"Returns a Dictionary describing all objects that are currently locked.  For
 each Association in the result Dictionary, the key is a SmallInteger (the
 session number of a GemStone session that holds locks) and the value is the
 three-element Array described in the sessionLocks method.  If no sessions hold
 any locks, the result Dictionary is empty.

 The Arrays in the result Dictionary contain only those objects that are
 visible to the current session.  This method does not return locks that the
 current session cannot see (objects that have been committed since the
 beginning of the current transaction, uncommitted objects from other sessions,
 or locks on objects for which this session has no read authorization).

 Locks on temporary objects in other sessions are not reported."

| anArray result |
anArray := self _systemLocksPrim  .
result := Dictionary new .
anArray do:[ :assoc | result add: assoc ].
^ result
%
category: 'User-Defined Actions'
classmethod: System
systemUserActionReport

"Returns a SymbolDictionary that provides information about GemStone system user
 actions.  These are user actions that are automatically installed in every
 GemStone session to support classes such as GsFile and GsSocket.

 In the resulting SymbolDictionary, the keys are the symbolic names of the user
 actions, and the values are Booleans.  A values is true if the user action is
 linked with your application, and false if the user action is linked with the
 current GemStone session."

| anArray result |

anArray :=  self _zeroArgCateg2Prim: 9 . "get an Array of SymbolAssociations"
result := SymbolDictionary new .
1 to: anArray size do:[:j | result addAssociation: (anArray at: j ) ].
^ result
%
category: 'Time'
classmethod: System
timeGmt

"Returns a LargePositiveInteger, the time since January 1, 1970, in seconds.
 The time is computed from the clock of the machine on which the session is 
 running, using the offset from the clock on the Stone's (GemStone repository 
 monitor process) machine which is cached in the session at login."

^ self _timeGmt: false
%
category: 'Backward Compatibility'
classmethod: System
timeGmt95

"Obsolete - use timeGmt2005 instead of this method.

 Returns a SmallInteger or a LargePositiveInteger, the time since January 1, 
 1995, in seconds.  It will return a SmallInteger until January 6, 2012.
 The time is computed from the clock of the machine on which the session is 
 running, using the offset from the clock on the Stone's (GemStone repository 
 monitor process) machine which is cached in the session at login."

^ self _timeGmt95: false
%
category: 'Backward Compatibility'
classmethod: System
timeGmt2005

"Returns a SmallInteger or a LargePositiveInteger, the time since January 1, 
 2005, in seconds.  It will return a SmallInteger until January 5, 2022.
 The time is computed from the clock of the machine on which the session is 
 running, using the offset from the clock on the Stone's (GemStone repository 
 monitor process) machine which is cached in the session at login."

^ self _timeGmt2005: false
%
category: 'Transaction Control'
classmethod: System
transactionConflicts

"Returns a SymbolDictionary that contains an Association whose key is
 #commitResult and whose value is one of the following Symbols: #success,
 #failure, #retryFailure, #commitDisallowed, #rcFailure, or #allSymbolsFailure.

 The remaining Associations in the dictionary are used to report the conflicts
 found.  Each Association's key indicates the kind of conflict detected; its
 associated value is an Array of OOPs for the objects that are conflicting.
 If there are no conflicts for the transaction, the returned SymbolDictionary
 has no additional Associations.

 The conflict sets are cleared at the beginning of a commit or abort and
 therefore may be examined until the next commit, continue or abort.

 The keys for the conflicts are as follows:

     Key                Conflicts
 Read-Write          ReadSet and WriteSetUnion conflicts.
 Write-Read          WriteSet and ReadSetUnion conflicts.
 Write-Write         WriteSet and WriteSetUnion conflicts.
 Read-ExclusiveLock  ReadSet and ExclusiveLockSet conflicts.
 Write-ReadLock      WriteSet and ExclusiveLockSet conflicts.
 Write-WriteLock     WriteSet and WriteLockSet conflicts.
 Rc-Write-Write      Logical write-write conflict on reduced conflict object.

 Note: You should be sure to disconnect conflict sets before committing to
 avoid making them persistent."

| symbolDict rcObject sess commitResult resultSymbol assoc |
symbolDict := SymbolDictionary new.

commitResult := self _commitResult.
(commitResult == 0) 
  ifTrue: [ resultSymbol := #success ]
  ifFalse: [
    (commitResult == 1)
       ifTrue: [resultSymbol := #rcFailure ]
       ifFalse: [
         (commitResult == 2)
           ifTrue: [resultSymbol := #allSymbolsFailure ]
           ifFalse: [
             (commitResult == 3)
               ifTrue: [resultSymbol:= #failure ]
                 ifFalse: [
                   (commitResult == 4)
                     ifTrue: [resultSymbol := #retryFailure ]
                     ifFalse: [resultSymbol := #commitDisallowed]
                 ]
             ]
         ]
    ].

assoc := SymbolAssociation newWithKey: #commitResult value: resultSymbol.
symbolDict add: assoc.

((System _hiddenSetSize: 13) > 0) ifTrue: [
  symbolDict add: (SymbolAssociation newWithKey: ('Read-Write' asSymbol)
                                     value: (System _hiddenSetAsArray: 13)) ].
((System _hiddenSetSize: 14) > 0) ifTrue: [
  symbolDict add: (SymbolAssociation newWithKey: ('Write-Read' asSymbol)
                                     value: (System _hiddenSetAsArray: 14)) ].
((System _hiddenSetSize: 15) > 0) ifTrue: [
  symbolDict add: (SymbolAssociation newWithKey: ('Write-Write' asSymbol)
                                     value: (System _hiddenSetAsArray: 15)) ].
((System _hiddenSetSize: 16) > 0) ifTrue: [
  symbolDict add: (SymbolAssociation newWithKey: ('Read-ExclusiveLock' asSymbol)
                                     value: (System _hiddenSetAsArray: 16)) ].
((System _hiddenSetSize: 17) > 0) ifTrue: [
  symbolDict add: (SymbolAssociation newWithKey: ('Write-ReadLock' asSymbol)
                                     value: (System _hiddenSetAsArray: 17)) ].
((System _hiddenSetSize: 18) > 0) ifTrue: [
  symbolDict add: (SymbolAssociation newWithKey: ('Write-WriteLock' asSymbol)
                                     value: (System _hiddenSetAsArray: 18)) ].

rcObject := System rcValueCacheAt: #'Rc-Write-Write' for: self otherwise: nil.
rcObject ~~ nil
  ifTrue: [
    symbolDict add:
      (SymbolAssociation newWithKey: (#'Rc-Write-Write') value: #[ rcObject ])
  ].

sess := System rcValueCacheAt: #'Synchronized-Commit' for: self otherwise: nil.
sess ~~ nil
  ifTrue: [
    assoc value: #synchronizedCommitFailure.
    symbolDict add:
      (SymbolAssociation newWithKey: (#'Synchronized-Commit') value: #[ sess ])
  ].

^symbolDict
%
category: 'Transaction Control'
classmethod: System
transactionMode

"Returns the current transaction mode for the current GemStone session, either
 #autoBegin, #manualBegin or #transactionless.  The default is #autoBegin."

^ self _zeroArgPrim: 4
%
category: 'Transaction Control'
classmethod: System
transactionMode: newMode

"Sets a new transaction mode for the current GemStone session and exits the
 previous mode by aborting the current transaction.  Valid arguments are
 #autoBegin, #manualBegin and #transactionless.
 The mode transactionless is intended primarily for idle sessions. Users
 may scan database objects, but are at risk of obtaining inconsistent views.
"

| coordinator |
^ (coordinator := self _commitCoordinator) == nil
  ifTrue: [ self _localTransactionMode: newMode ]
  ifFalse: [ coordinator transactionMode: newMode ].
%
category: 'User-Defined Actions'
classmethod: System
userAction: aSymbol

"Invokes the user-defined action represented by aSymbol.  Generates an error if
 the user action is not installed in this session, or if it expects any
 arguments.

 A maximum of 47 user actions may be active at any one time on the current
 GemStone Smalltalk stack."

<primitive: 200>
^ self _primitiveFailed: #userAction:
%
category: 'User-Defined Actions'
classmethod: System
userAction: aSymbol with: anArg

"Invokes the user-defined action represented by aSymbol, passing it the argument
 anArg.  Generates an error if the user action is not installed in this session,
 or if the number of arguments expected by the user action is not 1.

 A maximum of 47 user actions may be active at any one time on the current
 GemStone Smalltalk stack."

<primitive: 200>
^ self _primitiveFailed: #userAction:with:
%
category: 'User-Defined Actions'
classmethod: System
userAction: aSymbol with: firstArg with: secondArg

"Invokes the user-defined action represented by aSymbol, passing it the
 arguments firstArg and secondArg.  Generates an error if the user action is not
 installed in this session, or if the number of arguments expected by the user
 action is not 2.

 A maximum of 47 user actions may be active at any one time on the current
 GemStone Smalltalk stack."

<primitive: 200>
^ self _primitiveFailed: #userAction:with:with:
%
category: 'User-Defined Actions'
classmethod: System
userAction: aSymbol with: firstArg with: secondArg with: thirdArg

"Invokes the user-defined action represented by aSymbol, passing it the
 arguments firstArg, secondArg, and thirdArg.  Generates an error if the user
 action is not installed in this session, or if the number of arguments expected
 by the user action is not 3.

 A maximum of 47 user actions may be active at any one time on the current
 GemStone Smalltalk stack."

<primitive: 200>
^ self _primitiveFailed: #userAction:with:with:with:
%
category: 'User-Defined Actions'
classmethod: System
userAction: aSymbol with: firstArg with: secondArg with: thirdArg
  with: fourthArg

"Invokes the user-defined action represented by aSymbol, passing it the
 arguments firstArg, secondArg, thirdArg, and fourthArg.  Generates an error if
 the user action is not installed in this session, or if the number of
 arguments expected by the user action is not 4."

<primitive: 200>
^ self _primitiveFailed: #userAction:with:with:with:with:
%
category: 'User-Defined Actions'
classmethod: System
userAction: aSymbol with: firstArg with: secondArg with: thirdArg
  with: fourthArg with: fifthArg

"Invokes the user-defined action represented by aSymbol, passing it the
 arguments firstArg, secondArg, thirdArg, fourthArg, and fifthArg.  Generates
 an error if the user action is not installed in this session, or if the number
 of arguments expected by the user action is not 5."

<primitive: 200>
^ self _primitiveFailed: #userAction:with:with:with:with:with:
%
category: 'User-Defined Actions'
classmethod: System
userAction: aSymbol with: firstArg with: secondArg with: thirdArg
  with: fourthArg with: fifthArg with: sixthArg

"Invokes the user-defined action represented by aSymbol, passing it the
 arguments firstArg, secondArg, thirdArg, fourthArg, fifthArg, and sixthArg.
 Generates an error if the user action is not installed in this session, or if
 the number of arguments expected by the user action is not 6."

<primitive: 200>
^ self _primitiveFailed: #userAction:with:with:with:with:with:with:
%
category: 'User-Defined Actions'
classmethod: System
userAction: aSymbol with: firstArg with: secondArg with: thirdArg
  with: fourthArg with: fifthArg with: sixthArg with: seventhArg

"Invokes the user-defined action represented by aSymbol, passing it the
 arguments firstArg, secondArg, thirdArg, fourthArg, fifthArg, sixthArg, and
 seventhArg.  Generates an error if the user action is not installed in this
 session, or if the number of arguments expected by the user action is not 7."

<primitive: 200>
^ self _primitiveFailed: #userAction:with:with:with:with:with:with:with:
%
category: 'User-Defined Actions'
classmethod: System
userAction: aSymbol with: firstArg with: secondArg with: thirdArg
  with: fourthArg with: fifthArg with: sixthArg with: seventhArg
  with: eighthArg

"Invokes the user-defined action represented by aSymbol, passing it the
 arguments firstArg, secondArg, thirdArg, fourthArg, fifthArg, sixthArg,
 seventhArg, and eighthArg.  Generates an error if the user action is not
 installed in this session, or if the number of arguments expected by the
 user action is not 8."

<primitive: 200>
^ self _primitiveFailed: #userAction:with:with:with:with:with:with:with:with:
%
category: 'User-Defined Actions'
classmethod: System
userAction: aSymbol withArgs: anArray

"Invokes the user-defined action represented by aSymbol, passing it the elements
 of anArray as arguments.  Generates an error if the user action is not
 installed in this session, or if the number of arguments expected by the user
 action is not the same as the number of elements in anArray.

 A maximum of 47 user actions may be active at any one time on the current
 GemStone Smalltalk stack."

<primitive: 201>
^ self _primitiveFailed: #userAction:withArgs:
%
category: 'User-Defined Actions'
classmethod: System
userActionReport

"Returns a SymbolDictionary that provides information about all user actions
 installed in this GemStone session.  In that SymbolDictionary, the keys are
 the symbolic names of the user actions, and the values are Booleans (true if
 the user action is linked with your application, false if the user action is
 linked with the current GemStone session)."

 | anArray result |
 anArray :=  self _zeroArgCateg2Prim: 7 . "get an Array of SymbolAssociations"
 result := SymbolDictionary new .
 1 to: anArray size do:[:j | result addAssociation: (anArray at: j ) ].
 ^ result

%
category: 'Session Control'
classmethod: System
userProfileForSession: aSessionId

"Returns the UserProfile attached to the specified session (a SmallInteger).
 If the indicated session is not active, returns nil.

 Requires SessionAccess privilege if aSessionId is not the current session."

^ ( self descriptionOfSession: aSessionId) at: 1
%
category: 'Session Control'
classmethod: System
users

"Returns a Set of UserProfiles for all users known to the system."

^ AllUsers
%
category: 'Setting Locks'
classmethod: System
writeLock: anObject

"Analogous to System | exclusiveLock:.  However, this method requests and
 grants write locks."

| result |

result := System _lock: anObject kind: #write.
(result == 1)
  ifTrue: [^self].
self _lockError: result obj: anObject args: #[#write]
%
category: 'Setting Locks'
classmethod: System
writeLock: anObject ifDenied: denyBlock ifChanged: changeBlock

"Analogous to System | exclusiveLock:ifDenied:ifChanged:.  However,
 this method requests and grants write locks."

| result |
result := System _lock: anObject kind: #write.
(result == 1) ifTrue:
   [^self].
^ self _lockEvaluateErr: result denied: denyBlock changed: changeBlock
%
category: 'Backward Compatibility'
classmethod: System
writeLock: anObject ifDenied: denyBlock ifChanged: changeBlock
  ifNotCommitted: notCommittedBlock

"Obsolete in GemStone 4.0.  Locks are now allowed on objects that have not been
 committed."

^self writeLock: anObject ifDenied: denyBlock ifChanged: changeBlock
%
category: 'Setting Locks'
classmethod: System
writeLockAll: aCollection

"Analogous to System | exclusiveLockAll:.  However, this method requests and
 grants write locks."

| result |
result := System _lockAll: aCollection kind: #write.
(result == System)
  ifTrue: [^self]
  ifFalse: " else we generate the error lock incomplete "
           [^ (result at: 1) _error: #lockErrIncomplete
                             args: #[(result at:2), (result at:3)]].
%
category: 'Setting Locks'
classmethod: System
writeLockAll: aCollection ifIncomplete: incompleteBlock

"Analogous to System | exclusiveLockAll:ifIncomplete:.  However, this method
 requests and grants write locks."

| result |
result := System _lockAll: aCollection kind: #write.
(result == System)
  ifTrue: [^self]
  ifFalse: " else we execute the incomplete block with 3 arguments "
          [^incompleteBlock value: (result at: 1)
                            value: (result at:2)
                            value: (result at:3)].
%
category: 'Backward Compatibility'
classmethod: System
writeLockObjAndIndexes: anObject

"Obsolete in GemStone 4.0.  Indexes now have reduced conflict behavior and
 locks are now allowed on objects that have not been committed."

^ self writeLock: anObject
%
category: 'Backward Compatibility'
classmethod: System
writeLockObjAndIndexes: anObject ifDenied: denyBlock ifChanged: changeBlock
  ifNotCommitted: notCommittedBlock

"Obsolete in GemStone 4.0.  Indexes now have reduced conflict behavior and
 locks are now allowed on objects that have not been committed."

^ self writeLock: anObject ifDenied: denyBlock ifChanged: changeBlock
%
category: 'Private'
classmethod: System
_abortTransactionAndKeepAttached

"Has no effect"
^self 
%
category: 'Hidden Set Support'
classmethod: System
_add: anObject to: hiddenSetSpecifier

"Adds anObject to the hiddenSet."
<primitive: 168>
hiddenSetSpecifier _validateClass: SmallInteger.
(hiddenSetSpecifier < 1 _or:
       [hiddenSetSpecifier > (self HiddenSetSpecifiers size)])
  ifTrue: [hiddenSetSpecifier _error: #rtErrArgOutOfRange].
self _primitiveFailed: #_add:to:  .
self _uncontinuableError
%
category: 'Hidden Set Support'
classmethod: System
_addAll: anArray to: hiddenSetSpecifier

"Adds contents of anArray to a hiddenSet. anArray itself is not
 added to hiddenSet."

<primitive: 520>
hiddenSetSpecifier _validateClass: SmallInteger.
(hiddenSetSpecifier < 1 _or:
       [hiddenSetSpecifier > (self HiddenSetSpecifiers size)])
  ifTrue: [hiddenSetSpecifier _error: #rtErrArgOutOfRange].
self _primitiveFailed: #_addAll:to:  .
self _uncontinuableError
%
category: 'System Control'
classmethod: System
_addAllToStoneLog: aString

"Appends text to the Stone's informational log file.  First, this method writes
 a banner that identifies the session from which aString came.  It then appends
 aString itself.  The argument must be a kind of String or DoubleByteString.

 Generates an error if aString is larger than approximately 8000 bytes."

<primitive: 463>

aString _validateClasses:#[ String, DoubleByteString].
self _primitiveFailed: #addAllToStoneLog:
%
category: 'Reduced Conflict Support'
classmethod: System
_addEntireObjectToRcReadSet: anObject

"Adds anObject to the reduced-conflict read set.  If the object is a large
 object, all nodes of the object are added."

self _addToRcReadSet: anObject includingAllNodes: true.
%
category: 'Reduced Conflict Support'
classmethod: System
_addRootObjectToRcReadSet: anObject

"Adds anObject to the reduced-conflict read set.  Only the root object is added
 (even if the object is a large object)."

self _addToRcReadSet: anObject includingAllNodes: false.
%
category: 'Backward Compatibility'
classmethod: System
_addToRcReadSet: anObject

"Obsolete in GemStone 4.0.  Use the _addToRcReadSet:includingAllNodes: method
 instead."

^ self _addToRcReadSet: anObject includingAllNodes: true .
%
category: 'Reduced Conflict Support'
classmethod: System
_addToRcReadSet: anObject includingAllNodes: aBoolean

"Adds anObject to the reduced-conflict read set.  This read set is used to
 determine if there are any objects in the read/write or write/read conflict
 sets that were part of a reduced conflict data structure.  All objects that are
 part of composite used to represent a reduced conflict object should be
 added to this set to avoid unnecessary conflict handling.  If aBoolean is
 true, the entire object (including all interior nodes of a large object) are
 added.  If aBoolean is false, only the root object is added."

<primitive: 116>
aBoolean _validateClass: Boolean.
self _primitiveFailed: #_addToRcReadSet:includingAllNodes: .
self _uncontinuableError
%
category: 'Private'
classmethod: System
_allErrorMessagesReport

"Private."

| errorSymbolsArray symsWithNoMsg msgsWithNoSym report lf line errMsgsArray 
  firstNil last j |

errorSymbolsArray  := Array new .
ErrorSymbols keysAndValuesDo:[:aSym :aNum |
  aNum > errorSymbolsArray size ifTrue:[ errorSymbolsArray size: aNum ].
  (errorSymbolsArray at: aNum) ~~ nil ifTrue:[
    self _halt:'Duplicate symbol for error number ' + aNum asString .
    ].
  errorSymbolsArray at: aNum put: aSym .
  ].
errMsgsArray := GemStoneError at: #English .

msgsWithNoSym := String new .
symsWithNoMsg := String new .
report := String new .
lf := Character lf .

report addAll:'ERROR MESSAGES STORED IN THE REPOSITORY:'; add: lf; add: lf .

1 to: errorSymbolsArray size do:[:j | | aSym msgParts |
  aSym := errorSymbolsArray at: j .
  msgParts := errMsgsArray at: j .
  aSym == nil ifTrue:[
    msgParts ~~ nil ifTrue:[
       msgsWithNoSym addAll: '*** No error symbol for error number ';
         addAll: j asString; add: lf .
       ]
    ]
  ifFalse:[
    msgParts == nil ifTrue:[
       symsWithNoMsg addAll: '*** No message for error symbol #';
         addAll: aSym quoted; add: lf .
       ]
    ifFalse:[
       line := j asString. 
       line addAll: ', #' ; addAll: aSym; addAll: ' :  ' ;
            addAll: (self _messageForErrorNum: j)  .
       report addAll: (line _wrapTo: 80 indentingWith: '    '); add: lf .
       ]
    ].
  ].

report add: lf .
report addAll:
'ERROR MESSAGES FROM errmsg.c, TEXT FROM english.err'; add: lf ;
addAll: '  (Repository message followed by english.err message)'; add: lf;
add: lf .

1 to: errorSymbolsArray size do:[:j | | aSym internalMsg |
  aSym := errorSymbolsArray at: j .
  aSym ~~ nil ifTrue:[
    internalMsg := self _internalMessageForErrorNum: j .
    internalMsg ~~ nil ifTrue:[
      line := j asString. 
      line addAll: ', #' ; addAll: aSym; addAll: ' :  ' ;
           addAll: (self _messageForErrorNum: j) ; add: lf ;
           addAll: internalMsg .
      report addAll: (line _wrapTo: 80 indentingWith: '    '); add: lf; add: lf.
      ].
    ].
  ].

report add: lf ; addAll:'UNUSED ERROR NUMBERS:'; add: lf .

firstNil := 1 .
j := 1.
last := errorSymbolsArray size .
[j < last] whileTrue:[
  [ (errorSymbolsArray at: j) == nil _and:[j < last] ] whileTrue:[ j := j + 1 ]. 
  j > firstNil ifTrue:[
    report addAll: firstNil asString ; addAll: ' to: '; 
      addAll: (j - 1) asString; add: lf .
    ]
  ifFalse:[
    report addAll: firstNil asString; add: lf .
    ].
  firstNil := nil . 
  [ (errorSymbolsArray at: j) ~~ nil _and:[j < last] ] whileTrue:[ j := j + 1 ]. 
  firstNil := j .
  ].

report add: lf .
msgsWithNoSym size > 0 
  ifTrue:[ report addAll: msgsWithNoSym ]
  ifFalse:[ report addAll:'No messages found without an error symbol.'].
report add: lf .
symsWithNoMsg size > 0 
  ifTrue:[ report addAll: symsWithNoMsg ]
  ifFalse:[ report addAll: 'No error symbols found without a message.'].
report add: lf .
^ report
%
category: 'Runtime Configuration Access'
classmethod: System
_atConfigId: rtConfigId put: aValue

"Change the specified Stone runtime configuration parameter.  The current user
 must be SystemUser to set any configuration parameter except the
 ConcurrencyMode and StoneLogins, which are protected by privileges. 
 
 Changing a runtime parameter does not alter the configuration file.
 The new value will show up in the result of configuration report methods, 
 but will not be used when Stone next starts up."

<primitive: 355>
rtConfigId _validateClass: SmallInteger .
aValue _validateClass: SmallInteger .
(self _configParameterName: rtConfigId kind: $C ) == nil 
   ifTrue:[ rtConfigId _error: #rtErrArgOutOfRange ].
rtConfigId >= 0 ifTrue:[ 
  (self _configParameterName: rtConfigId kind: $C ) _error: #rtErrConfigReadOnly
  ].
   
self _primitiveFailed: #_atConfigId:put: .
self _uncontinuableError
%

category: 'Runtime Configuration Access'
classmethod: System
gemIsBigEndian

" Returns true if the gem process is running 
  on a machine using big endian byte ordering "

^ self _zeroArgPrim: 63 

%

category: 'Runtime Configuration Access'
classmethod: System
stoneIsBigEndian

" Returns true if the stone process is running 
  on a machine using big endian byte ordering "

^ self _zeroArgPrim: 64 

%

category: 'Private'
classmethod: System
_beginTransactionAndKeepAttached

"Has no effect"
^self 
%
category: 'Private'
classmethod: System
_cacheName: aString

"Sets the name the current session is known by in the shared cache
 to 'aString'. Does nothing if the current session is not attached
 to a shared cache. Raises an error if 'aString' is too long.
 Returns self."

<primitive: 509>
self _primitiveFailed: #_cacheName:
%
category: 'Private'
classmethod: System
_cacheStatsForSlotOrSession: aProcessSlotOrSessionId

"The argument must be an instance of SmallInteger.
 A positive argument means it is a cache process slot number.
 A negative argument means it is a gem session ID. "

<primitive: 220>
aProcessSlotOrSessionId _validateClass: SmallInteger.
self _error: #rtErrArgOutOfRange args: #[ aProcessSlotOrSessionId ] .
^ self _primitiveFailed: #_cacheStatsForSlotOrSession:
%
category: 'Private'
classmethod: System
_checkAuthorization: anObject forRead: aBoolean

"This method tests whether the user has authorization to read (if aBoolean is
 true, write if false) anObject without adding it to the read set or write set
 and returns a Boolean result."

<primitive: 103>
self _primitiveFailed: #_checkAuthorization:forRead: .
self _uncontinuableError
%
category: 'Garbage Collection'
classmethod: System
_checkGcPrivileges

"Check that we're authorized to do GC operations"

| priv |
priv := System myUserProfile privileges.
( priv includes: #GarbageCollection ) ifFalse: [ self _error: #rtErrNoPriv ].
( priv includes: #SystemControl ) ifFalse: [ self _error: #rtErrNoPriv ].
( priv includes: #SessionAccess ) ifFalse: [ self _error: #rtErrNoPriv ]
%
category: 'Private'
classmethod: System
_clearDeadObjs

"Clears the stone's lists of possibly dead and dead objects.
 Recent garbage collection operations that recorded  dead
 objects may need to be repeated after this call is made.

 This method may only be called by SystemUser."

^ self _zeroArgPrim: 39 
%
category: 'Performance Monitoring'
classmethod: System
_clearStoneStatistics

"Clear Stone statistics after printing them in the log."

^ self _zeroArgCateg2Prim: 5
%
category: 'Reduced Conflict Support'
classmethod: System
_commit: commitMode

"commitMode is a SmallInteger:

 0 for normal,
 1 for release locks,
 2 for checkpoint and release locks,
 3 for normal commit called from GciCommit,
 4 normal with finalizedIndexedNscs "

| coordinator |

^ (coordinator := self _commitCoordinator) == nil
  ifTrue: [ self _localCommit: commitMode ]
  ifFalse: [ coordinator commit: commitMode ].
%
category: 'Private'
classmethod: System
_commitCoordinator

"Returns the commit coordinator stored in temporary session state."

^ self _sessionStateAt: 14
%
category: 'Private'
classmethod: System
_commitCoordinator: aCoordinator

"Sets the commit coordinator stored in temporary session state."

^ self _sessionStateAt: 14 put: aCoordinator
%
category: 'Private'
classmethod: System
_commitPrintingDiagnostics

    "Commits the current transaction. Answers true if successful,
    otherwise false. If false, it also prints to stderr the OOPs for
    each kind of conflict, ordered by kind. This is used in the
    upgrade scripts."

    | conflicts |
    System commitTransaction 
        ifTrue: [^true]
        ifFalse:
            [conflicts := System transactionConflicts.
             (conflicts at: #commitResult) == #success ifFalse:
                [GsFile stderr log: (conflicts at: #commitResult).
                 GsFile stderr log: 'Read-Write Conflicts...'.
                 (conflicts at: ('Read-Write' asSymbol) ifAbsent: [#()]) do:
                     [ :each |
                      GsFile stderr log: '    ', each asOop asString].
                 GsFile stderr log: 'Write-Read Conflicts...'.
                 (conflicts at: ('Write-Read' asSymbol) ifAbsent: [#()]) do:
                     [ :each |
                      GsFile stderr log: '    ', each asOop asString].
                 GsFile stderr log: 'Write-Write Conflicts...'.
                 (conflicts at: ('Write-Write' asSymbol) ifAbsent: [#()]) do:
                     [ :each |
                      GsFile stderr log: '    ', each asOop asString].
                 GsFile stderr log: 'Read-ExclusiveLock Conflicts...'.
                 (conflicts at: ('Read-ExclusiveLock' asSymbol) ifAbsent: [#()]) do:
                     [ :each |
                      GsFile stderr log: '    ', each asOop asString].
                 GsFile stderr log: 'Write-ReadLock Conflicts...'.
                 (conflicts at: ('Write-ReadLock' asSymbol) ifAbsent: [#()]) do:
                     [ :each |
                      GsFile stderr log: '    ', each asOop asString].
                 GsFile stderr log: 'Write-WriteLock Conflicts...'.
                 (conflicts at: ('Write-WriteLock' asSymbol) ifAbsent: [#()]) do:
                     [ :each |
                      GsFile stderr log: '    ', each asOop asString].
                 GsFile stderr log: 'Rc-Write-Write Conflicts...'.
                 (conflicts at: ('Rc-Write-Write' asSymbol) ifAbsent: [#()]) do:
                     [ :each |
                      GsFile stderr log: '    ', each asOop asString].
                 GsFile stderr log: 'Synchronized-Commit Conflicts...'.
                 (conflicts at: ('Synchronized-Commit' asSymbol) ifAbsent: [#()]) do:
                     [ :each |
                      GsFile stderr log: '    ', each asOop asString]].
             ^false]
%
category: 'Private'
classmethod: System
_commitResult

"Private.  Returns a SmallInteger which represents the result of the last 
 commitTransaction or continueTransaction operation.  If abortTransaction was 
 issued since the last commit or continue then it returns SmallInteger 0.  
 The possible return values are:

    0 - success
    1 - rcFailure
    2 - allSymbolsFailure
    3 - failure
    4 - retryFailure
    5 - commitDisallowed"

^ self _zeroArgPrim: 30
%
category: 'Private'
classmethod: System
_committedDataPages

"Returns an Array of the committed data pages referenced from the session's
 view of GemStone.  The contents of this Array are invalid after the next
 commit or abort."

^ self _zeroArgCateg2Prim: 10
%
category: 'Configuration File Access'
classmethod: System
_configConstantsDict

"Returns a Dictionary of internal configuration constants that
 are fixed at compilation of the gem and stone executables. The dictionary
 keys are Symbols.  Its values are SmallInteger configuration constants."


| anArray result |
anArray := self _zeroArgCateg2Prim: 8 .
result := SymbolDictionary new .
1 to: anArray size do:[ :j | | anAssoc |
  anAssoc := anArray at: j .
  result add: anAssoc .
  ].
^ result
%
category: 'Configuration File Access'
classmethod: System
_configFileParameterDict

"Returns a Dictionary of names for configuration file parameters.  The
 dictionary keys are Symbols.  Its values are SmallInteger configuration IDs."

| result cfgId cfgName |
result := SymbolKeyValueDictionary new .
cfgId := 0 .
[
  cfgName := self _configParameterName: cfgId kind: $C .
  cfgName == nil ifFalse:[
    result at: cfgName put: cfgId .
    cfgId := cfgId + 1 .
    ] .
  cfgName == nil 
  ] untilTrue .

cfgId := -1 .
[
  cfgName := self _configParameterName: cfgId kind: $C .
  cfgName == nil ifFalse:[
    result at: cfgName put: cfgId .
    cfgId := cfgId - 1 .
    ] .
  cfgName == nil 
  ] untilTrue .

^ result
%
category: 'Configuration File Access'
classmethod: System
_configParameterName: anInt kind: aChar

"Returns a String, which is the name of the configuration parameter whose
 internal identifier is anInt.  Returns nil if anInt is out of range.

 aChar must be one of
   $C - anInt is identifier of a configuration parameter
   $V - anInt is identifier of a version information item
"

<primitive: 342>
anInt _validateClass: SmallInteger .
^ self _primitiveFailed: #_configParameterName:kind:
%
category: 'System Control'
classmethod: System
_configurationAt: cfgId isStone: aBool kind: aKind

"Returns the configuration value specified. If aBool is true, get the
 configuration value from the Stone process, otherwise from the current
 session.

 cfgId must be a SmallInteger; if >= 0, returns the value of the specified
 configuration parameter;  if < 0, returns the value of the specified
 runtime control parameter.

 aKind must be either $C (for configuration info) or $V (for version info)"

<primitive: 338>
aBool _validateClass: Boolean .
cfgId _validateClass: SmallInteger .
aKind == $C ifTrue:[
  ConfigurationParameterDict keyAtValue: cfgId ifAbsent:[
     cfgId _error: #rtErrArgOutOfRange
     ].
  ]
  ifFalse:[
  VersionParameterDict keyAtValue: cfgId ifAbsent:[
     cfgId _error: #rtErrArgOutOfRange
     ].
  ].
self _primitiveFailed: #_configurationAt:isStone:kind: .
self _uncontinuableError
%
category: 'Configuration File Access'
classmethod: System
_configurationReport: isStone

"Private."

| result |
result := SymbolDictionary new .
ConfigurationParameterDict keysAndValuesDo:[ :aName :anId | | aVal |
  aVal := self _configurationAt: anId isStone: isStone kind: $C .
  aVal ~~ nil ifTrue:[ 
    result at: aName put: aVal
    ].
  ].
^ result 
%
category: 'Private'
classmethod: System
_deadNotReclaimed

"Private.  Returns an Array that contains the current contents of the 
 global set of objects which have been identified as dead but have not 
 yet been reclaimed. Before accessing this set, the user should disable
 the reclaiming of dead (set reclaimDeadEnabled in GcUser's UserGlobals to 
 false).  The user should be sure to reset the size of the Array returned
 to zero when done analyzing the dead objects before re-enabling the
 reclaiming of dead objects."

^ self _zeroArgPrim: 31
%
category: 'Session Control'
classmethod: System
_descriptionOfSessionSerialNum: serialNum sessionId: aSessionId

"Returns an eleven-element Array describing the session identified by the arguments.   
 The session is looked up by serial number if serialNum is > 0.  Otherwise
 it is looked up by sessionId.

 See System (C) | descriptionOfSession: for documentation on the contents of the
 result Array.
 Requires SessionAccess privilege if the session being looked up is not 
 the current session."

<primitive: 334>
aSessionId _validateClass: SmallInteger.
^ self _primitiveFailed: #_descriptionOfSessionSerialNum:sessionId:
%
category: 'Indexing Support'
classmethod: System
_disableProtectedMode
 
"Disables the execution of protected methods."
 
"i.e, exit protected mode."
" Was primitive 174, now a special selector optimized by the compiler. This
  method is so perform will work."
 
self _disableProtectedMode
%
category: 'Debugging Support'
classmethod: System
_disableTraceObjs

"Disable the tracing of objects.  It is a good idea to disable the tracing
 before starting the analysis of the objsDisposed set so that the analysis
 doesn't continue to add objects to the set."

^ self _zeroArgPrim: 16
%
category: 'Reduced Conflict Support'
classmethod: System
_disallowSubsequentCommits

"Disallows subsequent attempts to commit."

^ self _zeroArgPrim: 22
%
category: 'Private'
classmethod: System
_dumpFrameData

"Private.  Prints out shared page cache frame data to the session log file."

^ self _zeroArgPrim: 44
%
category: 'Debugging Support'
classmethod: System
_enableTraceObjs

"Start tracing object creation and garbage collection.
 The OOP of each object created is added to the hidden set objsCreated.
 Instead of disposing object identified by the generation scavenging, 
 the objects are written to disk and their OOPs are added to objsDisposed.

 It is recommended that the user not run very long in this mode since all
 temporary objects will be written to disk for analysis.  It is also best
 to avoid committing a session that has been used for object tracing since
 that would make all of the objects permanent and require that they be 
 collected by either the EpochGc or markForCollection processes.  Instead
 simply logout when finished analyzing the garbage generated."

^ self _zeroArgPrim: 15
%
category: 'Private'
classmethod: System
_encodedIntToOopFiltered: array serial: serialNum
"Same as _intToOopFiltered: serial: except that the 
 integers in the array are assumed to be encoded oops rather than oops.

 Refer to the method Object>>_asEncodedOop for more information on 
 encoded oops."

<primitive: 544>
self _primitiveFailed: #_encodedIntToOopFiltered:serial:.
self _uncontinuableError
%
category: 'Private'
classmethod: System
_exclusiveLockWriteSubset

"Start with the write set and perform filtering of AllSymbol components, 
 non-committed objects, and objects already locked, putting the results 
 in a new hidden set (writeLockWriteSubset).  Next, attempt to acquire an
 exclusive lock on the objects in the hidden set.  I all locks are acquired 
 cleanly, return true.  If all locks acquired but some are dirty, return 
 an array of dirty locked objects.  If not all locks could be acquired, 
 return false (locks that were acquired, including dirty locks are available
 in the writeLockWriteSubset hidden set.  Any locks that are acquired are 
 also put in the CommitOrAbortReleaseLocksSet."

^self _zeroArgPrim: 37
%
category: 'Private'
classmethod: System
_findAllPublishedOn: machine
  "Description:
     Find all published servers on the given machine.
   Input:
     machine <String or nil>: name of machine to find on or nil if local.
   Result <Array or nil>:
     nil if no servers were found.
     instance of Array that contains one element for each server found.
     Each element will itself be an Array with the contents described
     for the _findPublished:on: method.
   Exceptions:
     Illegal input raises various errors.
     Error 2367 raised if low level C code fails with system error.
  "

<primitive: 499>
self _primitiveFailed: #_findAllPublishedOn:
%
category: 'Private'
classmethod: System
_findPublished: server on: machine
  "Description:
     Find a published server on the given machine.
   Input:
     server <String>: name of server to findunpublish.
     machine <String or nil>: name of machine to find on or nil if local.
   Result <Array or nil>:
     nil if server was not found.
     instance of Array if server was found. The array contents are:
       1. <String>: the name of the server
       2. <SmallInteger>: the port number of ther servers listening socket.
       3. <Integer>: the proces id of the server's main process.
       4. <Boolean>: true if process exists, false if not.
       5. <String>: type of server: 'Stone', 'NetLDI', 'Cache', 'Broker',
                                    'Agent', 'RCPD', 'Service-Stone',
                                    'Service-NetLDI', or '?'.
       6. <String>: name of user that created the server.
       7. <String>: time server was created.
       8. <String>: name of machine server is running on.
       9. <String or nil>: version of GemStone used by server.
       10.<String or nil>: GemStone product directory.
       11.<String or nil>: log file name of server.
       12.<String or nil>: startup options of server.
       13.<String or nil>: GemStone system configuration file used by server.
       14.<String or nil>: GemStone executable configuration file used by server.
   Exceptions:
     Illegal input raises various errors.
     Error 2367 raised if low level C code fails with system error.
  "

<primitive: 498>
self _primitiveFailed: #_findPublished:on:
%

category: 'Private'
classmethod: System
_flushAllExtents

"Deprecated, has no effect. Only the stone process flushes extents."

^ self 
%
category: 'Private'
classmethod: System
_flushConnectedToPom

"Flushes all LOM objects that are transitively connected to POM."

"WARNING - This method is designed for internal system use by the 
GemStone Kernel classes. It is also used by GemStone staff for 
diagnostic purposes.  Improper use of this method will cause 
repository corruption manifesting in 'object does not exist' errors. 
This method should not be used in production applications."

^ self _zeroArgPrim: 29
%
category: 'Garbage Collection'
classmethod: System
_gcSessionIndexFor: gcType extentId: extentId

"For a given gcType and extentId, return the appropriate index into
the status array returned by System>>_gcSessions "

| numExtents |
( ( gcType < 0 ) _or: [ gcType > 5 ]) ifTrue: [ 
  System signal: ( ErrorSymbols at: #rtErrBadGcType )
    args: #[ gcType ] 
    signalDictionary: GemStoneError ].
( gcType < 4 ) ifTrue: [ ^gcType ].
numExtents := SystemRepository numberOfExtents.
( ( extentId <= 0 ) _or: [ extentId > numExtents ] ) ifTrue: [
  System signal: ( ErrorSymbols at: #rtErrBadExtentId )
  args: #[ extentId ] 
  signalDictionary: GemStoneError ].
^ ( gcType - 4 ) * numExtents + extentId + 3.
%
category: 'Garbage Collection'
classmethod: System
_gcSessions

"Returns an array of session ID's ( or zero if not running ) indicating:
   1:    GcGem
   2:    RCL (Reclaim) Gem
   3:    EPC (Epoch) Gem
   next NumExtent elements:  PSR (ParallelShadowReclaim) Gem for each extent
   next NumExtent elements:  PDR (ParallelDeadReclaim) Gem for each extent "

| priv |

" Check that we have #SessionAccess privileges."
priv := System myUserProfile privileges.
( priv includes: #SessionAccess ) ifFalse: [ self _error: #rtErrNoPriv ].

^self _zeroArgPrim: 45
%
category: 'Garbage Collection'
classmethod: System
_gcSessionsRunning

  "Returns true if any GcGems are running"
  self _gcSessions do: [ :session |
    ( session ~~ 0 ) ifTrue: [ ^ true ] ].
  ^ false
%
category: 'Garbage Collection'
classmethod: System
_gcSessionType: sessionId

  "Given a particular sessionId, returns an appropriate gcType
   ( #GC, #RCL, #EPC, #PSR, #PDR ) if that session is a GcGem,
   otherwise returns nil if it is not a GcGem. "

  | gcSessions |
  gcSessions := self gcSession: #ALL.
  ( sessionId = ( gcSessions at: 1 ) ) ifTrue: [ ^ #GC ].
  ( sessionId = ( gcSessions at: 2 ) ) ifTrue: [ ^ #RCL ].
  ( sessionId = ( gcSessions at: 3 ) ) ifTrue: [ ^ #EPC ].
  ( gcSessions at: 4 ) do: [ :psrSession |
    ( sessionId = psrSession ) ifTrue: [ ^ #PSR ] ].
  ( gcSessions at: 5 ) do: [ :pdrSession |
    ( sessionId = pdrSession ) ifTrue: [ ^ #PDR ] ].
  ^ nil
%
category: 'Garbage Collection'
classmethod: System
_gcType: gcCode

"Convert gcCode symbol to gcType number:
    #GC  = 1 (Normal GcGem)
    #RCL = 2 (ReclaimGem)
    #EPC = 3 (EpochGem)
    #PSR = 4 (ParallelShadowReclaim)
    #PDR = 5 (ParallelDeadReclaim) "

^#( GC RCL EPC PSR PDR) indexOf: gcCode 
  ifAbsent: [
    #( GcGem ReclaimGem EpochGem 
       ParallelShadowReclaim ParallelDeadReclaim ) indexOf: gcCode
    ifAbsent: [
      System signal: (ErrorSymbols at: #rtErrBadGcType )
        args: #[ gcCode ]
        signalDictionary: GemStoneError ] ]
%
category: 'Version Management'
classmethod: System
_gemVersion

"Private."

^ self _zeroArgCateg2Prim: 3 
%
category: 'Private'
classmethod: System
_generationScavenge

"Private."

"Explicitly triggers in-memory garbage collection of temporary objects.
 Should only be used when debugging internal problems. Frequent use
 can degrade performance."

^ self _zeroArgPrim: 14 
%
category: 'Reduced Conflict Support'
classmethod: System
_getRedoAndConflictObjects

"Builds a list of objects that need to be scanned because they need to
 have conflicts resolved.  If the result array is empty, then the transaction
 has conflicts that cannot be resolved because we should only get here if
 some Rc conflicts have been detected."
          
| conflictDict wwConflicts scanArray redoObject redoLog array
conflictObject  idx |
     
conflictDict := self transactionConflicts.
" if there are read-write and write-read conflicts, we cannot resolve them "
( (conflictDict at: #'Read-Write' otherwise: nil) ~~ nil _and:
[ (conflictDict at: #'Write-Read' otherwise: nil) ~~ nil ] )
  ifTrue: [ ^ nil ].

wwConflicts := conflictDict at: #'Write-Write' otherwise: nil.
wwConflicts == nil
 ifTrue: [ ^ nil ].
              
scanArray := Array new.

redoLog := self _redoLog.

" make an initial scan to gather objects to be replayed "
1 to: wwConflicts size do: [ :i |
  conflictObject := wwConflicts at: i.
  
  redoLog == nil
    ifTrue: [   
      " no redo log, treat the object itself as the one on which conflicts
        must be resolved "
      (scanArray includesIdentical: conflictObject)
        ifFalse: [
          scanArray add: conflictObject;
            add: (Array with: conflictObject).
        ]
    ]
    ifFalse: [
      " get the object that has to be replayed due to this conflict "
      redoObject := redoLog getRedoObjectForConflictingObject: conflictObject.
      redoObject == nil
        ifTrue: [
          " no redo object in the redo log, treat the object itself as
            the one on which conflicts must be resolved "
          (scanArray includesIdentical: conflictObject)  
            ifFalse: [
              scanArray add: conflictObject;
                add: (Array with: conflictObject).
            ]
        ]
        ifFalse: [
          idx := scanArray _indexOfIdentical: redoObject.
          idx > 0
            ifTrue: [
              array := scanArray at: idx + 1.
              array add: conflictObject.
            ]
            ifFalse: [
              scanArray add: redoObject;
                add: (Array with: conflictObject).
            ]
        ]
    ]
].   
     
^ scanArray
%
category: 'Hidden Set Support'
classmethod: System
_hiddenSetAsArray: hiddenSetSpecifier

"Returns an Array containing the contents of the hiddenSet."

<primitive: 212>
hiddenSetSpecifier _validateClass: SmallInteger.
(hiddenSetSpecifier < 1 _or:
       [hiddenSetSpecifier > (self HiddenSetSpecifiers size)])
  ifTrue: [hiddenSetSpecifier _error: #rtErrArgOutOfRange].
self _primitiveFailed: #_hiddenSetAsArray:.
self _uncontinuableError
%
category: 'Hidden Set Support'
classmethod: System
_hiddenSetReinit: hiddenSetSpecifier

"Reinitializes the hiddenSet to empty.  Must be system user for all
 hidden sets except the reserved ones."

<primitive: 172>
hiddenSetSpecifier _validateClass: SmallInteger.
(hiddenSetSpecifier < 1 _or:
       [hiddenSetSpecifier > (self HiddenSetSpecifiers size)])
  ifTrue: [hiddenSetSpecifier _error: #rtErrArgOutOfRange].
self _primitiveFailed: #_hiddenSetReinit:.
self _uncontinuableError
%
category: 'Hidden Set Support'
classmethod: System
_hiddenSetSize: hiddenSetSpecifier

"Returns the number of elments in the hiddenSet."

<primitive: 171>
hiddenSetSpecifier _validateClass: SmallInteger.
(hiddenSetSpecifier < 1 _or:
       [hiddenSetSpecifier > (self HiddenSetSpecifiers size)])
  ifTrue: [hiddenSetSpecifier _error: #rtErrArgOutOfRange].
self _primitiveFailed: #_hiddenSetSize:.
self _uncontinuableError
%
category: 'Private'
classmethod: System
_highestAllocatedOop

"Return the highest committed OOP that is currently assigned to
 an object (not a free oop).   Similar to _oopHighWaterMark
 except that the high water OOP may be a free oop."

^self _zeroArgPrim: 46
%
category: 'Private'
classmethod: System
_hostCallDebugger

"Private."

"For debugging.  Allows GemStone Smalltalk access to HostCallDebugger which
 causes a core dump in customer executables and sleeps in non-customer
 executables."

^ self _zeroArgPrim: 25
%
category: 'Private'
classmethod: System
_ignoreModTracking

"Private. Disables Modification Tracking."

<primitive: 516>
self _primitiveFailed: #_ignoreModTracking
%
category: 'Version Management'
classmethod: System
_imageVersion

"Returns a String, which contains the history of the image."

^Globals at: #DbfHistory.
%
category: 'Private'
classmethod: System
_initMap: aMap

"Initializes the underlying data structures used to perform the mapping
 of passivated objects.  The argument aMap must be in the range 1 - 4."

<primitive: 319>
self _primitiveFailed: #_initMap:.
%
category: 'Private'
classmethod: System
_inMap: aMap at: anOop putIfAbsent: newValue

"If there is no current value at anOop in the passive object map aMap,
 (that is, it is absent), then this method stores newValue and returns newValue.
 If the current value at anOop is valid however, it returns the value and does
 not store the newValue in the map.  

 The argument aMap must be in the range 1 - 4
 and newValue must be a positive SmallInteger.

 anOop must not be a special object."

<primitive: 320>
aMap _validateClass: SmallInteger .
(aMap < 1 _or:[ aMap > 4]) ifTrue:[ aMap _halt:'argument out of range' ].
anOop isSpecial ifTrue:[ anOop _halt:'special object not allowed' ].
newValue _validateClass: SmallInteger .
newValue < 1 ifTrue:[ newValue _error: #rtErrArgNotPositive ].
self _primitiveFailed: #_inMap:at:putIfAbsent: .
%
category: 'Private'
classmethod: System
_internalMessageForErrorNum: aNumber 

"Private.  Returns a String describing the error message generated from the
 errmsg.c module for GemStone error numbered aNumber.  Returns nil if no such
 error number or message exists in errmsg.c."

<primitive: 384>
aNumber _validateClass: SmallInteger .
self _primitiveFailed: #_internalMessageForErrorNum:
%
category: 'Private'
classmethod: System
_intToOopFiltered: array serial: serialNum
"Take an array of oops represented as integers and replace the integers
 in the array with the corresponding objects.  Oops of objects that are
 dead or do not exist are replaced with nil.

 This method requires a session hold the garbage collection lock.  The
 serial number of that session must be specified in the serialNum argument.
 The garbage collection lock may be acquired with the  _lockGc method."

<primitive: 522>
self _primitiveFailed: #_intToOopFiltered:serial:.
self _uncontinuableError
%
category: 'Private'
classmethod: System
_intToOopFiltered: array serial: serialNum status: statusArray

"Same as _intToOopFiltered: array serial: serialNum,
 except additional information is returned in  statusArray.

 Convert an array of integers to objects using their OID.
 OIDs corresponding to dead or free oops will be replaced by a 
 nil in the array.  The serial number of the session holding the GC
 lock must be specified in serialNum or the primitive will fail.

 statusArray must be a 2 element array with both elements set to zero.
 Upon return, the first element is set to the number of oops which
 were dead, the second element refers to the number of oops
 which were free (unused)."

<primitive: 535>
self _primitiveFailed: #_intToOopFiltered:serial:status:.
self _uncontinuableError
%
category: 'Error Handling'
classmethod: System
_lastGsErrorNumber

"Return the value of the last error number.  Sets the error
 number to zero."
^self _zeroArgPrim: 42
%
category: 'Transaction Control'
classmethod: System
_localAbort

"Rolls back all modifications made to committed GemStone objects and provides
 the session with a new view of the most recently committed GemStone state.

 These operations are performed whether or not the session was previously in a
 transaction.  If the transaction mode is set to #autoBegin, then a new
 transaction is started.  If the transaction mode is set to #manualBegin, then
 a new transaction is not started."

self _pendingCommitAbort .  "Signal error to application if appropriate"

^ self _primitiveAbort "Abort the current transaction"
%
category: 'Transaction Control'
classmethod: System
_localBeginTransaction

"Starts a new transaction for the session."

self _pendingCommitAbort .  "Signal error to application if appropriate"
self _primitiveBegin "Begin a transaction"
%
category: 'Reduced Conflict Support'
classmethod: System
_localCommit: commitMode

"commitMode is a SmallInteger:

 0 for normal,
 1 for release locks,
 2 for checkpoint and release locks,
 3 for normal commit called from GciCommit,
 4 normal with finalizedIndexedNscs "

| commitResult actualMode |

self _processDeferredGciUpdates .

commitMode == 3 ifTrue:[
  "called from GciCommit, no pending commit action."
  actualMode := 0
  ]
ifFalse:[
  self _pendingCommitAbort .
  actualMode := commitMode
] .

commitResult := self __commit: actualMode.

commitResult == 0
  ifTrue: [ ^ true ]
  ifFalse: [
    commitResult == 1
      ifTrue: [ ^ self _resolveRcConflictsForCommit: actualMode ]
      ifFalse: [
        commitResult == 2 "retry for AllSymbols"
          ifTrue: [ ^ self _localCommit: actualMode]
          ifFalse: [ 
            commitResult == 3
              ifTrue: [^ false "validation failure"]
                   "3 means validation failure,
                    4 means maximum number of commit retries was reached,
                    5 means commit disallowed (usually because of indexes)"
            ]
        ]
    ].
^ false "if we get here it is a logic error"
%
category: 'Transaction Control'
classmethod: System
_localTransactionMode: newMode

"Sets a new transaction mode for the current GemStone session and exits the
 previous mode by aborting the current transaction.  Valid arguments are
 #autoBegin, #manualBegin and #transactionless."

self _pendingCommitAbort .
^ self _primTransactionMode: newMode
%
category: 'Setting Locks'
classmethod: System
_lock: anObject kind: lockKind

"Generalized routine to request a lock kind for a single object.  The lockKind
 argument is one of the three Symbols: #read, #write, or #exclusive.

 If the lock is granted, this method returns 1, if the lock operation
 fails due to the error #lockErrDenied it returns 0, and if
 the operation fails due to #lockErrObjHasChanged it returns 2."

<primitive: 97>
self _primitiveFailed: #_lock:kind: .
self _uncontinuableError
%
category: 'Setting Locks'
classmethod: System
_lockAll: aCollection kind: lockKind

"Generalized routine to request a lock kind for all the objects in aCollection.
 The lockKind argument is one of the three Symbols: #read, #write, or
 #exclusive.

 If you lack the necessary authorization for any object in aCollection,
 no locks are granted.

 If the lock operation is incomplete (that is, #lockErrIncomplete) you have
 locks for those objects that are not listed as arguments to #lockErrIncomplete.
 Otherwise, all the locks are successfully granted."

<primitive: 100>

| anArray |
anArray := aCollection asArray.
anArray == aCollection ifFalse:[
  ^ self _lockAll: anArray kind: lockKind
  ] .
aCollection _validateClass: Collection.
self _primitiveFailed: #_lockAll:kind: .
self _uncontinuableError
%
category: 'Setting Locks'
classmethod: System
_lockError: errNum obj: anObj args: argList

"Translate from SmallInteger to Symbol, for error reporting."
"Ugh."

(errNum == 2071) ifTrue: [^anObj _error: #lockErrUndefinedLock args: argList].
(errNum == 2073) ifTrue: [^anObj _error: #lockErrIncomplete args: argList].
(errNum == 2074) ifTrue: [^anObj _error: #lockErrObjHasChanged
                                  args: argList].
(errNum == 2075) ifTrue: [^anObj _error: #lockErrDenied args: argList].
"desperate, now"
^System signal: errNum args: argList signalDictionary: GemStoneError
%
category: 'Setting Locks'
classmethod: System
_lockEvaluateErr: errNum denied: denyBlock changed: changeBlock

"Private."

(errNum == 2074) ifTrue: " lock error changed "
   [^changeBlock value].
(errNum == 2075) ifTrue: " lock error denied "
   [^denyBlock value].
" at this point we have an unexpected error"
^System signal: errNum args: #[] signalDictionary: GemStoneError
%
category: 'Private'
classmethod: System
_lockGc: aBool
"Requests or releases the garbage collection lock from the stone. 
 If aBool is true, the GC lock is requested and the method will return
 true if the lock was granted or false if it was not.

 If aBool is  false, the lock is released if it was previously acquired by
 this session."

<primitive: 525>
self _primitiveFailed: #_lockGc:.
self _uncontinuableError
%
category: 'Performance Monitoring'
classmethod: System
_logStoneStatistics

"Write Stone process statistics to gemsys.log."

^ self _zeroArgCateg2Prim: 4
%
category: 'Private'
classmethod: System
_markNotConnectedForCollection

"Private."

"Explicitly triggers a mark for collection process on the objects in the
 notConnectedObjects set (temporary objects that have moved to disk).
 If the objects have not been committed, they are reclaimed immediately.
 If they are committed, they are recorded dead on the next commit."

^ self _zeroArgPrim: 19
%
category: 'Private'
classmethod: System
_messageForErrorNum: aNumber 

"Private.  Returns a String describing the error message produced for GemStone 
 error numbered aNumber in native language English, or nil if no such error
 number or language exists."

^ self _messageForErrorNum: aNumber inLanguage: #English
%
category: 'Private'
classmethod: System
_messageForErrorNum: aNumber inLanguage: aSymbol

"Private.  Returns a String describing the error message produced for GemStone 
 error numbered aNumber in native language aSymbol, or nil if no such error
 number or language exists."

 | errArray msgParts result |

 errArray := GemStoneError at: aSymbol otherwise: nil . 
 errArray == nil ifTrue:[ ^ nil ].
 (aNumber < 1 _or:[ aNumber > errArray size]) ifTrue:[ ^ nil ].
 msgParts := errArray at: aNumber .
 msgParts == nil ifTrue:[ ^ nil ].
 result := String new .
 1 to: msgParts size do:[:j | | element |
   element := msgParts at: j .
   element _isSmallInteger 
     ifTrue:[ result addAll: '<arg'; addAll: element asString; add: $> ] 
    ifFalse:[ result addAll: element ].
   ].
 ^ result 
%
category: 'Private'
classmethod: System
_needsCommit

"Answer true if persistent objects have been modified."

^ self _zeroArgPrim: 43
%
category: 'Private'
classmethod: System
_numGcSessions

"Returns the number of GC sessions logged in"

^ self _zeroArgPrim: 50
%
category: 'Private'
classmethod: System
_numNonGcSessions

"Returns the number of Non-GC sessions logged in"

^ self _zeroArgPrim: 51
%
category: 'Private'
classmethod: System
_numObjectTablePages

"Counts and returns the number of pages used by the object table."

^self _zeroArgPrim: 47
%
category: 'Private'
classmethod: System
_numSessions

"Returns the number of sessions logged in"

^ self _zeroArgPrim: 49
%
category: 'Performance Monitoring'
classmethod: System
_oopHighWaterMark

"Returns an Integer that gives the highest oop allocated in GemStone."


^self _zeroArgPrim: 23
%
category: 'Private'
classmethod: System
_parseClientNrs: aString

"aString should be the result of reading a request sent to a netldi
 on a socket. A session reading nrs requests from clients can tell
 that a full request has come in by reading until it gets a zero byte.
 This method raises the error #genericKernelError if something is
 wrong with the input.
 Otherwise it returns a 8 element Array with the following contents:
 1  Symbol describing type of client. It will be one of the following:
      #RpcApplication, #Gem, #PageServer, #Stone, #Unknown
 2  Boolean: true if client is trusted.
 3  String: name of user.
 4  Boolean: true if using kerberos.
 5  Symbol describing type of request. It will be one of the following:
      #invalid, #file, #monitor, #server, #task, #spawn, #cmd, #dbf
 6  String: working directory
 7  String: logfile name
 8  String: body of request
"

<primitive: 515>
self _primitiveFailed: #_parseClientNrs:
%
category: 'Transaction Control'
classmethod: System
_pendingCommitAbort

""

| commitAction |
commitAction := self _sessionStateAt: 7 .
commitAction ~~ nil ifTrue:[
  commitAction _isSmallInteger
    ifTrue:[
      "Signal commit-abort pending error"
      self signal: commitAction "ErrorSymbols at:#rtErrCommitAbortPending"
           args: #[]
           signalDictionary: GemStoneError.
      "expect the application to continue with GciContinue"
    ]
    ifFalse: [ self userAction: commitAction ]
].
%
category: 'Private'
classmethod: System
_prepareToCommit

"This is a placeholder for the implementation of voting in a two-phase commit
 protocol.  The current implementation uses 'hasConflicts' to return
 whether the current transaction could commit at this exact point in time;
 however, this does not guarantee that concurrent transactions do not commit
 changes that can cause later attempts to commit to fail.

 Note: See the comment for GsCurrentSession | hasConflicts to get a description of
 the side-effects of hasConflicts."

^ GsSession currentSession hasConflicts not
%
category: 'Private'
classmethod: System
_primitiveAbort

"Do not call this method directly; it is private to GemStone.  You must use
 abortTransaction.  Otherwise the transparent GemStone Smalltalk interface will
 break.

 This method rolls back all modifications made to committed GemStone objects
 (connected to the root) and provides the session with a new view of the most
 current committed state.  These operations are performed whether or not the
 session was previously in a transaction.  If the transactionMode is set to
 #autoBegin, then a new transaction is started.  If the transactionMode is set
 to #manualBegin or #transactionless, then a new transaction is not started, 
 but the session's view of the database is updated.
 This method always returns the receiver (System)."

<primitive: 335>
self _primitiveFailed: #_primitiveAbort .
self _uncontinuableError
%
category: 'Transaction Control'
classmethod: System
_primitiveBegin

"Starts a new transaction for the session.  If the session is already
 in a transaction, aborts the transaction and starts a new transaction.

 If any permanent objects had been written by the session, their
 state is aborted.  This method returns the receiver (System)."

<primitive: 362>
self _primitiveFailed: #primitiveBegin .
self _uncontinuableError
%
category: 'Private'
classmethod: System
_primitiveCommit: commitMode

"Do not call this method directly; it is private to GemStone.  You must use
 commitTransaction or commitAndReleaseLocks.  Otherwise Gem Builder breaks.

 The argument commitMode is a SmallInteger: 0 for normal, 1 for release locks,
 2 for checkpoint and release locks , 3 not used, 4 finalizedIndexedNscs

 The result of this primitive is a SmallInteger: 
       0 for success, 
       1 for rcFailure,
       2 for allSymbolsFailure, 
       3 for validationFailure,
       4 for retryFailure,
       5 for commitDisallowed."

<primitive: 336>
commitMode _validateClass: SmallInteger .

(commitMode < 0 _or:[ commitMode > 4 ])
  ifTrue:[ commitMode _error: #rtErrArgOutOfRange .  ^ 1 "failure" ] .

self _primitiveFailed: #_primitiveCommit: .
self _uncontinuableError
%
category: 'Transaction Control'
classmethod: System
_primTransactionMode: newMode

""

<primitive: 368>
self _primitiveFailed: #_primTransactionMode: .
self _uncontinuableError
%
category: 'Private'
classmethod: System
_processDeferredGciUpdates

"Process deferred GemBuilder for C updates and return the number of objects that
 had deferred updates."

"The deferred updates are stored in _sessionStateAt: 12.  These
 are updates to objects that do not allow direct structural update such
 as AbstractDictionary, Bag, and Set ."

<primitive: 901>  "enter protected mode to disable adding to dirtys set..."
| arr count |

arr := self _sessionStateAt: 13 "virtual machine constant".
(count := arr size) > 0 ifTrue:[
  1 to: count by: 2 do:[ :j |
    (arr at: j) _deferredGciUpdateWith: (arr at: j + 1).
    ].
  arr size: 0 .
  "reinitialize the identity dictionary"
  (self _sessionStateAt: 12 "virtual machine constant") 
      initialize: 751"a prime number for a large small object"
  ].
System _disableProtectedMode.
^ count // 2 
%
category: 'Private'
classmethod: System
_publish: server port: port log: logname options: optstr
  "Description:
     Publishes information about a server so that the gslist utility
     can display it. The information will be published on the local machine.
   Input:
     server <String>: name of server to publish.
     port <SmallInteger>: port number of server listening socket.
     logname <String or nil>: name of log file or nil if none.
     optstr <String or nil>: startup options or nil if none.
   Result <Boolean>:
     true if publish was successful.
     false if server was already published.
   Exceptions:
     Illegal input raises various errors.
     Error 2367 raised if low level C code fails with system error.
  "

<primitive: 496>
self _primitiveFailed: #_publish:port:log:options:
%
category: 'Reduced Conflict Support'
classmethod: System
_rcValueCache

"Returns the cache dictionary that is stored in temporary session state
 used to hold calculated values for reduced conflict classes. "

^ self _sessionStateAt: 5
%
category: 'Performance Monitoring'
classmethod: System
_readClock

"Returns an Integer indicating the amount of CPU time used by the current
 process, in units of milliseconds.  The resolution of the result is operating
 system dependent; typical resolution is 10 to 50 milliseconds."

^ self _zeroArgPrim: 10
%
category: 'Private'
classmethod: System
_reclaimAll

"Private."

"Explicitly triggers the reclamation of all shadowed and dead objects.
 Returns true if the reclaim process completed normally, otherwise returns
 false.  Successful completion requires this session be the only non-gc 
 session logged in, the right combination of GcGems be running, voting on 
 dead objects must be completed, and the GcUser configuration parameter
 reclaimDeadEnabled be true.  Valid GcGem combinations are: [none],
 [GcGem only], [RCL, EPC].

 This method aborts the current transaction; if an abort would cause unsaved
 changes to be lost, it signals an error, #rtErrAbortWouldLoseData.         

 This method requires the GarbageCollection privilege."

self needsCommit
  ifTrue: [ self _error: #rtErrAbortWouldLoseData ] .

^ self _zeroArgPrim: 17 
%
category: 'Private'
classmethod: System
_reclaimAllShadowPages

"Causes the stone to attempt to reclaim all shadow pages
 in the system.   All commit records are disposed and a 
 new checkpoint is written. Dead objects are not reclaimed.
 The current transaction is aborted.

 Requires the GarbageCollection privilege.

 This method requires no other sessions are present, and
 requires that voting on dead objects has completed .  Note that
 completion of voting requires a gc gem running.

 This method aborts the current transaction; if an abort would cause unsaved
 changes to be lost, it signals an error, #rtErrAbortWouldLoseData.         

 Returns true if  successful or false if some shadow pages could
 not be reclaimed."

self needsCommit
  ifTrue: [ self _error: #rtErrAbortWouldLoseData ] .

^ self _zeroArgPrim: 38 
%
category: 'Reduced Conflict Support'
classmethod: System
_redoLog

"Returns the redo log that is stored in the temporary session state."

^ self _sessionStateAt: 4
%

! deleted _reinitializeSharedCache

category: 'Private'
classmethod: System
_reloginAsUser: aUserId password: aPassword encrypted: aBoolean
       
"Relogin this session with the specified user identification."

<primitive: 495>
aUserId _validateClass: String.
aPassword _validateClass: String.
aBoolean _validateClass: Boolean.
self _primitiveFailed: #_reloginAsUser:password:encrypted:
%
category: 'Hidden Set Support'
classmethod: System
_remove: anObject from: hiddenSetSpecifier

"Removes anObject from the hiddenSet."

<primitive: 169>
hiddenSetSpecifier _validateClass: SmallInteger.
(hiddenSetSpecifier < 1 _or:
       [hiddenSetSpecifier > (self HiddenSetSpecifiers size)])
  ifTrue: [hiddenSetSpecifier _error: #rtErrArgOutOfRange].
self _primitiveFailed: #_remove:from:  .
self _uncontinuableError
%
category: 'Private'
classmethod: System
_removeLockWriteSubset

"Removes the locks acquired on any objects that were locked using
 _writeLockWriteSubset and empties the hidden set."

^self _zeroArgPrim: 35

%
category: 'Reduced Conflict Support'
classmethod: System
_resolveRcConflicts
        
"Checks for selected conflicts.  If some are found, attempt to resolve those
 conflicts.  If any conflicts could not be resolved, returns false."
  
|  scanArray redoObject commitFailed objsToRefresh conflicts |
              
scanArray := self _getRedoAndConflictObjects.
             
" if no redo objects were found, cannot resolve conflicts "
scanArray == nil ifTrue: [ self _disallowSubsequentCommits.
                            ^ false ].

commitFailed := false.
objsToRefresh := Array new.
1 to: scanArray size by: 2 do: [ :i |   
  redoObject := scanArray at: i.
  conflicts := scanArray at: i + 1.
              
  " keep list of objects to selectively abort in case of commit failure "
  redoObject _refreshAfterCommitFailure
    ifTrue: [ objsToRefresh add: redoObject ].
     
  commitFailed
    ifFalse: [
      (redoObject _resolveRcConflictsWith: conflicts)
        ifFalse: [
          System rcValueCacheAt: #'Rc-Write-Write' put: redoObject for: self.
          commitFailed := true.
        ]
    ]
].      

commitFailed
  ifTrue: [
    " selectively abort objects that might be logically inconsistent "
    1 to: objsToRefresh size do: [ :i |
      (objsToRefresh at: i) _selectiveAbort  
    ].       
    " force subsequent attempts to commit to fail with retry limit error"
    self _disallowSubsequentCommits.
    ^ false
  ].      

^ true
%
category: 'Reduced Conflict Support'
classmethod: System
_resolveRcConflictsForCommit: commitMode

""

| result |

self _processDeferredGciUpdates .

self _resolveRcConflicts
  ifFalse: [ self _disallowSubsequentCommits.
             ^ false ].

result := self __commit: commitMode.
result == 4
  ifTrue: [ self _error: #rtErrMaxCommitRetry]
  ifFalse: [ ^ (result == 0) ]
%
category: 'Session Control'
classmethod: System
_restartGcSession

"Restart the Garbage Collector Gem (GcGem) if the GcGem is not running, 
 and if (System configurationAt: #GcSessionEnabled) == 1.

 If the GcGem is already running, this method has no effect.

 The state of (System configurationAt: #GcSessionEnabled) is initialized from
 the Stone configuration parameter STN_GC_SESSION_ENABLED at Stone startup, and
 may be changed by the following message:

 System | configurationAt: #GcSessionEnabled put: <newValue>"

^ self _zeroArgPrim: 18
%
category: 'Private'
classmethod: System
_restoreModTracking

"Private. Enables Modification Tracking."

<primitive: 517>
self _primitiveFailed: #_RestoreModTracking
%
category: 'Signals'
classmethod: System
_sendSignal: aSignal toSess: sessionIdOrSerial withMessage: aString

"Sends a signal (a SmallInteger) to the specified session with aString as a
 message.  The aString argument is currently limited to 1023 bytes.

 The sessionIdOrSerial argument is a SmallInteger specifying a session.  A
 positive value is interpreted as a serialNumber.  A negative value is
 interpreted as a negated sessionId."

<primitive: 330>
aSignal _validateClass: SmallInteger.
sessionIdOrSerial _validateClass: SmallInteger.
aString _validateClass: String.
self _primitiveFailed: #_sendSignal:toSess:withMessage: .
self _uncontinuableError
%
category: 'Version Management'
classmethod: System
_serverVersionReport: isStone

"Private."

| result |
result := SymbolDictionary new .
VersionParameterDict keysAndValuesDo:[ :aName :anId | | aVal |
  aVal := self _configurationAt: anId isStone: isStone kind: $V .
  aVal ~~ nil ifTrue:[ 
    result at: aName put: aVal
    ].
  ].
^ result 
%
category: 'Transient Session State'
classmethod: System
_sessionCacheStatAt: anIndex 

"Returns the value of the session statistic at the specified index 
 (should be in the range 0 to 47)"

<primitive: 477>
anIndex _validateClass: SmallInteger.
self _primitiveFailed: #_sessionCacheStatAt: .
%
category: 'Transient Session State'
classmethod: System
_sessionCacheStatAt: anIndex put: i

"This method sets the session statistic at the specified index (which should be
 in the range 0 to 47) to the specified value i, which must be a positive
 SmallInteger."

<primitive: 476>
anIndex _validateClass: SmallInteger.
i _validateClass: SmallInteger.
self _primitiveFailed: #_sessionCacheStatAt:put: .
%
category: 'Transient Session State'
classmethod: System
_sessionCacheStatsForProcessSlot: aProcessSlot

"Return an array containing the 48 session statistics for the given
 process slot.  

 Returns nil if the given process slot is not found or
 if the slot is not in use by a gem process.

 See additional information in the method #_sessionCacheStatAt: put:"

^self _sessionCacheStatsForSlotOrSessionId: aProcessSlot
%
category: 'Transient Session State'
classmethod: System
_sessionCacheStatsForSessionId: aSessionId

"Return an array containing the 48 session statistics for the given
 session ID.  

 Returns nil if the given session is not found or if the slot is not
 in use by a gem process.

 See additional information in the method #_sessionCacheStatAt: put:"

^self _sessionCacheStatsForSlotOrSessionId: aSessionId negated.
%
category: 'Transient Session State'
classmethod: System
_sessionCacheStatsForSlotOrSessionId: aProcessSlotOrSessionId

"Return an array containing the 48 session statistics for the given
 session ID or process slot number.  If the argument is positive
 it is assumed to be a process slot.  If it is negative it is assumed
 to be a negated session ID. 

 Returns nil if the given session or cache slot is not found or
 if the slot is not in use by a gem process.

 See additional information in the method #_sessionCacheStatAt: put:"

<primitive: 543>
aProcessSlotOrSessionId _validateClass: SmallInteger.
self _error: #rtErrArgOutOfRange args: #[ aProcessSlotOrSessionId ].
^self _primitiveFailed: #_sessionCacheStatsForSlotOrSessionId: .
%
category: 'Transient Session State'
classmethod: System
_sessionStateAt: anIndex

"Returns an indexed instance variable of the Transient Session State object.
 This object is a temporary instance of Array created after each GciLogin.

 If the specified instance variable of the Session State has not been stored
 into during this session, the result is nil.  This is different from
 Array>>at: semantics, where accessing beyond the end of an Array
 generates an error.

 Users of this method must observe the following instance variable allocations
 for this Array.

    anIndex      assignedUse
    -------      -----------
    1            Reserved for SymbolDictionary of Temporaries
    2            Reserved for GemORB.
    3            Topaz session state
    4            Reduced Conflict classes logging info
    5            Cache for Reduced Conflict values
    6            The one instance of GsCurrentSession 
    7            Pending commit/abort errorNumber or userAction 
    8            Reserved for GemStone federation support
    9            Other known sessions (a dictionary keyed on sessionSerialNum)
    10           Reserved for GemStone relational support
    11           True if client wants GciAddSaveObjsToReadSet() to be performed
                 by a successful commit or abort, nil otherwise. Client
                 must initialize with _sessionStateAt:put:  .
    12           GemBuilder for C (GCI) deferred update Dictionary.
    13           GemBuilder for C (GCI) deferred update Array.
    14           Commit coordinator.
    15           Reserved for GemStone ProcessorScheduler support.
    16           Reserved for GemStone Locale support. 
    17           Reserved for timezone support.
    18           Reserved for GemStone GsFile support.
    19           Reserved for GemStone dictionary key canonicalization.
    20..         Available for customer use ."

<primitive: 111>

(anIndex _isSmallInteger)
  ifTrue: [self _errorIndexOutOfRange: anIndex .  ^ nil ]
  ifFalse: [self _errorNonIntegerIndex: anIndex .  ^ nil ].
self _primitiveFailed: #_sessionStateAt: .
self _uncontinuableError
%
category: 'Transient Session State'
classmethod: System
_sessionStateAt: anIndex put: aValue

"Modifies an indexed instance variable of the Transient Session State Array.
 The Session State Array will be grown as required to accommodate a store
 into the specified instance variable.

 See additional important documentation in the method #_sessionStateAt:"

<primitive: 372>
(anIndex _isSmallInteger)
  ifTrue: [self _errorIndexOutOfRange: anIndex]
  ifFalse: [self _errorNonIntegerIndex: anIndex].
self _primitiveFailed: #_sessionStateAt:put: .
self _uncontinuableError
%
category: 'Performance Monitoring'
classmethod: System
_sharedAttached

"This method returns the number of pages that are attached by this session
 and at least one other session.  It used to be reported in the cache
 statistics, but was removed because the computation was too costly
 for large numbers of users."

^self _zeroArgPrim: 24
%
category: 'Shared Counters'
classmethod: System
_sharedCounter: n incrementBy: i

"Shared counters provide a means for multiple sessions on the same host
 to share a common counter value (for creation of unique keys, etc.).
 The shared counters are implemented in the shared memory region used for
 the shared cache, so the SharedPageCache must be enabled for access to 
 the counters.  The SharedPageCache is created with 1900 shared counters.
 Operations on the counters are protected with a single spinlock/semaphore.

 This method increments one of the counters (in the range 0 to 1899) by the 
 specified value i and returns the result.  Results are masked to always
 be in the range 0 to MAX_SMALL_INTEGER."

<primitive: 354>
self _primitiveFailed: #_sharedCounter:incrementBy: .
self _uncontinuableError
%
category: 'Shared Counters'
classmethod: System
_sharedCounter: n setValue: i

"Shared counters provide a means for multiple sessions on the same host
 to share a common counter value (for creation of unique keys, etc.).
 The shared counters are implemented in the shared memory region used for
 the shared cache, so the SharedPageCache must be enabled for access to 
 the counters.  The SharedPageCache is created with 1900 shared counters.
 Operations on the counters are protected with a single spinlock/semaphore.

 This method initializes one of the counters (in the range 0 to 1899) to the 
 specified value i, which must be a positive SmallInteger."

<primitive: 350>
self _primitiveFailed: #_sharedCounter:value: .
self _uncontinuableError
%
category: 'Signals'
classmethod: System
_signalErrorStatus: aSignal

"Returns true if the error for the specified signal is enabled, false
 otherwise.  Signals are specified by SmallIntegers, the following are defined:

 1.  SignaledObjects.
 2.  SignaledAbort.
 3.  SignaledGemStoneSession."

<primitive: 366>
self _primitiveFailed: #_signalErrorStatus: .
self _uncontinuableError
%
category: 'Private'
classmethod: System
_signalFromGemStoneSession

"Returns a three-element Array containing information about a signal from
 another GemStone session:

 1.  The sessionSerialNumber of the session that sent the signal.
 2.  The signal value (a SmallInteger).
 3.  A signal message (a String).

 If there is no signal in the queue, returns nil."

<primitive: 369>
self _primitiveFailed: #signalFromGemStoneSession .
self _uncontinuableError
%
category: 'Error Handling'
classmethod: System
_signalGciError: anInteger args: anArray signalDictionary: anErrorDict

"This method generates the specified signal (or error), along
 with its associated arguments and returns control to GemBuilder for C."

"GciContinueWith() depends upon this implementation."

<primitive:907>

anInteger _validateClass: SmallInteger.
anInteger < 1 ifTrue: [ ^System _error: #rtErrBadErr
                 args: #[ anInteger, anArray, anErrorDict]].
anArray _validateClass: Array.
anArray size > 10 ifTrue: [ ^System _error: #rtErrTooManyErrArgs
                args: #[ anArray size]].

anErrorDict _validateClass: SymbolDictionary. 

self _primitiveFailed: #signal:args:signalDictionary: .
self _uncontinuableError
%
category: 'Backward Compatibility'
classmethod: System
_sleep: aTime

"Obsolete in GemStone 4.0."

^ self sleep: aTime
%
category: 'Session Control'
classmethod: System
_sleepMs: milliSecondsToSleep

"Sleep for the number of milliseconds specified by milliSecondsToSleep.
 The argument milliSecondsToSleep must be a positive SmallInteger.
 If milliSecondsToSleep is zero, this method has no effect.
 This method is not currently interruptible.
 Returns the receiver."

<primitive: 344>
milliSecondsToSleep _validateClass: SmallInteger.
milliSecondsToSleep < 0 
   ifTrue: [ milliSecondsToSleep _error: #rtErrArgOutOfRange ] .
^ self _primitiveFailed: #_sleepMs:
%
category: 'Garbage Collection'
classmethod: System
_startGcSession: gcType extentId: extentId

"Start up a GC session, according to gcType and extentId.
 Valid gcTypes:
    1:  Normal GcGem (not currently supported)
    2:  RCL (ReclaimGem)
    3:  EPC (EpochGem)
    4:  PSR (ParallelShadowReclaim)
    5:  PDR (ParallelDeadReclaim)
    7:  (Special code to force epoch GC to start)
Note that extentId is ignored for GcGem, RCL, and EPC sessions.  "

<primitive: 523>
gcType _validateClass: SmallInteger.
extentId _validateClass: SmallInteger.
self _primitiveFailed: #_startGcSession:extentId:.
self _uncontinuableError
%
category: 'Session Control'
classmethod: System
_stopSession: aSerialNumber 

"Aborts the transaction of the specified session (a SmallInteger), then
 terminates that session.  Returns the receiver.  If the indicated session is
 not active, no operation is performed.

 To execute this method, you must have SessionAccess and SystemControl
 privileges. "


<primitive: 332>
aSerialNumber _validateClass: SmallInteger.
^ self _primitiveFailed: #stopSession
%
category: 'Private'
classmethod: System
_systemLocksPrim

"For use in implementation of System(C) | systemLocks.  Returns an Array of
 Associations, with key sessionId."

<primitive: 353>

self _primitiveFailed: #_systemLocksPrim .
self _uncontinuableError
%
category: 'Hidden Set Support'
classmethod: System
_testIf: anObject isIn: hiddenSetSpecifier

"Returns true if anObject is in the hiddenSet, false otherwise."

<primitive: 170>
hiddenSetSpecifier _validateClass: SmallInteger.
(hiddenSetSpecifier < 1 _or:
       [hiddenSetSpecifier > (self HiddenSetSpecifiers size)])
  ifTrue: [hiddenSetSpecifier _error: #rtErrArgOutOfRange].
self _primitiveFailed: #_testif:isIn:.
self _uncontinuableError
%
category: 'Time'
classmethod: System
_timeGmt95: aBoolean

"Returns a SmallInteger or a LargePositiveInteger, the time since January 1, 
 1995, in seconds.  It will return a SmallInteger until January 6, 2012.
 aBoolean is true, call Stone and update our offset from Stone's clock,
 otherwise compute the current time locally using the offset from Stone's time
 that was cached in the session at login."

<primitive: 381>
^ self _primitiveFailed: #_timeGmt95:
%
category: 'Time'
classmethod: System
_timeGmt2005: aBoolean

"Returns a SmallInteger or a LargePositiveInteger, the time since January 1, 
 2005, in seconds.  It will return a SmallInteger until January 5, 2022.
 aBoolean is true, call Stone and update our offset from Stone's clock,
 otherwise compute the current time locally using the offset from Stone's time
 that was cached in the session at login."

<primitive: 561>
^ self _primitiveFailed: #_timeGmt2005:
%
category: 'Time'
classmethod: System
_timeGmt: aBoolean

"Returns a LargePositiveInteger, the time since January 1, 1970, in seconds.
 aBoolean is true, call Stone and update our offset from Stone's clock,
 otherwise compute the current time locally using the offset from Stone's time
 that was cached in the session at login."

<primitive: 371>
^ self _primitiveFailed: #_timeGmt:
%
category: 'Time'
classmethod: System
_timeGmtFloat

"Returns a Float representing the time since January 1, 1970 in units
 of seconds, to millisecond resolution, or such resolution as provided
 by the operating system's ftime() call.  This time is computed locally
 in the session process, using the offset from Stone's time that was
 cached in the session at login."

^ self _zeroArgPrim: 13 
%
category: 'Time'
classmethod: System
_timeMs

"Returns a SmallInteger representing the current relative time in milliseconds.
 The result is a SmallInteger between 0 and 524287999, equivalent to
    (System _timeGmtFloat * 1000) asInteger \\ 524288000

 The result is computed locally in the session process, using the offset 
 from Stone's time that was cached in the session at login."
 
^ self _zeroArgPrim: 12
%
category: 'Private'
classmethod: System
_uncommittedDataPages

"Returns an Array of the data pages referenced from the session's shadowed
 view of GemStone.  After the next commit or abort, the contents of this
 Array are invalid and may be incomplete.  That is, very shortly after the
 commit or abort operation, they may not show all data pages, because more pages
 may be added by the currently running methods.  However, it might be useful to
 check on modified objects (in particular, reclustering) before the transaction
 commits."

^ self _zeroArgCateg2Prim: 11
%
category: 'Private'
classmethod: System
_unpublish: server on: machine
  "Description:
     Unpublishes information about a server on the given machine.
   Input:
     server <String>: name of server to unpublish.
     machine <String or nil>: name of machine to unplubish on or nil if local.
   Result <Boolean>:
     true if unpublish was successful.
     false if server was not published.
   Exceptions:
     Illegal input raises various errors.
     Error 2367 raised if low level C code fails with system error.
  "

<primitive: 497>
self _primitiveFailed: #_unpublish:on:
%
category: 'Private'
classmethod: System
_updateNoRollback: anyObj add: aBoolean

"Add or remove anyObj from the no-rollback set.  If an object is in the
 no-rollback set, then it is treated as a temporary and does not get rolled
 back on aborts."

aBoolean ifTrue: [System _add: anyObj to: 2]
         ifFalse: [System _remove: anyObj from: 2].
^System
%
category: 'Notification'
classmethod: System
_updateNotifySet: aCollection add: aBoolean

"Update the notify set by adding or removing the elements of the anArray to or
 from the set.  If aBoolean is true, the elements are added, otherwise they are
 removed."

<primitive: 363>
| anArray |
anArray := aCollection asArray.
anArray == aCollection ifFalse:[
  ^ self _updateNotifySet: anArray add: aBoolean
  ] .
aCollection _validateClasses: #[ Array, IdentityBag ].
self _primitiveFailed: #_updateNotifySet:add: .
self _uncontinuableError
%
category: 'Signals'
classmethod: System
_updateSignalErrorStatus: aSignal toState: newState

"The argument newState must be a Boolean.  If it is true, this method enables
 the generation of the error for the specified signal, otherwise the error
 generation is disabled.  Signals are specified by SmallIntegers, and the
 following are defined:

 1.  SignaledObjects.
 2.  SignaledAbort.
 3.  SignaledGemStoneSession."

<primitive: 367>
self _primitiveFailed: #_updateSignalErrorStatus:toState .
self _uncontinuableError
%
category: 'Private'
classmethod: System
_validateTransaction

" Access the validation primitive"

^ self _zeroArgPrim: 32
%
category: 'Version Management'
classmethod: System
_version

"Returns a String describing the versions of the pieces of the running GemStone
 system."

^ 'GEMSTONE: ' + System _gemVersion + '; IMAGE: ' + System _imageVersion
%
category: 'Private'
classmethod: System
_versionParameterDict

"Returns a Dictionary of names for version information parameters.  The
 dictionary keys are Symbols.  Its values are SmallInteger parameter IDs."

| result cfgId cfgName |
result := SymbolKeyValueDictionary new .
cfgId := 0 .
[
  cfgName := self _configParameterName: cfgId kind: $V .
  cfgName == nil ifFalse:[
    result at: cfgName put: cfgId .
    cfgId := cfgId + 1 .
    ] .
  cfgName == nil 
  ] untilTrue .

^ result
%
category: 'Private'
classmethod: System
_writeLockWriteSubset

"Start with the write set and perform filtering of AllSymbol components, 
 non-committed objects, and objects already locked, putting the results 
 in a new hidden set (writeLockWriteSubset).  Next, attempt to acquire a 
 write lock on the objects in the hidden set.  I all locks are acquired 
 cleanly, return true.  If all locks acquired but some are dirty, return 
 an array of dirty locked objects.  If not all locks could be acquired, 
 return false (locks that were acquired, including dirty locks are available
 in the writeLockWriteSubset hidden set.  Any locks that are acquired are 
 also put in the CommitOrAbortReleaseLocksSet."

^self _zeroArgPrim: 34
%
category: 'Private'
classmethod: System
_writeTranlog: aString integers: anArray

"Write aString and anArray of Integers to the transaction log.
 This private method should only be used for debugging purposes when
 requested by GemStone Technical Support."

< primitive: 550>
aString _validateClass: String.
anArray _validateClass: Array.
self _primitiveFailed: #_writeTranlog:integers:
%
category: 'Private'
classmethod: System
_writeTranlog: aString oops: anArray

"Write aString and anArray of objects to the transaction log.
 This private method should only be used for debugging purposes when
 requested by GemStone Technical Support."

< primitive: 549>
aString _validateClass: String.
anArray _validateClass: Array.
self _primitiveFailed: #_writeTranlog:oops:
%
category: 'Private'
classmethod: System
_zeroArgCateg2Prim: opcode

"Private."

"opcode 0 = activeRepositories
        1 = gemStatistics
        2 = stoneStatistics
        3 = gemVersion
        4 = logStoneStatistics
        5 = clearStoneStatistics
        6 = clusterAllSymbols
        7 = userActionReport - returns an Array of SymbolAssociations
        8 = configConstantsArray
        9 = systemUserActionReport
       10 = _committedDataPages
       11 = _uncommittedDataPages"

<primitive: 325> 

self _primitiveFailed: #_zeroArgCateg2Prim: .
self _uncontinuableError
%
category: 'Private'
classmethod: System
_zeroArgPrim: opcode

"Private."

"opcode 0 = removeLocksForSession
        1 = stackLimit
        2 = pageReads
        3 = pageWrites 
        4 = transactionMode   (also sent by GsCurrentSession)
        5 = inTransaction   (also sent by GsCurrentSession)
        6 = maxSessionId
        7 = myCacheProcessSlot
        8 = shutDown
        9 = continueTransaction
       10 = _readClock   (get CPU time used)
       11 = stoneName
       12 = _timeMs 
       13 = _timeGmtFloat
       14 = _generationScavenge
       15 = _enableTraceObjs
       16 = _disableTraceObjs
       17 = _reclaimAll
       18 = restart GcGem
       19 = _markNotConnectedForCollection
       20 = clientIsRemote
       21 = sessionsReferencingOldestCr
       22 = _failSubsequentCommits
       23 = _oopHighWaterMark
       24 = _sharedAttached
       25 = _hostCallDebugger
       26 = stackDepth
       27 = stackDepthHighwater
       28 = clearAttachedPages
       29 = _flushConnectedToPom
       30 = _commitResult
       31 = _deadNotReclaimed
       32 = _validateTransaction
       33 = session
       34 = _writeLockWriteSubset
       35 = _removeLockWriteSubset
       36 = UNUSED
       37 = _exclusiveLockWriteSubset
       38 = _reclaimAllShadowPages
       39 = _clearDeadObjs
       40 = UNUSED
       41 = UNUSED
       42 = _lastGsErrorNumber
       43 = _needsCommit
       44 = _dumpFrameData
       45 = _gcSessions
       46 = _highestAllocatedOop
       47 = _numObjectTablePages
       48 = _backupInit (not used)
       49 = _numSessions
       50 = _numGcSessions
       51 = _numNonGcSessions
       52 = _flushAllExtents
       53 = startFreeListPageServer
       54 = currentTransctionHasWWConflicts
       55 = currentTransctionWWConflicts
       56 = _locale
       57 = timeNs
       58 = _simpleCommit
       59 = _getGcStats
       60 = _checkpointSync
       61 = logout
       62 = inContinueTransaction
       63 = gemIsBigEndian
       64 = stoneIsBigEndian
       65 = disableTerminationSignals
       66 = _oopNumberHighWaterMark
       67 = voteState ( gets GC voteState from stone)
      100 = myPageServerProcessId
      101 = myReadPageServerProcessId
      102 = terminateAllSessionsReferencingOldestCr
      103 = writePageDebugDataToTranlog
      104 = sessionIsRemote
      105 = sessionIsUNIX
"

<primitive: 98>

self _primitiveFailed: #_zeroArgPrim: .
self _uncontinuableError
%
category: 'Reduced Conflict Support'
classmethod: System
__commit: commitMode

"This method is provided to facilitate reimplementation of commit
 in Gem Builder."

^ self _primitiveCommit: commitMode.
%
category: 'Transaction Control'
classmethod: System
currentTransactionHasWWConflicts

"Returns a Boolean indicating if the current transaction has one or
 more write-write conflicts.  A result of true indicates a commit
 will most likely fail.  However some RC objects have conflict
 resolution mechanisms which could allow a commit to suceed.
 A result of false indicates no write-write conflicts exist."

^ self _zeroArgPrim: 54.
%
category: 'Transaction Control'
classmethod: System
currentTransactionWWConflicts

"Returns an Array of objects which have write-write conflicts.
 The array is created by building the write set union of all
 commit records created since the session's current transaction 
 and intersecting it with the session's write set."

^ self _zeroArgPrim: 55.
%

category: 'Private'
classmethod: System
_deferCanonicalization: dict oldKey: oldKey newKey: newKey

" Record information on dictionaries needing to 
  have keys rehashed for symbol canonicalization "

<protected>

| pending |
pending := System _sessionStateAt: 19.
pending == nil ifTrue: [ 
    pending := Array new.
    System _sessionStateAt: 19 put: pending ].
pending add: ( Array with: dict with: oldKey with: newKey )
%

category: 'Private'
classmethod: System
_finalizeCanonicalization

" Process pending requests for dictionary key canonicalization "

<protected>

| pending |
pending := System _sessionStateAt: 19.
pending == nil ifFalse: [
    pending do: [ :entry | | dict oldKey newKey ok value |
        dict := entry at: 1.
        oldKey := entry at: 2.
        newKey := entry at: 3.
        ok := true.
        value := dict removeKey: oldKey ifAbsent: [ ok := false. ].
        ok ifTrue: [ dict at: newKey put: value ] ] ].
System _sessionStateAt: 19 put: nil
%

category: 'Private'
classmethod: System
_locale

"Returns a Locale object setup according to the session's Locale configuration.

Applications should not call this directly -- use 'Locale current' instead. "

^ self _zeroArgPrim: 56.
%

! fix 36394
! fix 37843: modify category argument to be categorySym
category: 'Private'
classmethod: System
_setCategory: categorySym locale: localeStr

" Set Locale information.  See man page for setlocale( ) for details.

Valid category types include:

   LC_CTYPE        /* locale's ctype handline */
   LC_NUMERIC      /* locale's decimal handling */
   LC_TIME         /* locale's time handling */
   LC_COLLATE      /* locale's collation data */
   LC_MONETARY     /* locale's monetary handling */
   LC_MESSAGES     /* locale's messages handling.  Note: not supported on Windows */
   LC_ALL          /* name of locale's category name */

Applications should not call this directly -- instead use:

  Locale setCategory: <sym> locale: <str>' 

"

< primitive: 563>
categorySym _validateClass: Symbol.
localeStr _validateClass: String.
self _primitiveFailed: #_setCategory:locale:
%

category: 'Time'
classmethod: System
timeNs

"Returns a LargePositiveInteger representing the current high-resolution
 real time in nanoseconds since some arbitrary time in the past.  The 
 result is not correlated in any way the time of day."
 
^ self _zeroArgPrim: 57
%

category: 'Session Control'
classmethod: System
terminateSession: aSessionId 

"Attempts to shutdown the given session and reclaim its resources as quickly
 as possible.  Unlike the stopSession: method, this method attempts to 
 terminate the session immediately with no grace period to allow the session to
 logout.  However some delay is still possible depending on the state of the 
 session when this method is executed.  Remote sessions must wait for their 
 pageserver process to terminate before they can be completely shutdown.

 A aSessionId is expected to be a SmallInteger.

 A message indicating the session which was terminated and by whom is written
 to the stone log.
 
 Returns false if the session could not be terminated, usually because it no 
 longer exists.  Otherwise returns true.

 To execute this method, you must have SessionAccess and SystemControl
 privileges. "

<primitive: 565>
aSessionId _validateClass: SmallInteger.
^ self _primitiveFailed: #terminateSession
%
category: 'Private'
classmethod: System
_simpleCommit

"Requests the stone perform a simple commit which creates or modifies
 no objects.

 Returns true if successful, false if some other session is currently
 committing.  Requires the Garbage Collection privilege."

^self _zeroArgPrim: 58
%
category: 'Private'
classmethod: System
_getGcStats

"Returns an Array of 4 Integer elements as follows:
  1 - number of commit records.
  2 - number of dead not reclaimed objects.
  3 - number of scavengable pages available to be reclaimed.
  4 - number of scavengable pages in outstanding commit records."

^self _zeroArgPrim: 59
%
category: 'Private'
classmethod: System
_checkpointSync

"Requests the stone perform a synchronous checkpoint if no checkpoint
 is currently in progress and no gem is currently committing a 
 transaction.  

 Returns true if the checkpoint was completed successfully, false if
 the checkpoint could not be performed because any of the above 
 conditions are not satisfied.

 Requires the Garbage Collection privilege."

^self _zeroArgPrim: 60
%

! Request 36492
category: 'Session Control'
classmethod: System
logout

"Immediately logs out the current session and generates a fatal error
 (GS_ERR_GEM_NORMAL_SHUTDOWN).  This method does not return."

self _zeroArgPrim: 61
%

category: 'Private'
classmethod: System
myPageServerProcessId
"Answer an Integer which is the process ID of the page server
on stone's machine for this session.
Returns 0 if the session does not have a page server."

^ self _zeroArgPrim: 100
%

category: 'Private'
classmethod: System
myReadPageServerProcessId
"Answer an Integer which is the process ID of the page server
being used for disk reads.
Returns 0 if the session does not have a page server.
Result may be different from the result of myPageServerProcessId if
the session is using a mid-level cache."

^ self _zeroArgPrim: 101
%

category: 'Shared Cache Management'
classmethod:
_remoteCachesListStartingAt: anOffset midLevelOnly: aBoolean

"Return a portion of the result for remoteCachesList,
 starting at anOffset in stone's list.    anOffset is zero-based.

 Result size is zero if anOffset is beyond end of stone's list,
 otherwise last element of result is a SmallInteger, the nextEnumIdx.
 If nextEnumIdx is < 0 there are no more elements in stone's list,
 otherwise pass nextEnumIdx as anOffset argument in next call to this
 primitive to continue the enumeration.

 Result size is limited by size of gem to stone communication buffers."

<primitive: 567>
anOffset _validateClass: SmallInteger .
anOffset < 0 ifTrue:[ anOffset _error: #rtErrArgOutOfRange ].
aBoolean _validateClass: Boolean .
^ self _primitiveFailed: #_remoteCachesListStartingAt:midLevelOnly:
%

category: 'Shared Cache Management'
classmethod: System
_remoteCachesList: forMidLevel

"Return an Array describing shared caches, per remoteCachesList."

| idx result batch |
idx := 0 .
result := Array new .
[ idx >= 0 ] whileTrue:[ | nextIdx batSize |
  batch := self _remoteCachesListStartingAt: idx midLevelOnly: forMidLevel .
  batSize := batch size .
  batSize > 0 ifTrue:[
    nextIdx := batch at: batSize .
    batch size: batSize - 1.
    result addAll: batch .
    idx := nextIdx .
  ] ifFalse:[
    idx := -1 . "done"
  ].
].
^ result
%

category: 'Shared Cache Management'
classmethod: System
_formatCacheReport: anArray

"given an Array produced by remoteCachesList or midLevelCachesList
 return a String containing one line per element of the list."

| rpt buf LF formatBlk header |
rpt := String new .
buf := String new: 50 .
LF := Character lf .

formatBlk := [:aCache :aRpt :aBuf | |kind kindStr |
  aBuf size: 0 ; addAll: ((aCache at: 1) width: 30) . "hostname"
  aRpt addAll: aBuf .
  kind := aCache at: 2 .
  kind == true ifTrue:[ 
    kindStr := 'mid'
  ] ifFalse:[ 
    kind == false ifTrue:[ kindStr := 'remote' ]
                 ifFalse:[ kindStr := 'kind' . "header line"]
  ].
  kindStr := String withAll: kindStr .
  aBuf size: 0 ; addAll: (kindStr width: 10)  . "kind"
  aRpt addAll: aBuf .
  aBuf size: 0 ; addAll: ((aCache at: 3) asString width: -11 ). "sessions"
  aRpt addAll: aBuf .
  aBuf size: 0 ; addAll: ((aCache at: 4) asString width: -10 ). "maxsessions"
  aRpt addAll: aBuf .
  aBuf size: 0 ; addAll: ((aCache at: 5) asString width: -10 ). "sizeKb"
  aRpt addAll: aBuf ; add: LF .
].  
  
header := #[ 'hostname', 'kind' , 'sessions', 'maxSess', 'sizeKB'].
1 to: header size do:[:k | header at:k put:(String withAll:(header at:k)) ].

formatBlk value: header value: rpt value: buf  . "header line"

1 to: anArray size do:[:k || aCache |
  aCache := anArray at: k .
  formatBlk value: aCache value: rpt value: buf .
].
^ rpt
%

category: 'Shared Cache Management'
classmethod: System
remoteCachesList

"Return an Array describing all shared caches that the
 stone process is managing, not including the cache on
 the stone machine.

 Result contains one element per cache. Each
 element of result is a 4 element Array containing  
   hostName, 
   a Boolean - true if cache was created as a mid-level cache
   a SmallInteger - number of sessions connected to the cache  
   a SmallInteger - max number of sessions for the cache  
   a SmallInteger - size of cache in KB "

^ self _remoteCachesList: false
%

category: 'Shared Cache Management'
classmethod: System
midLevelCachesList

"Return an Array as described for remoteCachesList, 
 filtered to include only the mid-level caches. "

^ self _remoteCachesList: true
%

category: 'Shared Cache Management'
classmethod: System
remoteCachesReport

"Return the result of remoteCachesList formatted as a string,
 one line per cache."

^ self _formatCacheReport: self remoteCachesList

%

category: 'Shared Cache Management'
classmethod: System
midLevelCachesReport

"Return the result of midLevelCachesList formatted as a string,
 one line per cache."

^ self _formatCacheReport: self midLevelCachesList
%

category: 'Shared Cache Management'
classmethod: System
midLevelCacheConnect: hostName

"Attempt to connect to mid-level cache on the
 specified host, if cache already exists.

 Returns true if connection succeeded, or a String
 describing the failure.

 See  midLevelCacheConnect:cacheSizeKB:maxSessions: for more details."

^ self midLevelCacheConnect: hostName cacheSizeKB:-1  maxSessions:-1
%

category: 'Shared Cache Management'
classmethod: System
midLevelCacheConnect: hostName cacheSizeKB: aSize maxSessions: nSess 

"Returns true if connection succeeded, or a String
 describing the failure.

 If a mid-level cache does not exist on a specified host,
 and aSize > 0, attempt to start one.
 Then attempt to connect to the mid-level cache.
 Connection may fail if cache could not be started or
 if the cache could not accept another session .

 The session's gem process must be on a machine different from
 the machine running the stone process in order for this to succeed.

 If session A's gem process is running on a machine where the   
 shared cache is in use as a mid-level cache by gem processes
 on other machines, then session A is cannot use this method;
 i.e. cross-connected mid-level caches are not allowed .

 hostName must be a String, else an error is generated.

 If aSize <= 0, nSess is ignored , 
 otherwise nSess must be >= 1 and aSize must be >= 4096 .
 The nSess value may be reduced by stone to the
 number of sessions stone is configured and licensed for. 

 When creating a cache
   aSize is cache size in K bytes ,
   nSess is maximum number of sessions ,

 The spinLockCount, targetFreeFrameCount, numFreeFrameServers,
 and numSharedCounters for creating a cache come from gem's config file
 just as for a cache the gem would have created as a side effect
 of login .

 Use of mid-level caches and replicates of extents are mutually exclusive;
 if replicates exist the method generates an error.
 For this method to succeed, DBF_REPLICATE_NAMES in the stone config
 file must be empty, and  Repository>>createReplicateOf:named:
 must not have been used since stone was started."
  
<primitive: 566 >
hostName  _validateClass: String .
aSize _validateClass: SmallInteger .
aSize > 0 ifTrue:[
  nSess _validateClass: SmallInteger .
  nSess < 1 ifTrue:[ nSess _error: #rtErrArgOutOfRange ].
  nSess > 10000 ifTrue:[ nSess _error: #rtErrArgOutOfRange ].
].
hostName size > 2000 ifTrue:[ hostName _error: #rtErrArgOutOfRange].
^ self _primitiveFailed: #midLevelCacheConnect:
%

category: 'Session Control'
classmethod: System
disableTerminationSignals

"Notifies the stone that this session may not be terminated if it should 
 become unresponsive to a shutdown or abort request.

 By default on UNIX-based systems, sessions that do not shutdown or abort within
 a time period may be terminated by sending the session a SIGTERM signal.  The 
 SIGTERM signal is handled by the process and causes it to immediately shutdown
 in a safe manner.

 Sessions that execute this method will not be sent a shutdown signal and may
 linger indefinitely after the stone logs it out.

 This method should normally only be called by processes which support more than one
 concurrent session.  In that case, if any one session becomes unresponsive,
 the process running all the sessions may be terminated.

 To execute this method, you must have the SessionAccess privilege."

^self _zeroArgPrim: 65
%

category: 'Debugging Support'
classmethod: System
_oopNumberHighWaterMark

"Indicates the higest oop number allocated in the system.  An oop number has a 
 corresponding oop.  Disk Oops are spaced apart by 4 (1, 5, 9...) while oop
 numbers are contiguous (1,2,3,4...)."

^self _zeroArgPrim: 66
%

category: 'Garbage Collection Management'
classmethod: System
voteState

"Returns an integer, the voteState of the Stone garbage
 collection voting state machine.
 The states are  0 IDLE, 1 VOTING,
         2 DONE_VOTING, 3  IN_PDWSU_SWEEP, 4 PDWSU_DONE"

 ^ self _zeroArgPrim: 67
%

category: 'Debugging Support'
classmethod: System
_printCacheStructureWithIndex: index opCode: opCode

"Prints the contents of a shared cache structure with the given index to
 stdout of the process.  Both arguments must be instances of SmallInteger.

 The type of cache structure printed is determined by the value of opCode
 as follows:

 0 - contents of cache frame 
 1 - contents of frame lock
 2 - contents of hash table lock
 3 - contents of PCE (page cache entry)

 Returns true on success."
 
<primitive: 570 >
index _validateClass: SmallInteger.
opCode _validateClass: SmallInteger.
((opCode < 0)  _or:[opCode > 3])
  ifTrue:[ opCode _error: #rtErrArgOutOfRange ]
  ifFalse:[ index _error: #rtErrArgOutOfRange ].
^ self _primitiveFailed: #_printCacheStructureWithIndex:opCode: .
%

category: 'Extreme Measures'
classmethod: System
terminateAllSessionsReferencingOldestCr

"Causes the stone to take the following actions:
 1 - immediately log off all sessions which reference the oldest commit
     record, except for the current session.
 2 - dispose the oldest commit record and all other commit records 
     eligible for disposal.

 Does not terminate the current session, even if it references the
 oldest commit record.  Has no effect if there is only one commit
 record present.

 WARNING: This method bypasses certain internal mechanims which
          prevent the immediate termination of active sessions,
          and therefore should only be used if the oldest CR session(s)
          could not be stopped by any other mechanisms.  Usage of
          this method when one or more sessions referencing the oldest
          CR are active may cause cache coherency errors.

 Requires the SystemControl privilege.

 Returns a SmallInteger indicating the number of commit records 
 disposed."

^ self _zeroArgPrim: 102
%

category: 'Debugging Support'
classmethod: System
writePageDebugDataToTranlog

"This method should only be run if requested by GemStone Technical Support.

 Causes the stone process to write a list of pages allocated to each session
 to the tranlog for analysis by GemStone Systems personnel.

 Warning: Executing this method on a busy production system may cause the stone
          to become unresponsive for several seconds.

 This method may only be executed by SystemUser.  Returns the receiver."

^ System _zeroArgPrim: 103
%

category: 'Symbol Management'
classmethod: System
rebuildAllSymbols: tableSize createAllBuckets: aBoolean

"Rebuilds the dictionary AllSymbols.  The new table size will be a prime
 number greater than tableSize.   For best performance, the table size
 should be no larger than the prime 2044201.

 If aBoolean is true, then all possible collision buckets are also 
 created.  The number of buckets created will equal the tableSize
  argument.

 If aBoolean is false, collision buckets are only created as needed.

 Repositories which are to be operated with the GcGem's AllSymbols
 bitmap rebuild operation disabled should set this value to true.  
 See the description of the #allSymbolsMinCommitsBetweenBmRebuilds 
 option in $GEMSTONE/upgrade/gcuser.gs for information on how to
 disable rebuilding the AllSymbols bitmap.

 AllSymbols should be reclustered using the #clusterAllSymbols method after
 the transaction executing this method has been committed."

| newSize allSyms |
newSize := Integer _selectedPrimeGreaterThan: tableSize .
allSyms := Globals at: #AllSymbols .
allSyms _rebuildTable: newSize createAllBuckets: aBoolean .
^ self
%

category: 'Hidden Set Support'
classmethod: System
_primHiddenSetEnumerate: hiddenSetSpecifier limit: maxResultSize storeAsInts: aBool

"Returns an Array containing the first maxResultSize elements of the
 hiddenSet.  If the hidden set contains fewer than maxResultSize
 elements, returns an array containing the contents of the hiddenSet.
 If maxResultSize = 0, the result will contain all elements of the
 hidden set.

 All of the returned elements are removed from the hidden set.

 If aBool is true, then the object ID's of the hidden set are stored in
 the result array as SmallIntegers.  Otherwise the objects themselves 
 are stored in the result array."

<primitive: 579>
| sz |
hiddenSetSpecifier _validateClass: SmallInteger.
aBool _validateClass: Boolean.
(hiddenSetSpecifier < 1 or:[
  hiddenSetSpecifier > (sz := self HiddenSetSpecifiers size)])
  ifTrue: [hiddenSetSpecifier _error: #rtErrArgOutOfRange args: #[ 1,  sz ] ].
maxResultSize _validateClass: SmallInteger.
maxResultSize < 0 ifTrue:[ maxResultSize _error: #errArgTooSmall args: #[ 0 ] ].
self _primitiveFailed: #_primHiddenSetEnumerate:limit:storeInts: .
self _uncontinuableError
%

category: 'Hidden Set Support'
classmethod: System
_hiddenSetEnumerate: hiddenSetSpecifier limit: maxResultSize

"Returns an Array containing the first maxResultSize objects
 in the hiddenSet.  If the hidden set contains fewer than
 maxResultSize elements, returns an array containing the contents
 of the hiddenSet.  If maxResultSize = 0, the result will contain
 all elements of the hidden set.

 All of the returned elements are removed from the hidden set."

^self _primHiddenSetEnumerate: hiddenSetSpecifier
      limit: maxResultSize 
      storeAsInts: false.
%

category: 'Hidden Set Support'
classmethod: System
_hiddenSetEnumerateAsInts: hiddenSetSpecifier limit: maxResultSize

"Returns an Array containing the numeric object identifiers of the 
 first maxResultSize objects in the hiddenSet.  If the hidden set 
 contains fewer than maxResultSize elements, returns an array containing
 the contents of the hiddenSet.  If maxResultSize = 0, the result will
 contain all elements of the hidden set.

 All of the returned elements are removed from the hidden set."

^self _primHiddenSetEnumerate: hiddenSetSpecifier
      limit: maxResultSize 
      storeAsInts: true.
%

category: 'Session Control'
classmethod: System
sessionIsRemote

"Answers true if this session is running on a different host
 than the stone.  Otherwise answers false."

^ self _zeroArgPrim: 104
%

category: 'Session Control'
classmethod: System
sessionIsLocal

"Answers true if this session is running on the same host
 as the stone.  Otherwise answers false."

^ self sessionIsRemote == false
%

category: 'Session Control'
classmethod: System
sessionIsUNIX

"Answers true if this host operating system for this session is a type
 of UNIX.  Otherwise answers false."

^ self _zeroArgPrim: 105
%
